@@ -2,6 +2,7 @@
  * libopensync - A synchronization framework
  * Copyright (C) 2006  Armin Bauer <armin.bauer@opensync.org>
  * Copyright (C) 2006  NetNix Finland Ltd <netnix@netnix.fi>
+ * Copyright (C) 2008  Michael Bell <michael.bell@opensync.org>
  * 
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -25,40 +26,22 @@
 #include "opensync.h"
 #include "opensync_internals.h"
 
-#include "opensync-archive.h"
+#include "opensync_archive_private.h"
 #include "opensync_archive_internals.h"
-#include "opensync-db.h"
-
-/**
- * @defgroup OSyncArchivePrivateAPI OpenSync Archive Internals
- * @ingroup OSyncPrivate
- * @brief The private part of the OSyncArchive API
- * 
- */
-/*@{*/
-
-/*@}*/
-
-/**
- * @defgroup OSyncArchiveAPI OpenSync Archive
- * @ingroup OSyncPublic
- * @brief Functions for manipulating the archive
- * 
- * The archive is a persistent store of all of the objects seen/handled 
- * in a sync group. It is necessary for the merger code to work.
- */
-/*@{*/
+#include "db/opensync_db_internals.h"
 
 static osync_bool osync_archive_create_changes(OSyncDB *db, const char *objtype, OSyncError **error)
 {
+	int ret = 0;
+	const char *query = NULL;
+
 	osync_trace(TRACE_ENTRY, "%s(%p, %s, %p)", __func__, db, objtype, error); 
 
 	osync_assert(db);
 	osync_assert(objtype);
+	osync_assert(strlen(objtype) <= 64);
 
-	char *tbl_changes = g_strdup_printf("tbl_changes_%s", objtype);
-	int ret = osync_db_table_exists(db, tbl_changes, error);
-	g_free(tbl_changes);
+	ret = osync_db_table_exists(db, "tbl_changes", error);
 
 	/* error if ret -1 */
 	if (ret < 0)
@@ -70,32 +53,36 @@ static osync_bool osync_archive_create_changes(OSyncDB *db, const char *objtype,
 		return TRUE;
 	}
 
-	char *query = g_strdup_printf("CREATE TABLE tbl_changes_%s (id INTEGER PRIMARY KEY, uid VARCHAR, memberid INTEGER, mappingid INTEGER)", objtype);
+	/* The primary key is conceptually (objtype, id).  SQLite3 will only AUTOINCREMENT
+	 * a single integer primary key so the primary key restricted to being id.  To
+	 * enforce the entity-relationship model all queries for a single item should be:
+	 * WHERE objtype='%s' AND id=%lli
+	 */ 
+	query = "CREATE TABLE tbl_changes (objtype VARCHAR(64) NOT NULL, id INTEGER PRIMARY KEY AUTOINCREMENT, uid VARCHAR NOT NULL, memberid INTEGER NOT NULL, mappingid INTEGER NOT NULL, objengine VARCHAR(64) NOT NULL )";
 	if (!osync_db_query(db, query, error)) {
-		g_free(query);
 		goto error;
 	}
 
-	g_free(query);
-
 	osync_trace(TRACE_EXIT, "%s: created table.", __func__);
 	return TRUE;
 
-error:	
+ error:	
 	osync_trace(TRACE_EXIT_ERROR, "%s: %s", __func__, osync_error_print(error));
 	return FALSE;
 }
 
 static osync_bool osync_archive_create_changelog(OSyncDB *db, const char *objtype, OSyncError **error)
 {
+	int ret = 0;
+	const char *query = NULL;
+
 	osync_trace(TRACE_ENTRY, "%s(%p, %s, %p)", __func__, db, objtype, error); 
 
 	osync_assert(db);
 	osync_assert(objtype);
+	osync_assert(strlen(objtype) <= 64);
 
-	char *tbl_changelog = g_strdup_printf("tbl_changelog_%s", objtype);
-	int ret = osync_db_table_exists(db, tbl_changelog, error);
-	g_free(tbl_changelog);
+	ret = osync_db_table_exists(db, "tbl_changelog", error);
 
 	/* error if ret -1 */
 	if (ret < 0)
@@ -107,31 +94,31 @@ static osync_bool osync_archive_create_changelog(OSyncDB *db, const char *objtyp
 		return TRUE;
 	}
 
-	char *query = g_strdup_printf("CREATE TABLE tbl_changelog_%s (id INTEGER PRIMARY KEY, entryid INTEGER, changetype INTEGER)", objtype);
+	query = "CREATE TABLE tbl_changelog (objtype VARCHAR(64), id INTEGER, entryid INTEGER, changetype INTEGER, PRIMARY KEY (objtype, id) )";
 	if (!osync_db_query(db, query, error)) {
-		g_free(query);
 		goto error;
 	}
-	g_free(query);
 
 	osync_trace(TRACE_EXIT, "%s: created table.", __func__);
 	return TRUE;
 
-error:	
+ error:	
 	osync_trace(TRACE_EXIT_ERROR, "%s: %s", __func__, osync_error_print(error));
 	return FALSE;
 }
 
 static osync_bool osync_archive_create(OSyncDB *db, const char *objtype, OSyncError **error)
 {
+	int ret = 0;
+	const char *query = NULL;
+
 	osync_trace(TRACE_ENTRY, "%s(%p, %s, %p)", __func__, db, objtype, error); 
 
 	osync_assert(db);
 	osync_assert(objtype);
+	osync_assert(strlen(objtype) <= 64);
 
-	char *tbl_archive = g_strdup_printf("tbl_archive_%s", objtype);
-	int ret = osync_db_table_exists(db, tbl_archive, error);
-	g_free(tbl_archive);
+	ret = osync_db_table_exists(db, "tbl_archive", error);
 
 	/* error if ret -1 */
 	if (ret < 0)
@@ -144,33 +131,26 @@ static osync_bool osync_archive_create(OSyncDB *db, const char *objtype, OSyncEr
 	}
 
 
-	char *query = g_strdup_printf("CREATE TABLE tbl_archive_%s (mappingid INTEGER PRIMARY KEY, data BLOB)", objtype);
+	query = "CREATE TABLE tbl_archive (objtype VARCHAR(64), mappingid INTEGER, data BLOB, PRIMARY KEY (objtype, mappingid) )";
 	if (!osync_db_query(db, query, error)) {
-		g_free(query);
 		goto error;
 	}
-	g_free(query);
 
 	osync_trace(TRACE_EXIT, "%s: created table.", __func__); 
 	return TRUE;
 
-error:
+ error:
 	osync_trace(TRACE_EXIT_ERROR, "%s: %s", __func__, osync_error_print(error));
 	return FALSE;
 }
 
-/**
- * @brief Creates a new archive object
- * @param filename the full path to the archive database file
- * @param error Pointer to an error struct
- * @return The pointer to the newly allocated archive object or NULL in case of error
- */
 OSyncArchive *osync_archive_new(const char *filename, OSyncError **error)
 {
+	OSyncArchive *archive = NULL;
 	osync_trace(TRACE_ENTRY, "%s(%s, %p)", __func__, filename, error);
 	osync_assert(filename);
 	
-	OSyncArchive *archive = osync_try_malloc0(sizeof(OSyncArchive), error);
+	archive = osync_try_malloc0(sizeof(OSyncArchive), error);
 	if (!archive)
 		goto error;
 
@@ -181,25 +161,21 @@ OSyncArchive *osync_archive_new(const char *filename, OSyncError **error)
 		goto error_and_free;
 
 	if (!osync_db_open(archive->db, filename, error)) {
-		g_free(archive->db);
+		osync_free(archive->db);
 		goto error_and_free;
 	}
 
 	osync_trace(TRACE_EXIT, "%s: %p", __func__, archive);
 	return archive;
 
-error_and_free:	
-	g_free(archive);
+ error_and_free:	
+	osync_free(archive);
 
-error:
+ error:
 	osync_trace(TRACE_EXIT_ERROR, "%s: %s", __func__, osync_error_print(error));
 	return NULL;
 }
 
-/**
- * @brief Increments the reference counter
- * @param archive The pointer to an archive object
- */
 OSyncArchive *osync_archive_ref(OSyncArchive *archive)
 {
 	osync_assert(archive);
@@ -209,11 +185,6 @@ OSyncArchive *osync_archive_ref(OSyncArchive *archive)
 	return archive;
 }
 
-/**
- * @brief Decrement the reference counter. The archive object will 
- *  be freed if there is no more reference to it.
- * @param archive The pointer to an archive object
- */
 void osync_archive_unref(OSyncArchive *archive)
 {
 	osync_assert(archive);
@@ -226,26 +197,18 @@ void osync_archive_unref(OSyncArchive *archive)
 				osync_trace(TRACE_INTERNAL, "Can't close database");
 		}
 		
-		g_free(archive->db);
-		g_free(archive);
+		osync_free(archive->db);
+		osync_free(archive);
 
 		osync_trace(TRACE_EXIT, "%s", __func__);
 	}
 }
 
-/**
- * @brief Stores data of an entry in the group archive database (blob).
- *
- * @param archive The group archive
- * @param id Archive (database) id of entry to save.
- * @param objtype The object type of the entry
- * @param data The data to store 
- * @param size Total size of data 
- * @param error Pointer to an error struct
- * @return Returns TRUE on success otherwise FALSE
- */ 
 osync_bool osync_archive_save_data(OSyncArchive *archive, long long int id, const char *objtype, const char *data, unsigned int size, OSyncError **error)
 {
+	char *query = NULL;
+	char *escaped_objtype = NULL;
+
 	osync_trace(TRACE_ENTRY, "%s(%p, %lli, %s, %p, %u, %p)", __func__, archive, id, objtype, data, size, error);
 	osync_assert(archive);
 	osync_assert(data);
@@ -255,36 +218,33 @@ osync_bool osync_archive_save_data(OSyncArchive *archive, long long int id, cons
 		goto error;
 
 	// FIXME: Avoid subselect - this query needs up to 0.5s
-	char *query = g_strdup_printf("REPLACE INTO tbl_archive_%s (mappingid, data) VALUES(%lli, ?)", objtype, id);
+	escaped_objtype = osync_db_sql_escape(objtype);
+	query = osync_strdup_printf("REPLACE INTO tbl_archive (objtype, mappingid, data) VALUES('%s', %lli, ?)", escaped_objtype, id);
+	osync_free(escaped_objtype);
+	escaped_objtype = NULL;
 	
 	if (!osync_db_bind_blob(archive->db, query, data, size, error)) {
-		g_free(query);
+		osync_free(query);
 		goto error;
 	}
 
-	g_free(query);
+	osync_free(query);
 
 	osync_trace(TRACE_EXIT, "%s", __func__);
 	return TRUE;
 
-error:
+ error:
 	osync_trace(TRACE_EXIT_ERROR, "%s: %s", __func__, osync_error_print(error));
 	return FALSE;
 }
 
-/**
- * @brief Loads data of an entry which is stored in the group archive database (blob).
- *
- * @param archive The group archive
- * @param uid UID of requestd entry
- * @param objtype The objtype type of the entry
- * @param data Pointer to store the requested data 
- * @param size Pointer to store the size of requested data
- * @param error Pointer to an error struct
- * @return Returns 0 if no data is present else 1. On error -1.
- */ 
 int osync_archive_load_data(OSyncArchive *archive, const char *uid, const char *objtype, char **data, unsigned int *size, OSyncError **error)
 {
+	char *escaped_uid = NULL;
+	char *escaped_objtype = NULL;
+	char *query = NULL;
+	int ret = 0;
+
 	osync_trace(TRACE_ENTRY, "%s(%p, %s, %s, %p, %p, %p)", __func__, archive, uid, objtype, data, size, error);
 	osync_assert(archive);
 	osync_assert(uid);
@@ -294,11 +254,17 @@ int osync_archive_load_data(OSyncArchive *archive, const char *uid, const char *
 	if (!osync_archive_create(archive->db, objtype, error))
 		goto error;
 
-	char *escaped_uid = osync_db_sql_escape(uid);
-	char *query = g_strdup_printf("SELECT data FROM tbl_archive_%s WHERE mappingid=(SELECT mappingid FROM tbl_changes_%s WHERE uid='%s' LIMIT 1)", objtype, objtype, escaped_uid);
-	int ret = osync_db_get_blob(archive->db, query, data, size, error);
-	g_free(query);
-	g_free(escaped_uid);
+	escaped_uid = osync_db_sql_escape(uid);
+	escaped_objtype = osync_db_sql_escape(objtype);
+	query = osync_strdup_printf("SELECT data FROM tbl_archive WHERE objtype='%s' AND mappingid=(SELECT mappingid FROM tbl_changes WHERE objtype='%s' AND uid='%s' LIMIT 1)", escaped_objtype, escaped_objtype, escaped_uid);
+	osync_free(escaped_objtype);
+
+	escaped_objtype = NULL;
+
+	ret = osync_db_get_blob(archive->db, query, data, size, error);
+
+	osync_free(query);
+	osync_free(escaped_uid);
 
 	if (ret < 0) {
 		goto error;
@@ -310,52 +276,49 @@ int osync_archive_load_data(OSyncArchive *archive, const char *uid, const char *
 	osync_trace(TRACE_EXIT, "%s", __func__);
 	return 1;
 	
-error:
+ error:
 	osync_trace(TRACE_EXIT_ERROR, "%s: %s", __func__, osync_error_print(error));
 	return -1;
 }
 
-/**
- * @brief Saves an entry in the group archive. 
- *
- * @param archive The group archive
- * @param id Archive (database) id of entry to update (if it already exists), specify 0 to add a new entry.
- * @param uid Reported UID of entry
- * @param objtype Reported object type of entry
- * @param mappingid Mapped ID of entry 
- * @param memberid ID of member which reported entry 
- * @param error Pointer to an error struct
- * @return Returns number of entries in archive group database. 0 on error. 
- */ 
-long long int osync_archive_save_change(OSyncArchive *archive, long long int id, const char *uid, const char *objtype, long long int mappingid, long long int memberid, OSyncError **error)
+long long int osync_archive_save_change(OSyncArchive *archive, long long int id, const char *uid, const char *objtype, long long int mappingid, long long int memberid, const char *objengine, OSyncError **error)
 {
-	
-	osync_trace(TRACE_ENTRY, "%s(%p, %lli, %s, %s, %lli, %lli, %p)", __func__, archive, id, uid, objtype, mappingid, memberid, error);
+	char *query = NULL;
+	char *escaped_uid = NULL;
+	char *escaped_objtype = NULL;
+	char *escaped_objengine = NULL;
+
+	osync_trace(TRACE_ENTRY, "%s(%p, %lli, %s, %s, %lli, %lli, %p, %s)", __func__, archive, id, uid, objtype, mappingid, memberid, __NULLSTR(objengine), error);
 	osync_assert(archive);
 	osync_assert(uid);
 	osync_assert(objtype);
+	osync_assert(objengine);
 
 	if (!osync_archive_create_changes(archive->db, objtype, error))
 		goto error;
 
-	char *query = NULL;
-	char *escaped_uid = osync_db_sql_escape(uid);
-
-
+	escaped_uid = osync_db_sql_escape(uid);
+	escaped_objtype = osync_db_sql_escape(objtype);
+	escaped_objengine = osync_db_sql_escape(objengine);
 
 	if (!id) {
-		query = g_strdup_printf("INSERT INTO tbl_changes_%s (uid, mappingid, memberid) VALUES('%s', '%lli', '%lli')", objtype, escaped_uid, mappingid, memberid);
+		query = osync_strdup_printf("INSERT INTO tbl_changes (objtype, uid, mappingid, memberid, objengine) VALUES('%s', '%s', '%lli', '%lli', '%s')", escaped_objtype, escaped_uid, mappingid, memberid, objengine);
 	} else {
-		query = g_strdup_printf("UPDATE tbl_changes_%s SET uid='%s', mappingid='%lli', memberid='%lli' WHERE id=%lli", objtype, escaped_uid, mappingid, memberid, id);
+		query = osync_strdup_printf("UPDATE tbl_changes SET uid='%s', mappingid='%lli', memberid='%lli', objengine='%s' WHERE objtype='%s' AND id=%lli", escaped_uid, mappingid, memberid, escaped_objengine, escaped_objtype, id);
 	}
-	g_free(escaped_uid);
+	osync_free(escaped_objengine);
+	osync_free(escaped_objtype);
+	osync_free(escaped_uid);
+	escaped_objengine = NULL;
+	escaped_objtype = NULL;
+	escaped_uid = NULL;
 	
 	if (!osync_db_query(archive->db, query, error)) {
-		g_free(query);
+		osync_free(query);
 		goto error;
 	}
 
-	g_free(query);
+	osync_free(query);
 	
 	if (!id)
 		id = osync_db_last_rowid(archive->db);
@@ -363,22 +326,15 @@ long long int osync_archive_save_change(OSyncArchive *archive, long long int id,
 	osync_trace(TRACE_EXIT, "%s: %lli", __func__, id);
 	return id;
 	
-error:
+ error:
 	osync_trace(TRACE_EXIT_ERROR, "%s: %s", __func__, osync_error_print(error));
 	return 0;
 }
 
-/**
- * @brief Deletes an entry from a group archive.
- *
- * @param archive The group archive
- * @param id Archive (database) id of entry to be deleted
- * @param objtype The object type of the entry
- * @param error Pointer to an error struct
- * @return TRUE if the specified change was deleted successfully, otherwise FALSE
- */ 
 osync_bool osync_archive_delete_change(OSyncArchive *archive, long long int id, const char *objtype, OSyncError **error)
 {
+	char *query = NULL;
+	char *escaped_objtype = NULL;
 	osync_trace(TRACE_ENTRY, "%s(%p, %lli, %s, %p)", __func__, archive, id, objtype, error);
 	osync_assert(archive);
 	osync_assert(objtype);
@@ -386,36 +342,34 @@ osync_bool osync_archive_delete_change(OSyncArchive *archive, long long int id,
 	if (!osync_archive_create_changes(archive->db, objtype, error))
 		goto error;
 
-	char *query = g_strdup_printf("DELETE FROM tbl_changes_%s WHERE id=%lli", objtype, id);
+	escaped_objtype = osync_db_sql_escape(objtype);
+	query = osync_strdup_printf("DELETE FROM tbl_changes WHERE objtype='%s' AND id=%lli", escaped_objtype, id);
+	osync_free(escaped_objtype);
+	escaped_objtype = NULL;
 	if (!osync_db_query(archive->db, query, error)) {
-		g_free(query);
+		osync_free(query);
 		goto error;
 	}
 
-	g_free(query);
+	osync_free(query);
 	
 	osync_trace(TRACE_EXIT, "%s", __func__);
 	return TRUE;
 
-error:	
+ error:	
 	osync_trace(TRACE_EXIT_ERROR, "%s: %s", __func__, osync_error_print(error));
 	return FALSE;
 }
 
-/**
- * @brief Loads all changes from group archive for a certain object type.
- *
- * @param archive The group archive
- * @param objtype Requested object type 
- * @param ids List to store the archive (database) ids of each entry
- * @param uids List to store uids of each entry
- * @param mappingids List to store mappingids for each entry
- * @param memberids List to store member IDs for each entry 
- * @param error Pointer to an error struct
- * @return TRUE on when all changes successfully loaded otherwise FALSE
- */ 
 osync_bool osync_archive_load_changes(OSyncArchive *archive, const char *objtype, OSyncList **ids, OSyncList **uids, OSyncList **mappingids, OSyncList **memberids, OSyncError **error)
 {
+	char *query = NULL;
+	char *escaped_objtype = NULL;
+	OSyncList *result = NULL, *row = NULL, *column = NULL;
+	long long int id = 0, mappingid = 0, memberid = 0;
+	const char *uid = NULL;
+	const char *value_str = NULL;
+
 	osync_trace(TRACE_ENTRY, "%s(%p, %s, %p, %p, %p, %p, %p)", __func__, archive, objtype, ids, uids, mappingids, memberids, error);
 
 	osync_assert(archive);
@@ -425,56 +379,78 @@ osync_bool osync_archive_load_changes(OSyncArchive *archive, const char *objtype
 	osync_assert(mappingids);
 	osync_assert(memberids);
 
-	OSyncList *result = NULL, *row = NULL;
+	*ids = NULL;
+	*uids = NULL;
+	*mappingids = NULL;
+	*memberids = NULL;
 
 	if (!osync_archive_create_changes(archive->db, objtype, error))
 		goto error;
 
-	char *query = g_strdup_printf("SELECT id, uid, mappingid, memberid FROM tbl_changes_%s ORDER BY mappingid", objtype);
+	escaped_objtype = osync_db_sql_escape(objtype);
+	query = osync_strdup_printf("SELECT id, uid, mappingid, memberid FROM tbl_changes WHERE objtype='%s' ORDER BY mappingid", escaped_objtype);
+	osync_free(escaped_objtype);
+	escaped_objtype = NULL;
 	result = osync_db_query_table(archive->db, query, error);
 
-	g_free(query);
+	osync_free(query);
 	
 	/* Check for error of osync_db_query_table() call. */
 	if (osync_error_is_set(error))
 		goto error;
 
 	for (row = result; row; row = row->next) { 
-		OSyncList *column = row->data;
+		column = row->data;
 
-		long long int id = g_ascii_strtoull(osync_list_nth_data(column, 0), NULL, 0);
-		const char *uid = osync_list_nth_data(column, 1); 
-		long long int mappingid = g_ascii_strtoull(osync_list_nth_data(column, 2), NULL, 0);
-		long long int memberid = g_ascii_strtoull(osync_list_nth_data(column, 3), NULL, 0);
+		value_str = osync_list_nth_data(column, 0);
+		if (!value_str) {
+			osync_error_set(error, OSYNC_ERROR_GENERIC, "Database table tbl_changes corrupt, id is NULL");
+			goto error;
+		}
+		id = g_ascii_strtoull(value_str, NULL, 0);
+
+		uid = osync_list_nth_data(column, 1);
+		if (!uid) {
+			osync_error_set(error, OSYNC_ERROR_GENERIC, "Database table tbl_changes corrupt, uid is NULL");
+			goto error;
+		}
+
+		value_str = osync_list_nth_data(column, 2);
+		if (!value_str) {
+			osync_error_set(error, OSYNC_ERROR_GENERIC, "Database table tbl_changes corrupt, mappingid is NULL");
+			goto error;
+		}
+		mappingid = g_ascii_strtoull(value_str, NULL, 0);
+
+		value_str = osync_list_nth_data(column, 3);
+		if (!value_str) {
+			osync_error_set(error, OSYNC_ERROR_GENERIC, "Database table tbl_changes corrupt, memberid is NULL");
+			goto error;
+		}
+		memberid = g_ascii_strtoull(value_str, NULL, 0);
 		
 		*ids = osync_list_append((*ids), GINT_TO_POINTER((int)id));
-		*uids = osync_list_append((*uids), g_strdup(uid));
+		*uids = osync_list_append((*uids), osync_strdup(uid));
 		*mappingids = osync_list_append((*mappingids), GINT_TO_POINTER((int)mappingid));
 		*memberids = osync_list_append((*memberids), GINT_TO_POINTER((int)memberid));
 		
-	    	osync_trace(TRACE_INTERNAL, "Loaded change with uid %s, mappingid %lli from member %lli", uid, mappingid, memberid);
+		osync_trace(TRACE_INTERNAL, "Loaded change with uid %s, mappingid %lli from member %lli", uid, mappingid, memberid);
 	}
 
 	osync_db_free_list(result);	
 
 	osync_trace(TRACE_EXIT, "%s", __func__);
 	return TRUE;
-error:
+ error:
 	osync_trace(TRACE_EXIT_ERROR, "%s: %s", __func__, osync_error_print(error));
 	return FALSE;	
 }
 
-/**
- * @brief Delete all changes from group archive for a certain object type.
- *
- * @param archive The group archive
- * @param objtype Reported object type of entry
- * @param error Pointer to an error struct
- * @return Returns TRUE on success, FALSE otherwise 
- */ 
 osync_bool osync_archive_flush_changes(OSyncArchive *archive, const char *objtype, OSyncError **error)
 {
-	
+	char *query = NULL;
+	char *escaped_objtype = NULL;
+
 	osync_trace(TRACE_ENTRY, "%s(%p, %s, %p)", __func__, archive, objtype, error);
 	osync_assert(archive);
 	osync_assert(objtype);
@@ -482,35 +458,34 @@ osync_bool osync_archive_flush_changes(OSyncArchive *archive, const char *objtyp
 	if (!osync_archive_create_changes(archive->db, objtype, error))
 		goto error;
 	
-	char *query = g_strdup_printf("DELETE FROM tbl_changes_%s", objtype);
+	escaped_objtype = osync_db_sql_escape(objtype);
+	query = osync_strdup_printf("DELETE FROM tbl_changes WHERE objtype='%s'", escaped_objtype);
+	osync_free(escaped_objtype);
+	escaped_objtype = NULL;
 	
 	if (!osync_db_query(archive->db, query, error)) {
-		g_free(query);
+		osync_free(query);
 		goto error;
 	}
 
-	g_free(query);
+	osync_free(query);
 	
 	osync_trace(TRACE_EXIT, "%s", __func__);
 	return TRUE;
 	
-error:
+ error:
 	osync_trace(TRACE_EXIT_ERROR, "%s: %s", __func__, osync_error_print(error));
 	return FALSE;
 }
 
-/**
- * @brief Loads all conficting changes which were ignored in the previous sync. 
- *
- * @param archive The group archive
- * @param objtype Requested object type 
- * @param ids List to store the archive (database) ids of each entry
- * @param changetypes List to store the changetypes for each entry
- * @param error Pointer to an error struct
- * @return TRUE on when all changes successfully loaded otherwise FALSE
- */ 
 osync_bool osync_archive_load_ignored_conflicts(OSyncArchive *archive, const char *objtype, OSyncList **ids, OSyncList **changetypes, OSyncError **error)
 {
+	OSyncList *result = NULL, *row = NULL, *column = NULL;
+	char *query = NULL;
+	char *escaped_objtype = NULL;
+	long long int id = 0;
+	int changetype = 0;
+
 	osync_trace(TRACE_ENTRY, "%s(%p, %s, %p, %p)", __func__, archive, objtype, ids, error);
 
 	osync_assert(archive);
@@ -518,54 +493,46 @@ osync_bool osync_archive_load_ignored_conflicts(OSyncArchive *archive, const cha
 	osync_assert(ids);
 	osync_assert(changetypes);
 
-	OSyncList *result = NULL;
-	OSyncList *row = NULL;
-
 	if (!osync_archive_create_changelog(archive->db, objtype, error))
 		goto error;
 
-	char *query = g_strdup_printf("SELECT entryid, changetype FROM tbl_changelog_%s ORDER BY id", objtype);
+	escaped_objtype = osync_db_sql_escape(objtype);
+	query = osync_strdup_printf("SELECT entryid, changetype FROM tbl_changelog WHERE objtype='%s' ORDER BY id", escaped_objtype);
+	osync_free(escaped_objtype);
+	escaped_objtype = NULL;
 	result = osync_db_query_table(archive->db, query, error);
 
-	g_free(query);
+	osync_free(query);
 	
 	/* Check for error of osync_db_query_table() call. */
 	if (osync_error_is_set(error))
 		goto error;
 
 	for (row = result; row; row = row->next) { 
-		OSyncList *column = row->data;
+		column = row->data;
 
-		long long int id = g_ascii_strtoull(osync_list_nth_data(column, 0), NULL, 0);
-		int changetype = atoi(osync_list_nth_data(column, 1));
+		id = g_ascii_strtoull(osync_list_nth_data(column, 0), NULL, 0);
+		changetype = atoi(osync_list_nth_data(column, 1));
 		
 		*ids = osync_list_append((*ids), GINT_TO_POINTER((int)id));
 		*changetypes = osync_list_append((*changetypes), GINT_TO_POINTER((int)changetype));
 		
-	    	osync_trace(TRACE_INTERNAL, "Loaded ignored mapping with entryid %lli", id);
+		osync_trace(TRACE_INTERNAL, "Loaded ignored mapping with entryid %lli", id);
 	}
 
 	osync_db_free_list(result);	
 
 	osync_trace(TRACE_EXIT, "%s", __func__);
 	return TRUE;
-error:
+ error:
 	osync_trace(TRACE_EXIT_ERROR, "%s: %s", __func__, osync_error_print(error));
 	return FALSE;	
 }
 
-/**
- * @brief Saves an entry in the ignored conflict list.
- *
- * @param archive The group archive
- * @param objtype Reported object type of entry
- * @param id Mapping Entry ID of entry 
- * @param changetype Changetype of entry 
- * @param error Pointer to an error struct
- * @return Returns TRUE on success, FALSE otherwise 
- */ 
 osync_bool osync_archive_save_ignored_conflict(OSyncArchive *archive, const char *objtype, long long int id, OSyncChangeType changetype, OSyncError **error)
 {
+	char *query = NULL;
+	char *escaped_objtype = NULL;
 	osync_trace(TRACE_ENTRY, "%s(%p, %s, %lli, %p)", __func__, archive, objtype, id, error);
 
 	osync_assert(archive);
@@ -574,34 +541,30 @@ osync_bool osync_archive_save_ignored_conflict(OSyncArchive *archive, const char
 	if (!osync_archive_create_changelog(archive->db, objtype, error))
 		goto error;
 	
-	char *query = g_strdup_printf("INSERT INTO tbl_changelog_%s (entryid, changetype) VALUES('%lli', '%i')", objtype, id, changetype);
+	escaped_objtype = osync_db_sql_escape(objtype);
+	query = osync_strdup_printf("INSERT INTO tbl_changelog (objtype, entryid, changetype) VALUES('%s', '%lli', '%i')", escaped_objtype, id, changetype);
+	osync_free(escaped_objtype);
+	escaped_objtype = NULL;
 	
 	if (!osync_db_query(archive->db, query, error)) {
-		g_free(query);
+		osync_free(query);
 		goto error;
 	}
 
-	g_free(query);
+	osync_free(query);
 	
 	osync_trace(TRACE_EXIT, "%s: %lli", __func__, id);
 	return TRUE;
 	
-error:
+ error:
 	osync_trace(TRACE_EXIT_ERROR, "%s: %s", __func__, osync_error_print(error));
 	return FALSE;
 }
 
-/**
- * @brief Deletes all ignored conflict entries of the changelog with the objtype.
- *
- * @param archive The group archive
- * @param objtype Reported object type of entry
- * @param error Pointer to an error struct
- * @return Returns TRUE on success, FALSE otherwise 
- */ 
 osync_bool osync_archive_flush_ignored_conflict(OSyncArchive *archive, const char *objtype, OSyncError **error)
 {
-	
+	char *query = NULL;
+	char *escaped_objtype = NULL;
 	osync_trace(TRACE_ENTRY, "%s(%p, %s, %p)", __func__, archive, objtype, error);
 	osync_assert(archive);
 	osync_assert(objtype);
@@ -609,21 +572,63 @@ osync_bool osync_archive_flush_ignored_conflict(OSyncArchive *archive, const cha
 	if (!osync_archive_create_changelog(archive->db, objtype, error))
 		goto error;
 	
-	char *query = g_strdup_printf("DELETE FROM tbl_changelog_%s", objtype);
+	escaped_objtype = osync_db_sql_escape(objtype);
+	query = osync_strdup_printf("DELETE FROM tbl_changelog WHERE objtype='%s'", escaped_objtype);
+	osync_free(escaped_objtype);
+	escaped_objtype = NULL;
 	
 	if (!osync_db_query(archive->db, query, error)) {
-		g_free(query);
+		osync_free(query);
 		goto error;
 	}
 
-	g_free(query);
+	osync_free(query);
 	
 	osync_trace(TRACE_EXIT, "%s", __func__);
 	return TRUE;
 	
-error:
+ error:
 	osync_trace(TRACE_EXIT_ERROR, "%s: %s", __func__, osync_error_print(error));
 	return FALSE;
 }
 
-/*@}*/
+osync_bool osync_archive_get_mixed_objengines(OSyncArchive *archive, const char *objengine, OSyncList **objengines, OSyncError **error)
+{
+	char *query = NULL, *escaped_objengine, *objengine_name;
+	OSyncList *result, *row, *column;
+
+	osync_assert(archive);
+	osync_assert(objengine);
+	osync_assert(objengines);
+
+	escaped_objengine = osync_db_sql_escape(objengine);
+
+	query = osync_strdup_printf("SELECT DISTINCT(b.objengine) FROM tbl_changes, tbl_changes as a, tbl_changes as b "
+			"WHERE a.mappingid == b.mappingid AND a.objengine == '%s';", escaped_objengine);
+	result = osync_db_query_table(archive->db, query, error);
+	osync_free(query);
+	
+	/* Check for error of osync_db_query_table() call. */
+	if (osync_error_is_set(error))
+		goto error;
+
+	for (row = result; row; row = row->next) { 
+		column = row->data;
+
+		objengine_name = osync_list_nth_data(column, 0);
+		if (!objengine_name) {
+			osync_error_set(error, OSYNC_ERROR_GENERIC, "Database table tbl_changes corrupt. Couldn't query for mixed object engines.");
+			goto error;
+		}
+		
+		*objengines = osync_list_append((*objengines), osync_strdup(objengine_name));
+	}
+
+	osync_db_free_list(result);	
+
+
+	return TRUE;
+error:
+	return FALSE;
+}
+
