@@ -1,7 +1,7 @@
 /*
  * libosengine - A synchronization engine for the opensync framework
  * Copyright (C) 2004-2005  Armin Bauer <armin.bauer@opensync.org>
- * Copyright (C) 2007       Daniel Gollub <dgollub@suse.de> 
+ * Copyright (C) 2007       Daniel Gollub <gollub@b1-systems.de> 
  * 
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -24,11 +24,19 @@
 
 #include "opensync_mapping_entry_engine_internals.h"
 
+/**
+ * @defgroup OSyncMappingEngineInternalAPI OpenSync Mapping Engine Internals
+ * @ingroup OSyncEnginePrivate
+ * @brief The internal part of the OSyncMappingEngine API
+ * 
+ */
+/*@{*/
+
 struct OSyncMappingEngine {
 	int ref_count;
 	OSyncMapping *mapping;
 	OSyncMappingEntryEngine *master;
-	GList *entries;
+	OSyncList *entries; /* OSyncMappingEntryEngine */
 	OSyncObjEngine *parent;
 	osync_bool conflict;
 	osync_bool synced;
@@ -39,7 +47,36 @@ OSyncMappingEngine *osync_mapping_engine_ref(OSyncMappingEngine *engine);
 void osync_mapping_engine_unref(OSyncMappingEngine *engine);
 
 osync_bool osync_mapping_engine_multiply(OSyncMappingEngine *engine, OSyncError **error);
-void osync_mapping_engine_check_conflict(OSyncMappingEngine *engine);
+
+/**
+ * @brief Checks for conflicts in the Mapping Engine and calls conflict
+ *        callbacks if required.
+ *
+ * On a conflict the conflict callbacks get called in a syncrhounous way
+ * and might block. But the application could also delay the conflict
+ * resolution and solve it later.
+ *
+ * If this functions fails, by returning FALSE, synchronization got aborted
+ * during the conflict resolution.
+ *
+ * @param engine Pointer to the Mapping Engine struct 
+ * @return Returns TRUE on success, FALSE if conflict resolution got aborted
+ */
+osync_bool osync_mapping_engine_check_conflict(OSyncMappingEngine *engine);
+
 OSyncMappingEntryEngine *osync_mapping_engine_get_entry(OSyncMappingEngine *engine, OSyncSinkEngine *sinkengine);
 
-#endif /*OPENSYNC_MAPPING_ENGINE_INTERNALS_H_*/
+OSYNC_TEST_EXPORT unsigned int osync_mapping_engine_num_changes(OSyncMappingEngine *engine);
+
+/** @brief Search for the nth entry in the mapping
+ *
+ * @param engine A pointer to the mapping engine
+ * @param nth The value of the position
+ * @returns The pointer to the nth change. NULL if there isn't enough entries in the mapping.
+ */
+OSYNC_TEST_EXPORT OSyncChange *osync_mapping_engine_nth_change(OSyncMappingEngine *engine, unsigned int nth);
+
+/*@}*/
+
+#endif /* OPENSYNC_MAPPING_ENGINE_INTERNALS_H_ */
+
