@@ -28,21 +28,23 @@
 #include "opensync-data.h"
 
 #include "opensync_engine_internals.h"
+#include "opensync_engine_private.h" /* FIXME: Access private struct from OSyncEngine */
+
 #include "opensync_mapping_engine_internals.h"
 
-void osync_status_free_member_update(OSyncMemberUpdate *update)
+void osync_status_free_member_update(OSyncEngineMemberUpdate *update)
 {
 	osync_assert(update);
 	
 	if (update->objtype)
-		g_free(update->objtype);
+		osync_free(update->objtype);
 	
 	osync_member_unref(update->member);
 	
 	if (update->error)
 		osync_error_unref(&update->error);
 	
-	g_free(update);
+	osync_free(update);
 }
 
 void osync_status_free_engine_update(OSyncEngineUpdate *update)
@@ -52,10 +54,10 @@ void osync_status_free_engine_update(OSyncEngineUpdate *update)
 	if (update->error)
 		osync_error_unref(&update->error);
 	
-	g_free(update);
+	osync_free(update);
 }
 
-void osync_status_free_change_update(OSyncChangeUpdate *update)
+void osync_status_free_change_update(OSyncEngineChangeUpdate *update)
 {
 	osync_assert(update);
 	
@@ -67,10 +69,10 @@ void osync_status_free_change_update(OSyncChangeUpdate *update)
 	if (update->error)
 		osync_error_unref(&update->error);
 	
-	g_free(update);
+	osync_free(update);
 }
 
-void osync_status_free_mapping_update(OSyncMappingUpdate *update)
+void osync_status_free_mapping_update(OSyncEngineMappingUpdate *update)
 {
 	osync_assert(update);
 	
@@ -79,28 +81,53 @@ void osync_status_free_mapping_update(OSyncMappingUpdate *update)
 	if (update->error)
 		osync_error_unref(&update->error);
 	
-	g_free(update);
+	osync_free(update);
 }
 
-void osync_status_conflict(OSyncEngine *engine, OSyncMappingEngine *mapping_engine)
+osync_bool osync_status_conflict(OSyncEngine *engine, OSyncMappingEngine *mapping_engine)
 {
 	osync_trace(TRACE_ENTRY, "%s(%p, %p)", __func__, engine, mapping_engine);
 
 	osync_assert_msg(engine->conflict_callback, "No conflict handler registered! OpenSync frontend very likely broken!");
+	osync_assert_msg(engine->conflict_callback, "Engine is already tainted with an error! Not going to call any conflict callback!");
+	if (osync_engine_has_error(engine))
+		goto error;
 
 	engine->conflict_callback(engine, mapping_engine, engine->conflict_userdata);
+
+	if (osync_engine_has_error(engine))
+		goto error;
+				
+	osync_trace(TRACE_EXIT, "%s", __func__);
+	return TRUE;
+
+error:
+	osync_trace(TRACE_EXIT_ERROR, "%s: Engine has an error. Sync got abortd?!", __func__);
+	return FALSE;
+}
+
+void osync_status_multiply(OSyncEngine *engine)
+{
+	osync_trace(TRACE_ENTRY, "%s(%p)", __func__, engine);
+
+	if (engine->multiply_callback)
+		engine->multiply_callback(engine, engine->multiply_userdata);
 				
 	osync_trace(TRACE_EXIT, "%s", __func__);
 }
 
-void osync_status_update_member(OSyncEngine *engine, OSyncMember *member, OSyncMemberEvent type, const char *objtype, OSyncError *error)
+void osync_status_update_member(OSyncEngine *engine, OSyncMember *member, OSyncEngineMemberEvent type, const char *objtype, OSyncError *error)
 {
 	osync_trace(TRACE_ENTRY, "%s(%p, %p, %i, %s, %p)", __func__, engine, member, type, objtype, error);
 	
 	if (engine->mebstat_callback) {
-		OSyncMemberUpdate *update = g_malloc0(sizeof(OSyncMemberUpdate));
-		if (!update)
+		OSyncError *internal_error = NULL;
+		OSyncEngineMemberUpdate *update = osync_try_malloc0(sizeof(OSyncEngineMemberUpdate), &internal_error);
+		if (!update) {
+			osync_trace(TRACE_EXIT_ERROR, "%s: %s", __func__, osync_error_print(&internal_error));
+			osync_error_unref(&internal_error);
 			return;
+		}
 		
 		update->type = type;
 		
@@ -110,7 +137,7 @@ void osync_status_update_member(OSyncEngine *engine, OSyncMember *member, OSyncM
 		update->error = error;
 		osync_error_ref(&error);
 		
-		update->objtype = g_strdup(objtype);
+		update->objtype = osync_strdup(objtype);
 		
 		engine->mebstat_callback(update, engine->mebstat_userdata);
 		
@@ -121,14 +148,18 @@ void osync_status_update_member(OSyncEngine *engine, OSyncMember *member, OSyncM
 	osync_trace(TRACE_EXIT, "%s", __func__);
 }
 
-void osync_status_update_change(OSyncEngine *engine, OSyncChange *change, OSyncMember *member, OSyncMapping *mapping, OSyncChangeEvent type, OSyncError *error)
+void osync_status_update_change(OSyncEngine *engine, OSyncChange *change, OSyncMember *member, OSyncMapping *mapping, OSyncEngineChangeEvent type, OSyncError *error)
 {
 	osync_trace(TRACE_ENTRY, "%s(%p, %p, %p, %p, %i, %p)", __func__, engine, change, member, mapping, type, error);
 	
 	if (engine->changestat_callback) {
-		OSyncChangeUpdate *update = g_malloc0(sizeof(OSyncChangeUpdate));
-		if (!update)
+		OSyncError *internal_error = NULL;
+		OSyncEngineChangeUpdate *update = osync_try_malloc0(sizeof(OSyncEngineChangeUpdate), &internal_error);
+		if (!update) {
+			osync_trace(TRACE_EXIT_ERROR, "%s: %s", __func__, osync_error_print(&internal_error));
+			osync_error_unref(&internal_error);
 			return;
+		}
 		
 		update->type = type;
 		
@@ -150,14 +181,18 @@ void osync_status_update_change(OSyncEngine *engine, OSyncChange *change, OSyncM
 	osync_trace(TRACE_EXIT, "%s", __func__);
 }
 
-void osync_status_update_mapping(OSyncEngine *engine, OSyncMappingEngine *mapping, OSyncMappingEvent type, OSyncError *error)
+void osync_status_update_mapping(OSyncEngine *engine, OSyncMappingEngine *mapping, OSyncEngineMappingEvent type, OSyncError *error)
 {
 	osync_trace(TRACE_ENTRY, "%s(%p, %p, %i, %p)", __func__, engine, mapping, type, error);
 	
 	if (engine->mapstat_callback) {
-		OSyncMappingUpdate *update = g_malloc0(sizeof(OSyncMappingUpdate));
-		if (!update)
+		OSyncError *internal_error = NULL;
+		OSyncEngineMappingUpdate *update = osync_try_malloc0(sizeof(OSyncEngineMappingUpdate), &internal_error);
+		if (!update) {
+			osync_trace(TRACE_EXIT_ERROR, "%s: %s", __func__, osync_error_print(&internal_error));
+			osync_error_unref(&internal_error);
 			return;
+		}
 		
 		update->type = type;
 		
@@ -180,9 +215,13 @@ void osync_status_update_engine(OSyncEngine *engine, OSyncEngineEvent type, OSyn
 	osync_trace(TRACE_ENTRY, "%s(%p, %i, %p)", __func__, engine, type, error);
 	
 	if (engine->engstat_callback) {
-		OSyncEngineUpdate *update = g_malloc0(sizeof(OSyncEngineUpdate));
-		if (!update)
+		OSyncError *internal_error = NULL;
+		OSyncEngineUpdate *update = osync_try_malloc0(sizeof(OSyncEngineUpdate), &internal_error);
+		if (!update) {
+			osync_trace(TRACE_EXIT_ERROR, "%s: %s", __func__, osync_error_print(&internal_error));
+			osync_error_unref(&internal_error);
 			return;
+		}
 		
 		update->type = type;
 		
