@@ -23,85 +23,22 @@
 
 #include "opensync-group.h"
 #include "opensync_group_env_internals.h"
+#include "opensync_group_env_private.h"
 
-/**
- * @defgroup PrivateAPI Private APIs
- * @brief Available private APIs
- * 
- */
-
-/**
- * @defgroup OSyncPrivate OpenSync Private API
- * @ingroup PrivateAPI
- * @brief The private API of opensync
- * 
- * This gives you an insight in the private API of opensync.
- * 
- */
-
-/**
- * @defgroup OSyncGroupEnvPrivate OpenSync Environment Internals
- * @ingroup OSyncPrivate
- * @brief The internals of the opensync environment
- * 
- */
-/*@{*/
-
-/*! @brief Returns the next free number for a group in the environments configdir
- * 
- * Returns the next free number for a group in the environments configdir
- * 
- * @param env The osync environment
- * @returns The next free number
- * 
- */
-static long long int _osync_group_env_create_group_id(OSyncGroupEnv *env)
+static long long int osync_group_env_create_group_id(OSyncGroupEnv *env)
 {
 	char *filename = NULL;
 	long long int i = 0;
 	do {
 		i++;
 		if (filename)
-			g_free(filename);
-		filename = g_strdup_printf("%s%cgroup%lli", env->groupsdir, G_DIR_SEPARATOR, i);
+			osync_free(filename);
+		filename = osync_strdup_printf("%s%cgroup%lli", env->groupsdir, G_DIR_SEPARATOR, i);
 	} while (g_file_test(filename, G_FILE_TEST_EXISTS));
-	g_free(filename);
+	osync_free(filename);
 	return i;
 }
 
-/*@}*/
-
-/**
- * @defgroup PublicAPI Public APIs
- * @brief Available public APIs
- * 
- */
-
-/**
- * @defgroup OSyncPublic OpenSync Public API
- * @ingroup PublicAPI
- * @brief The public API of opensync
- * 
- * This gives you an insight in the public API of opensync.
- * 
- */
-
-/**
- * @defgroup OSyncGroupEnvAPI OpenSync Environment
- * @ingroup OSyncPublic
- * @brief The public API of the opensync environment
- * 
- */
-/*@{*/
-
-
-/*! @brief This will create a new opensync environment
- * 
- * The environment will hold all information about plugins, groups etc
- * 
- * @returns A pointer to a newly allocated environment. NULL on error.
- * 
- */
 OSyncGroupEnv *osync_group_env_new(OSyncError **error)
 {
 	OSyncGroupEnv *env = NULL;
@@ -112,48 +49,46 @@ OSyncGroupEnv *osync_group_env_new(OSyncError **error)
 		osync_trace(TRACE_EXIT_ERROR, "%s: %s", __func__, osync_error_print(error));
 		return NULL;
 	}
+	env->ref_count = 1;
+
+	osync_trace(TRACE_EXIT, "%s: %p", __func__, env);
+	return env;
+
+}
+
+
+OSyncGroupEnv *osync_group_env_ref(OSyncGroupEnv *env)
+{
+	osync_trace(TRACE_ENTRY, "%s(%p)", __func__, env);
+	osync_assert(env);
+
+	g_atomic_int_inc(&(env->ref_count));
 	
 	osync_trace(TRACE_EXIT, "%s: %p", __func__, env);
 	return env;
 }
 
-/*! @brief Frees a osync environment
- * 
- * Frees a osync environment and all resources.
- * 
- * @param env Pointer to the environment to free
- * 
- */
-void osync_group_env_free(OSyncGroupEnv *env)
+void osync_group_env_unref(OSyncGroupEnv *env)
 {
 	osync_trace(TRACE_ENTRY, "%s(%p)", __func__, env);
-	g_assert(env);
+	osync_assert(env);
+
+	if (g_atomic_int_dec_and_test(&(env->ref_count))) {
+		if (env->groupsdir)
+			osync_free(env->groupsdir);
 	
-	if (env->groupsdir)
-		g_free(env->groupsdir);
+		/* Free the groups */
+		while (env->groups) {
+			osync_group_unref(env->groups->data);
+			env->groups = osync_list_remove(env->groups, env->groups->data);
+		}
 	
-	/* Free the groups */
-	while (env->groups) {
-		osync_group_unref(env->groups->data);
-		env->groups = g_list_remove(env->groups, env->groups->data);
+		osync_free(env);
 	}
-	
-	g_free(env);
-	
+
 	osync_trace(TRACE_EXIT, "%s", __func__);
 }
 
-/*! @brief Loads the plugins from a given directory
- * 
- * Loads all plugins from a directory into a osync environment.
- * The directory must exist prior to opening.
- * 
- * @param env Pointer to a OSyncGroupEnv environment
- * @param path The path where to look for groups
- * @param error Pointer to a error struct to return a error
- * @returns TRUE on success, FALSE otherwise
- * 
- */
 osync_bool osync_group_env_load_groups(OSyncGroupEnv *env, const char *path, OSyncError **error)
 {	
 	GDir *dir = NULL;
@@ -175,7 +110,7 @@ osync_bool osync_group_env_load_groups(OSyncGroupEnv *env, const char *path, OSy
 		if (!homedir)
 			homedir = g_get_home_dir();
 
-		env->groupsdir = g_strdup_printf("%s%c.opensync", homedir, G_DIR_SEPARATOR);
+		env->groupsdir = osync_strdup_printf("%s%c.opensync", homedir, G_DIR_SEPARATOR);
 		osync_trace(TRACE_INTERNAL, "Default home dir: %s", env->groupsdir);
 		
 		if (!g_file_test(env->groupsdir, G_FILE_TEST_EXISTS)) {
@@ -187,9 +122,9 @@ osync_bool osync_group_env_load_groups(OSyncGroupEnv *env, const char *path, OSy
 		}
 	} else {
 		if (!g_path_is_absolute(path)) {
-			env->groupsdir = g_strdup_printf("%s%c%s", g_get_current_dir(), G_DIR_SEPARATOR, path);
+			env->groupsdir = osync_strdup_printf("%s%c%s", g_get_current_dir(), G_DIR_SEPARATOR, path);
 		} else {
-			env->groupsdir = g_strdup(path);
+			env->groupsdir = osync_strdup(path);
 		}
 	}
 	
@@ -207,22 +142,22 @@ osync_bool osync_group_env_load_groups(OSyncGroupEnv *env, const char *path, OSy
 	}
 	
 	while ((de = g_dir_read_name(dir))) {
-		filename = g_strdup_printf ("%s%c%s", env->groupsdir, G_DIR_SEPARATOR, de);
+		filename = osync_strdup_printf ("%s%c%s", env->groupsdir, G_DIR_SEPARATOR, de);
 		
 		if (!g_file_test(filename, G_FILE_TEST_IS_DIR) || !g_pattern_match_simple("group*", de)) {
-			g_free(filename);
+			osync_free(filename);
 			continue;
 		}
 		
 		/* Try to open the confdir*/
 		group = osync_group_new(error);
 		if (!group) {
-			g_free(filename);
+			osync_free(filename);
 			goto error_close_dir;
 		}
 		
 		if (!osync_group_load(group, filename, error)) {
-			g_free(filename);
+			osync_free(filename);
 			osync_group_unref(group);
 			goto error_close_dir;
 		}
@@ -230,34 +165,25 @@ osync_bool osync_group_env_load_groups(OSyncGroupEnv *env, const char *path, OSy
 		osync_group_env_add_group(env, group, error);
 		osync_group_unref(group);
 		
-		g_free(filename);
+		osync_free(filename);
 	}
 	g_dir_close(dir);
 
 	osync_trace(TRACE_EXIT, "%s", __func__);
 	return TRUE;
 
-error_close_dir:
+ error_close_dir:
 	g_dir_close(dir);
-error_free_path:
-	g_free(env->groupsdir);
+ error_free_path:
+	osync_free(env->groupsdir);
 	env->groupsdir = NULL;
 	osync_trace(TRACE_EXIT_ERROR, "%s: %s", __func__, osync_error_print(error));
 	return FALSE;
 }
 
-/*! @brief Finds the group with the given name
- * 
- * Finds the group with the given name
- * 
- * @param env Pointer to a OSyncGroupEnv environment
- * @param name Name of the group to search
- * @returns Pointer to group. NULL if not found
- * 
- */
 OSyncGroup *osync_group_env_find_group(OSyncGroupEnv *env, const char *name)
 {
-	GList *g = NULL;
+	OSyncList *g = NULL;
 	osync_assert(env);
 	osync_assert(name);
 	
@@ -270,25 +196,15 @@ OSyncGroup *osync_group_env_find_group(OSyncGroupEnv *env, const char *name)
 	return NULL;
 }
 
-/*! @brief Adds the given group to the environment.
- * 
- * Adds the given group to the environment.
- * 
- * @param env Pointer to a OSyncGroupEnv environment
- * @param group The group to add. The group must have a name.
- * @param error Pointer to a error struct to return a error
- * @returns FALSE if group with the same name already exists. 
- * 
- */
 osync_bool osync_group_env_add_group(OSyncGroupEnv *env, OSyncGroup *group, OSyncError **error)
 {
+	const char *group_name = NULL;
 	osync_assert(env);
 	osync_assert(group);
 
-	const char *group_name = osync_group_get_name(group);
-
+	group_name = osync_group_get_name(group);
 	/* Fail if no group name is already set. The Group name must be set in
-	   advanced to check if a group with the same name already exists. */
+		 advanced to check if a group with the same name already exists. */
 	if (!group_name) {
 		osync_error_set(error, OSYNC_ERROR_GENERIC, "Group has no name. The Group can't be added to the environment without name.");
 		return FALSE;
@@ -301,61 +217,38 @@ osync_bool osync_group_env_add_group(OSyncGroupEnv *env, OSyncGroup *group, OSyn
 	}
 	
 	if (!osync_group_get_configdir(group)) {
-		char *configdir = g_strdup_printf("%s%cgroup%lli", env->groupsdir, G_DIR_SEPARATOR, _osync_group_env_create_group_id(env));
+		char *configdir = osync_strdup_printf("%s%cgroup%lli", env->groupsdir, G_DIR_SEPARATOR, osync_group_env_create_group_id(env));
 		osync_group_set_configdir(group, configdir);
-		g_free(configdir);
+		osync_free(configdir);
 	}
 	
-	env->groups = g_list_append(env->groups, group);
+	env->groups = osync_list_append(env->groups, group);
 	osync_group_ref(group);
 
 	return TRUE;
 }
 
-/*! @brief Removes the given group from the enviroment
- * 
- * Removes the given group from the environment
- * 
- * @param env Pointer to a OSyncGroupEnv environment
- * @param group The group to add
- * 
- */
 void osync_group_env_remove_group(OSyncGroupEnv *env, OSyncGroup *group)
 {
 	osync_assert(env);
 	osync_assert(group);
 	
-	env->groups = g_list_remove(env->groups, group);
+	env->groups = osync_list_remove(env->groups, group);
 	osync_group_unref(group);
 }
 
-/*! @brief Counts the groups in the environment
- * 
- * Returns the number of groups
- * 
- * @param env Pointer to a OSyncGroupEnv environment
- * @returns Number of groups
- * 
- */
 int osync_group_env_num_groups(OSyncGroupEnv *env)
 {
 	osync_assert(env);
-	return g_list_length(env->groups);
+	return osync_list_length(env->groups);
 }
 
-/*! @brief Returns the nth group
- * 
- * Returns the nth groups from the environment
- * 
- * @param env Pointer to a OSyncGroupEnv environment
- * @param nth Which group to return
- * @returns Pointer to the group
- * 
- */
 OSyncGroup *osync_group_env_nth_group(OSyncGroupEnv *env, int nth)
 {
 	osync_assert(env);
-	return (OSyncGroup *)g_list_nth_data(env->groups, nth);
+	return (OSyncGroup *)osync_list_nth_data(env->groups, nth);
 }
 
-/*@}*/
+OSyncList *osync_group_env_get_groups(OSyncGroupEnv *env) {
+	return osync_list_copy(env->groups);
+}
