@@ -1,6 +1,6 @@
 /*
  * libopensync - A synchronization framework
- * Copyright (C) 2008 Daniel Gollub <dgollub@suse.de> 
+ * Copyright (C) 2008 Daniel Gollub <gollub@b1-systems.de> 
  * 
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -18,28 +18,34 @@
  * 
  */
 
+#include "config.h"
+
+#include <libxml/xmlIO.h>
+#include <libxml/parser.h>
+#include <libxml/xmlmemory.h>
+
+#include <libxslt/xslt.h>
+#include <libxslt/transform.h>
+#include <libxslt/xsltutils.h>
+#include <libxslt/xsltInternals.h>
+
+#include <libexslt/exslt.h>
+
 #include "opensync.h"
 #include "opensync_internals.h"
 
 #include "opensync-group.h"
 
+#include "common/opensync_xml_internals.h"
+#include "common/opensync_file.h"
+#include "common/opensync_file_internals.h"
+
 #include "opensync_updater.h"
+#include "opensync_updater_private.h"
 #include "opensync_updater_internals.h"
 
-/**
- * @defgroup OSyncPrivateUpdaterAPI OpenSync Updater 
- * @ingroup OSyncPrivate
- * @brief OpenSync update facilities
- * 
- */
-/*@{*/
+#include "opensync_group_internals.h"
 
-/*! @brief Stacks errors on top of the Updater error stack.
- *
- * @param updater Pointer to the OSyncUpdater 
- * @param error Pointer to OSyncError which get stacked
- * @returns return code of thread as pointer 
- */
 static void osync_updater_set_error(OSyncUpdater *updater, OSyncError *error)
 {
 	osync_assert(updater);
@@ -53,44 +59,36 @@ static void osync_updater_set_error(OSyncUpdater *updater, OSyncError *error)
 		osync_error_ref(&error);
 }
 
-/*! @brief Apply stylesheet on configuration.
- *
- * @param updater Pointer to the OSyncUpdater 
- * @param config Path to configuration which gets processed 
- * @param stylesheet Path to stylesheet which gets invoked 
- * @param error Pointer to OSyncError which get stacked
- * @returns TRUE on success, FALSE on error. 
- */
 static osync_bool osync_updater_stylesheet_process(OSyncUpdater *updater, const char *config, const char *stylesheet, OSyncError **error)
 {
+	xmlDocPtr result = NULL, doc = NULL, style = NULL;
+	xsltTransformContextPtr ctxt = NULL;
+	xsltStylesheetPtr cur = NULL;
+
 	osync_assert(updater);
 	osync_assert(config);
 	osync_assert(stylesheet);
 	osync_trace(TRACE_ENTRY, "%s(%p, %s, %s, %p)", __func__, updater, config, stylesheet, error);
 
-	xmlDocPtr result = NULL, doc = NULL, style = NULL;
-	xsltTransformContextPtr ctxt = NULL;
-	xsltStylesheetPtr cur = NULL;
-
 	/* Register EXSLT extension.
-	   Required for exslt:node-set($rtf) */
+		 Required for exslt:node-set($rtf) */
 	exsltRegisterAll();
 
 	style = xmlReadFile(stylesheet, NULL, XSLT_PARSE_OPTIONS);
 	if (!style) {
-		osync_error_set(error, OSYNC_ERROR_GENERIC, "Couldn't read update stylesheet file: %s", stylesheet);
+		osync_error_set(error, OSYNC_ERROR_GENERIC, "Could not read update stylesheet file: %s", stylesheet);
 		goto error;
 	}
 
 	doc = xmlReadFile(config, NULL, XSLT_PARSE_OPTIONS);
 	if (!doc) {
-		osync_error_set(error, OSYNC_ERROR_GENERIC, "Couldn't read update stylesheet file: %s", config);
+		osync_error_set(error, OSYNC_ERROR_GENERIC, "Could not read update stylesheet file: %s", config);
 		goto error;
 	}
 
 	cur = xsltParseStylesheetDoc(style);
 	if (!cur || cur->errors) {
-		osync_error_set(error, OSYNC_ERROR_GENERIC, "Couldn't load update stylesheet.");
+		osync_error_set(error, OSYNC_ERROR_GENERIC, "Could not load update stylesheet.");
 		goto error;
 	}
 
@@ -101,7 +99,7 @@ static osync_bool osync_updater_stylesheet_process(OSyncUpdater *updater, const
 	}
 
 	result = xsltApplyStylesheetUser(cur, doc, NULL, NULL, NULL, ctxt);
-	xmlFreeDoc(doc);
+	osync_xml_free_doc(doc);
 
 	if (!result || ctxt->state != XSLT_STATE_OK) {
 		osync_error_set(error, OSYNC_ERROR_GENERIC, "Applying the stylesheet failed.");
@@ -110,29 +108,29 @@ static osync_bool osync_updater_stylesheet_process(OSyncUpdater *updater, const
 
 	xsltSaveResultToFilename(config, result, cur, 0);
 
-	xmlFreeDoc(result);
+	osync_xml_free_doc(result);
 	xsltFreeStylesheet(cur);
 
 	/* FIXME: Review; This got freed by xsltFreeStylesheet?!
-	xmlFreeDoc(style);
+		 osync_xml_free_doc(style);
 	*/
 
 	xsltFreeTransformContext(ctxt);
 
 	osync_trace(TRACE_EXIT, "%s", __func__);
 	return TRUE;
-error:
+ error:
 
 	if (cur)
 		xsltFreeStylesheet(cur);
 
 	/* FIXME: Review; This got freed by xsltFreeStylesheet?!
-	if (style)
-		xmlFreeDoc(style);
+		 if (style)
+		 osync_xml_free_doc(style);
 	*/
 
 	if (result)
-		xmlFreeDoc(result);
+		osync_xml_free_doc(result);
 
 	if (ctxt)
 		xsltFreeTransformContext(ctxt);
@@ -141,54 +139,48 @@ error:
 	return FALSE;
 }
 
-/*! @brief Process update on plugin configuration of a certain member.
- *
- * @param updater Pointer to the OSyncUpdater 
- * @param member Pointer to OSyncMember who gets updated
- * @param error Pointer to OSyncError which get stacked
- * @returns TRUE on success, FALSE on error. 
- */
 static osync_bool osync_updater_process_plugin_config(OSyncUpdater *updater, OSyncMember *member, OSyncError **error)
 {
-	osync_assert(updater);
-	osync_assert(member);
-	osync_trace(TRACE_ENTRY, "%s(%p, %p, %p)", __func__, updater, member, error);
 	const char *configdir;
 	char *plugin_config = NULL;
 	char *update_stylesheet = NULL;
 
+	osync_assert(updater);
+	osync_assert(member);
+	osync_trace(TRACE_ENTRY, "%s(%p, %p, %p)", __func__, updater, member, error);
+
 	/* Member configuration directory */
 	configdir = osync_member_get_configdir(member);
 	if (!configdir) {
-		osync_error_set(error, OSYNC_ERROR_GENERIC, "Couldn't find member's configuration directory.");
+		osync_error_set(error, OSYNC_ERROR_GENERIC, "Could not find member configuration directory.");
 		goto error;
 	}
 
-	plugin_config = g_strdup_printf("%s%c%s.conf", 
-			configdir, G_DIR_SEPARATOR,
-			osync_member_get_pluginname(member));
+	plugin_config = osync_strdup_printf("%s%c%s.conf", 
+	                                    configdir, G_DIR_SEPARATOR,
+	                                    osync_member_get_pluginname(member));
 
 	if (!plugin_config) {
-		osync_error_set(error, OSYNC_ERROR_GENERIC, "Couldn't get enough memory to build plugin configuration path.");
+		osync_error_set(error, OSYNC_ERROR_GENERIC, "Could not get enough memory to build plugin configuration path.");
 		goto error;
 	}
 
 	/* Plugin configuration is optional. So tolerate if none is present.
-	   If so - nothing to do. Skip the update process. */
+		 If so - nothing to do. Skip the update process. */
 	if (!g_file_test(plugin_config, G_FILE_TEST_IS_REGULAR))
 		goto end;
 
-	update_stylesheet = g_strdup_printf("%s%c%s-%u%s", updater->updatesdir, 
-			G_DIR_SEPARATOR, osync_member_get_pluginname(member),
-			updater->plugin_version, OSYNC_UPDATER_SUFFIX);
+	update_stylesheet = osync_strdup_printf("%s%c%s-%u%s", updater->updatesdir, 
+	                                        G_DIR_SEPARATOR, osync_member_get_pluginname(member),
+	                                        updater->plugin_version, OSYNC_UPDATER_SUFFIX);
 
 	if (!update_stylesheet) {
-		osync_error_set(error, OSYNC_ERROR_GENERIC, "Couldn't get enough memory to build plugin update stylesheet path.");
+		osync_error_set(error, OSYNC_ERROR_GENERIC, "Could not get enough memory to build plugin update stylesheet path.");
 		goto error;
 	}
 
 	if (!g_file_test(update_stylesheet, G_FILE_TEST_IS_REGULAR)) {
-		osync_error_set(error, OSYNC_ERROR_GENERIC, "Couldn't find update stylesheet for \"%s\" plugin in directory: %s", osync_member_get_pluginname(member), updater->updatesdir);
+		osync_error_set(error, OSYNC_ERROR_GENERIC, "Could not find update stylesheet for \"%s\" plugin in directory: %s", osync_member_get_pluginname(member), updater->updatesdir);
 		goto error;
 	}
 
@@ -196,69 +188,63 @@ static osync_bool osync_updater_process_plugin_config(OSyncUpdater *updater, OSy
 	if (!osync_updater_stylesheet_process(updater, plugin_config, update_stylesheet, error))
 		goto error;
 
-	g_free(update_stylesheet);
-end:
-	g_free(plugin_config);
+	osync_free(update_stylesheet);
+ end:
+	osync_free(plugin_config);
 
 	osync_trace(TRACE_EXIT, "%s", __func__);
 	return TRUE;
-error:	
+ error:	
 	if (plugin_config)
-		g_free(plugin_config);
+		osync_free(plugin_config);
 
 	if (update_stylesheet)
-		g_free(update_stylesheet);
+		osync_free(update_stylesheet);
 
 	osync_trace(TRACE_EXIT_ERROR, "%s: %s", __func__, osync_error_print(error));
 	return FALSE;
 }
 
 
-/*! @brief Process update on member configuration.
- *
- * @param updater Pointer to the OSyncUpdater 
- * @param member Pointer to OSyncMember who gets updated
- * @param error Pointer to OSyncError which get stacked
- * @returns TRUE on success, FALSE on error. 
- */
 static osync_bool osync_updater_process_member_config(OSyncUpdater *updater, OSyncMember *member, OSyncError **error)
 {
-	osync_assert(updater);
-	osync_assert(member);
-	osync_trace(TRACE_ENTRY, "%s(%p, %p, %p)", __func__, updater, member, error);
 	const char *configdir;
 	char *member_config = NULL;
 	char *update_stylesheet = NULL;
 
+	osync_assert(updater);
+	osync_assert(member);
+	osync_trace(TRACE_ENTRY, "%s(%p, %p, %p)", __func__, updater, member, error);
+
 	/* Member configuration directory */
 	configdir = osync_member_get_configdir(member);
 	if (!configdir) {
-		osync_error_set(error, OSYNC_ERROR_GENERIC, "Couldn't find member's configuration directory.");
+		osync_error_set(error, OSYNC_ERROR_GENERIC, "Couldnot find member configuration directory.");
 		goto error;
 	}
 
-	member_config = g_strdup_printf("%s%csyncmember.conf", configdir, G_DIR_SEPARATOR);
+	member_config = osync_strdup_printf("%s%csyncmember.conf", configdir, G_DIR_SEPARATOR);
 	if (!member_config) {
-		osync_error_set(error, OSYNC_ERROR_GENERIC, "Couldn't get enough memory to build member configuration path.");
+		osync_error_set(error, OSYNC_ERROR_GENERIC, "Could not get enough memory to build member configuration path.");
 		goto error;
 	}
 
 	if (!g_file_test(member_config, G_FILE_TEST_IS_REGULAR)) {
-		osync_error_set(error, OSYNC_ERROR_GENERIC, "Couldn't find member's configuration file: %s", member_config);
+		osync_error_set(error, OSYNC_ERROR_GENERIC, "Could not find member configuration file %s", member_config);
 		goto error;
 	}
 
-	update_stylesheet = g_strdup_printf("%s%c%s-%u%s", updater->updatesdir, 
-			G_DIR_SEPARATOR, "syncmember",
-			updater->member_version, OSYNC_UPDATER_SUFFIX);
+	update_stylesheet = osync_strdup_printf("%s%c%s-%u%s", updater->updatesdir, 
+	                                        G_DIR_SEPARATOR, "syncmember",
+	                                        updater->member_version, OSYNC_UPDATER_SUFFIX);
 
 	if (!update_stylesheet) {
-		osync_error_set(error, OSYNC_ERROR_GENERIC, "Couldn't get enough memory to build member configuration update stylesheet path.");
+		osync_error_set(error, OSYNC_ERROR_GENERIC, "Could not get enough memory to build member configuration update stylesheet path.");
 		goto error;
 	}
 
 	if (!g_file_test(update_stylesheet, G_FILE_TEST_IS_REGULAR)) {
-		osync_error_set(error, OSYNC_ERROR_GENERIC, "Couldn't find update stylesheet for member configuratoin in directory: %s", updater->updatesdir);
+		osync_error_set(error, OSYNC_ERROR_GENERIC, "Could not find update stylesheet for member configuration in directory: %s", updater->updatesdir);
 		goto error;
 	}
 
@@ -266,36 +252,32 @@ static osync_bool osync_updater_process_member_config(OSyncUpdater *updater, OSy
 	if (!osync_updater_stylesheet_process(updater, member_config, update_stylesheet, error))
 		goto error;
 
-	g_free(member_config);
-	g_free(update_stylesheet);
+	osync_free(member_config);
+	osync_free(update_stylesheet);
 
 	osync_trace(TRACE_EXIT, "%s", __func__);
 	return TRUE;
-error:	
+ error:	
 	if (member_config)
-		g_free(member_config);
+		osync_free(member_config);
 
 	if (update_stylesheet)
-		g_free(update_stylesheet);
+		osync_free(update_stylesheet);
 
 	osync_trace(TRACE_EXIT_ERROR, "%s: %s", __func__, osync_error_print(error));
 	return FALSE;
 }
 
-/*! @brief Process update on member. 
- *
- * @param updater Pointer to the OSyncUpdater 
- * @param nthmember Nth member in the group, who gets updated
- * @param error Pointer to OSyncError which get stacked
- * @returns TRUE on success, FALSE on error. 
- */
-static osync_bool osync_updater_process_member(OSyncUpdater *updater, int nthmember, OSyncError **error)
+static osync_bool osync_updater_process_member(OSyncUpdater *updater, unsigned int nthmember, OSyncError **error)
 {
+	OSyncMember *member = NULL;
 	osync_assert(updater);
 	osync_trace(TRACE_ENTRY, "%s(%p, %u, %p)", __func__, updater, nthmember, error);
-	OSyncMember *member = osync_group_nth_member(updater->group, nthmember);
+
+	osync_trace(TRACE_INTERNAL, "%s: %s", __func__, osync_error_print(error));
+	member = osync_group_nth_member(updater->group, nthmember);
 	if (!member) {
-		osync_error_set(error, OSYNC_ERROR_GENERIC, "Couldn't find #%i member in this group.", nthmember);
+		osync_error_set(error, OSYNC_ERROR_GENERIC, "Could not find #%i member in this group.", nthmember);
 		goto error;
 	}
 
@@ -309,55 +291,49 @@ static osync_bool osync_updater_process_member(OSyncUpdater *updater, int nthmem
 
 	osync_trace(TRACE_EXIT, "%s", __func__);
 	return TRUE;
-error:	
+ error:	
 	osync_trace(TRACE_EXIT_ERROR, "%s: %s", __func__, osync_error_print(error));
 	return FALSE;
 }
 
-/*! @brief Process update on group configuration.
- *
- * @param updater Pointer to the OSyncUpdater 
- * @param error Pointer to OSyncError which get stacked
- * @returns TRUE on success, FALSE on error. 
- */
 static osync_bool osync_updater_process_group(OSyncUpdater *updater, OSyncError **error)
 {
-	osync_assert(updater);
-	osync_trace(TRACE_ENTRY, "%s(%p, %p)", __func__, updater, error);
-
 	const char *configdir;
 	char *group_config = NULL;
 	char *update_stylesheet = NULL;
 
+	osync_assert(updater);
+	osync_trace(TRACE_ENTRY, "%s(%p, %p)", __func__, updater, error);
+
 	/* Group configuration directory */
 	configdir = osync_group_get_configdir(updater->group);
 	if (!configdir) {
-		osync_error_set(error, OSYNC_ERROR_GENERIC, "Couldn't find group configuration directory.");
+		osync_error_set(error, OSYNC_ERROR_GENERIC, "Could not find group configuration directory.");
 		goto error;
 	}
 
-	group_config = g_strdup_printf("%s%c%s", configdir, G_DIR_SEPARATOR, "syncgroup.conf");
+	group_config = osync_strdup_printf("%s%c%s", configdir, G_DIR_SEPARATOR, "syncgroup.conf");
 	if (!group_config) {
-		osync_error_set(error, OSYNC_ERROR_GENERIC, "Couldn't get enough memory to build group configuration path.");
+		osync_error_set(error, OSYNC_ERROR_GENERIC, "Could not get enough memory to build group configuration path.");
 		goto error;
 	}
 
 	if (!g_file_test(group_config, G_FILE_TEST_IS_REGULAR)) {
-		osync_error_set(error, OSYNC_ERROR_GENERIC, "Couldn't find group configuration file: %s", group_config);
+		osync_error_set(error, OSYNC_ERROR_GENERIC, "Could not find group configuration file: %s", group_config);
 		goto error;
 	}
 
-	update_stylesheet = g_strdup_printf("%s%c%s-%u%s", updater->updatesdir, 
-			G_DIR_SEPARATOR, "syncgroup",
-			updater->group_version, OSYNC_UPDATER_SUFFIX);
+	update_stylesheet = osync_strdup_printf("%s%c%s-%u%s", updater->updatesdir, 
+	                                        G_DIR_SEPARATOR, "syncgroup",
+	                                        updater->group_version, OSYNC_UPDATER_SUFFIX);
 
 	if (!update_stylesheet) {
-		osync_error_set(error, OSYNC_ERROR_GENERIC, "Couldn't get enough memory to build group update stylesheet path.");
+		osync_error_set(error, OSYNC_ERROR_GENERIC, "Could not get enough memory to build group update stylesheet path.");
 		goto error;
 	}
 
 	if (!g_file_test(update_stylesheet, G_FILE_TEST_IS_REGULAR)) {
-		osync_error_set(error, OSYNC_ERROR_GENERIC, "Couldn't find update stylesheet for group configuration in directory: %s", updater->updatesdir);
+		osync_error_set(error, OSYNC_ERROR_GENERIC, "Could not find update stylesheet for group configuration in directory: %s", updater->updatesdir);
 		goto error;
 	}
 
@@ -365,85 +341,78 @@ static osync_bool osync_updater_process_group(OSyncUpdater *updater, OSyncError
 	if (!osync_updater_stylesheet_process(updater, group_config, update_stylesheet, error))
 		goto error;
 
-	g_free(group_config);
-	g_free(update_stylesheet);
+	osync_free(group_config);
+	osync_free(update_stylesheet);
 
 
 	osync_trace(TRACE_EXIT, "%s", __func__);
 	return TRUE;
-error:
+ error:
 
 	if (group_config)
-		g_free(group_config);
+		osync_free(group_config);
 
 	if (update_stylesheet)
-		g_free(update_stylesheet);
+		osync_free(update_stylesheet);
 
 	osync_trace(TRACE_EXIT_ERROR, "%s: %s", __func__, osync_error_print(error));
 	return FALSE;
 }
 
-/*! @brief Create group backup.
- *
- * The entire group directory got moved to to groupN.bak.
- * If already backups present further .bak prefixes got appended.
- *
- * @param updater Pointer to the OSyncUpdater 
- * @param error Pointer to OSyncError which get stacked
- * @returns Path of the backup, NULL on error
- */
 static char *osync_updater_create_backup(OSyncUpdater *updater, OSyncError **error)
 {
-	osync_assert(updater);
-	osync_trace(TRACE_ENTRY, "%s(%p, %p)", __func__, updater, error);
-
 	GDir *dir = NULL, *member_dir = NULL;
 	GError *gerror = NULL;
 	const char *de = NULL, *member_de = NULL;
 	char *backup_groupdir = NULL;
 	struct stat orig_stat;
+	const char *orig_groupdir = NULL;
+	char *config, *copy_config, *content, *member_path;
+	gsize length;
 
-	const char *orig_groupdir = osync_group_get_configdir(updater->group);
 
-	backup_groupdir = g_strdup_printf("%s.bak", orig_groupdir);
+	osync_assert(updater);
+	osync_trace(TRACE_ENTRY, "%s(%p, %p)", __func__, updater, error);
+
+	orig_groupdir = osync_group_get_configdir(updater->group);
 
 	/* Sanity check if group got loaded and has a configdir set */
 	if (!orig_groupdir) {
-		osync_error_set(error, OSYNC_ERROR_GENERIC, "No group configuration directory set. Couldn't create Backup. Aborting.");
+		osync_error_set(error, OSYNC_ERROR_GENERIC, "No group configuration directory set. Could not create Backup. Aborting.");
 		goto error;
 	}
 
+	backup_groupdir = osync_strdup_printf("%s.bak", orig_groupdir);
+
 	/* #1 Move original group directory */
 	while (g_file_test(backup_groupdir, G_FILE_TEST_IS_DIR)) {
 		char *tmp = backup_groupdir;
-		backup_groupdir = g_strdup_printf("%s.bak", tmp);
-		g_free(tmp);
+		backup_groupdir = osync_strdup_printf("%s.bak", tmp);
+		osync_free(tmp);
 	}
 
 	if (g_stat(orig_groupdir, &orig_stat) < 0) {
-		g_set_error(&gerror, G_FILE_ERROR, errno, NULL);
-		osync_error_set(error, OSYNC_ERROR_GENERIC, "Couldn't get information about group directory: %s",  gerror->message);
+		g_set_error(&gerror, G_FILE_ERROR, g_file_error_from_errno(errno), "%s", orig_groupdir);
+		osync_error_set(error, OSYNC_ERROR_GENERIC, "Could not get information about group directory: %s", gerror->message);
 		g_error_free(gerror);
 		goto error;
 	}
 
 	if (g_rename(orig_groupdir, backup_groupdir) < 0) {
-		g_set_error(&gerror, G_FILE_ERROR, errno, NULL);
-		osync_error_set(error, OSYNC_ERROR_GENERIC, "Couldn't backup group directory: %s",  gerror->message);
+		g_set_error(&gerror, G_FILE_ERROR, g_file_error_from_errno(errno), "%s", orig_groupdir);
+		osync_error_set(error, OSYNC_ERROR_GENERIC, "Could not backup group directory: %s", gerror->message);
 		g_error_free(gerror);
 		goto error;
 	}
 
 	if (g_mkdir(orig_groupdir, orig_stat.st_mode) < 0) { 
-		g_set_error(&gerror, G_FILE_ERROR, errno, NULL);
-		osync_error_set(error, OSYNC_ERROR_GENERIC, "Couldn't create new group directory: %s",  gerror->message);
+		g_set_error(&gerror, G_FILE_ERROR, g_file_error_from_errno(errno), "%s", orig_groupdir);
+		osync_error_set(error, OSYNC_ERROR_GENERIC, "Could not create new group directory: %s", gerror->message);
 		g_error_free(gerror);
 		goto error;
 	}
 
 	/* #2 Copy group and member configurations */
-	char *config, *copy_config, *content, *member_path;
-	gsize length;
 	dir = g_dir_open(backup_groupdir, 0, &gerror);
 	if (!dir) {
 		osync_error_set(error, OSYNC_ERROR_IO_ERROR, "Unable to open group configdir %s", gerror->message);
@@ -452,22 +421,22 @@ static char *osync_updater_create_backup(OSyncUpdater *updater, OSyncError **err
 	}
 
 	while ((de = g_dir_read_name(dir))) {
-		member_path = g_strdup_printf("%s%c%s%c", backup_groupdir, G_DIR_SEPARATOR, de, G_DIR_SEPARATOR);
+		member_path = osync_strdup_printf("%s%c%s%c", backup_groupdir, G_DIR_SEPARATOR, de, G_DIR_SEPARATOR);
 		if (!g_file_test(member_path, G_FILE_TEST_IS_DIR)) {
-			g_free(member_path);
+			osync_free(member_path);
 			continue;
 		}
 
 		if (g_stat(member_path, &orig_stat) < 0) {
-			g_set_error(&gerror, G_FILE_ERROR, errno, NULL);
-			osync_error_set(error, OSYNC_ERROR_GENERIC, "Couldn't get information about member directory: %s",  gerror->message);
+			g_set_error(&gerror, G_FILE_ERROR, g_file_error_from_errno(errno), "%s", member_path);
+			osync_error_set(error, OSYNC_ERROR_GENERIC, "Could not get information about member directory: %s", gerror->message);
 			g_error_free(gerror);
-			g_free(member_path);
+			osync_free(member_path);
 			goto error;
 		}
 
 		member_dir = g_dir_open(member_path, 0, &gerror);
-		g_free(member_path);
+		osync_free(member_path);
 
 		if (!member_dir) {
 			osync_error_set(error, OSYNC_ERROR_IO_ERROR, "Unable to open member configdir %s", gerror->message);
@@ -475,43 +444,44 @@ static char *osync_updater_create_backup(OSyncUpdater *updater, OSyncError **err
 			goto error;
 		}
 
-		member_path = g_strdup_printf("%s%c%s%c", orig_groupdir, G_DIR_SEPARATOR, de, G_DIR_SEPARATOR);
+		member_path = osync_strdup_printf("%s%c%s%c", orig_groupdir, G_DIR_SEPARATOR, de, G_DIR_SEPARATOR);
 		if (g_mkdir(member_path, orig_stat.st_mode) < 0) { 
-			g_set_error(&gerror, G_FILE_ERROR, errno, NULL);
-			osync_error_set(error, OSYNC_ERROR_GENERIC, "Couldn't create new member directory: %s",  gerror->message);
+			g_set_error(&gerror, G_FILE_ERROR, g_file_error_from_errno(errno), "%s", member_path);
+			osync_error_set(error, OSYNC_ERROR_GENERIC, "Could not create new member directory: %s", gerror->message);
 			g_error_free(gerror);
-			g_free(member_path);
+			osync_free(member_path);
 			goto error;
 		}
-		g_free(member_path);
+		osync_free(member_path);
 
 		while ((member_de = g_dir_read_name(member_dir))) {
-			config = g_strdup_printf("%s%c%s%c%s", backup_groupdir, G_DIR_SEPARATOR, de, G_DIR_SEPARATOR, member_de);
+			config = osync_strdup_printf("%s%c%s%c%s", backup_groupdir, G_DIR_SEPARATOR, de, G_DIR_SEPARATOR, member_de);
 			
 			/* Only copy files ending with ".conf" */
-			if (!g_file_test(config, G_FILE_TEST_IS_REGULAR) || !sscanf(config, "%*s.conf")) {
-				g_free(config);
+			//			if (!g_file_test(config, G_FILE_TEST_IS_REGULAR) || !sscanf(config, "%*s.conf")) {
+			if (!g_file_test(config, G_FILE_TEST_IS_REGULAR) || !g_pattern_match_simple("*.conf", config)) {
+				osync_free(config);
 				continue;
 			}
 
 			if (!g_file_get_contents(config, &content, &length, &gerror)) {
 				osync_error_set(error, OSYNC_ERROR_IO_ERROR, "Failed reading configfile: %s", gerror->message);
 				g_error_free(gerror);
-				g_free(config);
+				osync_free(config);
 				goto error;
 			}
-			g_free(config);
+			osync_free(config);
 
-			copy_config = g_strdup_printf("%s%c%s%c%s", orig_groupdir, G_DIR_SEPARATOR, de, G_DIR_SEPARATOR, member_de);
+			copy_config = osync_strdup_printf("%s%c%s%c%s", orig_groupdir, G_DIR_SEPARATOR, de, G_DIR_SEPARATOR, member_de);
 
 			if (!g_file_set_contents(copy_config, content, length, &gerror)) {
 				osync_error_set(error, OSYNC_ERROR_IO_ERROR, "Failed writing configfile: %s", gerror->message);
 				g_error_free(gerror);
-				g_free(copy_config);
+				osync_free(copy_config);
 				g_free(content);
 				goto error;
 			}
-			g_free(copy_config);
+			osync_free(copy_config);
 			g_free(content);
 
 		}
@@ -522,36 +492,36 @@ static char *osync_updater_create_backup(OSyncUpdater *updater, OSyncError **err
 	g_dir_close(dir);
 
 
-	config = g_strdup_printf("%s%c%s", backup_groupdir, G_DIR_SEPARATOR, "syncgroup.conf");
+	config = osync_strdup_printf("%s%c%s", backup_groupdir, G_DIR_SEPARATOR, "syncgroup.conf");
 
 	if (!g_file_get_contents(config, &content, &length, &gerror)) {
 		osync_error_set(error, OSYNC_ERROR_IO_ERROR, "Failed reading group configfile: %s", gerror->message);
 		g_error_free(gerror);
-		g_free(config);
+		osync_free(config);
 		goto error;
 	}
-	g_free(config);
+	osync_free(config);
 
-	copy_config = g_strdup_printf("%s%c%s", orig_groupdir, G_DIR_SEPARATOR, "syncgroup.conf");
+	copy_config = osync_strdup_printf("%s%c%s", orig_groupdir, G_DIR_SEPARATOR, "syncgroup.conf");
 
 	if (!g_file_set_contents(copy_config, content, length, &gerror)) {
 		osync_error_set(error, OSYNC_ERROR_IO_ERROR, "Failed writing group configfile: %s", gerror->message);
 		g_error_free(gerror);
-		g_free(copy_config);
+		osync_free(copy_config);
 		g_free(content);
 		goto error;
 	}
-	g_free(copy_config);
+	osync_free(copy_config);
 	g_free(content);
 
 	osync_trace(TRACE_EXIT, "%s: %s", __func__, backup_groupdir);
 	return backup_groupdir;
-error:
+ error:
 	if (member_dir)
 		g_dir_close(member_dir);
 
 	if (backup_groupdir)
-		g_free(backup_groupdir);
+		osync_free(backup_groupdir);
 
 	if (dir)
 		g_dir_close(dir);
@@ -561,85 +531,60 @@ error:
 
 }
 
-/*! @brief Restore group backup.
- *
- * This function is intended to get called if update process failed.
- *
- * backup_pathgets used as backup directory, if set. Otherwise it restores the 
- * latest backup of the group, if several backups are present.
- *
- * @param updater Pointer to the OSyncUpdater 
- * @param backup_path Path to prefered backup
- * @returns TRUE on success, FALSE on error. 
- */
 static osync_bool osync_updater_restore_backup(OSyncUpdater *updater, const char *backup_path)
 {
-	osync_assert(updater);
-	osync_trace(TRACE_ENTRY, "%s(%p)", __func__, updater);
-
 	OSyncError *error = NULL;
 	GError *gerror = NULL;
 	char *backup_groupdir = NULL;
-	const char *groupdir = osync_group_get_configdir(updater->group);
+	const char *groupdir = NULL;
 
+	osync_assert(updater);
+	osync_trace(TRACE_ENTRY, "%s(%p)", __func__, updater);
+
+	groupdir = osync_group_get_configdir(updater->group);
 	/* Sanity check if group got loaded and a configdir is present */
 	if (!groupdir) {
-		osync_error_set(&error, OSYNC_ERROR_GENERIC, "No group configuration directory set. Couldn't restore Backup.");
+		osync_error_set(&error, OSYNC_ERROR_GENERIC, "No group configuration directory set. Could not restore Backup.");
 		goto error;
 	}
 
-	backup_groupdir = backup_path ? g_strdup(backup_path) : g_strdup_printf("%s.bak", groupdir);
+	backup_groupdir = backup_path ? osync_strdup(backup_path) : osync_strdup_printf("%s.bak", groupdir);
 
 	/* Search for latest backup if no certain backup_path was given.
 
-	   The latest backups might look like this:
-	   ~/.opensync/group1.bak.bak.bak.bak.bak.bak.bak */
+		 The latest backups might look like this:
+		 ~/.opensync/group1.bak.bak.bak.bak.bak.bak.bak */
 	while (!backup_path && !g_file_test(backup_groupdir, G_FILE_TEST_IS_DIR)) {
 		char *tmp = backup_groupdir;
-		backup_groupdir = g_strdup_printf("%s.bak", tmp);
-		g_free(tmp);
+		backup_groupdir = osync_strdup_printf("%s.bak", tmp);
+		osync_free(tmp);
 	}
-
-	if (g_unlink(groupdir) < 0) {
-		g_set_error(&gerror, G_FILE_ERROR, errno, NULL);
-		osync_error_set(&error, OSYNC_ERROR_GENERIC, "Couldn't remove current group directory: %s",  gerror->message);
+	if (osync_remove_directory_recursively(groupdir) < 0) {
+		g_set_error(&gerror, G_FILE_ERROR, g_file_error_from_errno(errno), "%s", groupdir);
+		osync_error_set(&error, OSYNC_ERROR_GENERIC, "Could not remove current group directory: %s", gerror->message);
 		g_error_free(gerror);
 		goto error;
 	}
 
 	if (g_rename(backup_groupdir, groupdir) < 0) {
-		g_set_error(&gerror, G_FILE_ERROR, errno, NULL);
-		osync_error_set(&error, OSYNC_ERROR_GENERIC, "Couldn't restore backup group directory: %s",  gerror->message);
+		g_set_error(&gerror, G_FILE_ERROR, g_file_error_from_errno(errno), "%s", backup_groupdir);
+		osync_error_set(&error, OSYNC_ERROR_GENERIC, "Could not restore backup group directory: %s", gerror->message);
 		g_error_free(gerror);
 		goto error;
 	}
 
-	g_free(backup_groupdir);
+	osync_free(backup_groupdir);
 
 	osync_trace(TRACE_EXIT, "%s", __func__);
 	return TRUE;
-error:
-	g_free(backup_groupdir);
+ error:
+	osync_free(backup_groupdir);
 	osync_trace(TRACE_EXIT_ERROR, "%s: %s", __func__, osync_error_print(&error));
 	osync_updater_set_error(updater, error);
 	return FALSE;
 
 }
 
-/*! @brief Updater thread function 
- *
- *  #1 Lock group
- *  #2 Check version of group configuration
- *  #3 Create group backup
- *  #4 Process Member configurations
- *  #5 Process Group configuration
- *  #6 Unlock group
- *
- *  On any error (expect locked group) restore backup and unlock group.
- *
- * @param userdata Pointer to the OSyncUpdater 
- * @returns return code of thread as pointer 
- */
 static void *osync_updater_run(void *userdata)
 {
 	OSyncUpdater *updater = (OSyncUpdater *) userdata;
@@ -650,8 +595,8 @@ static void *osync_updater_run(void *userdata)
 	osync_trace(TRACE_ENTRY, "%s(%p)", __func__, userdata);
 
 	/* #1 Lock group */
-	if (osync_group_lock(updater->group) == OSYNC_LOCKED) {
-		osync_error_set(&error, OSYNC_ERROR_GENERIC, "Group is locked. Can't process update on this group.");
+	if (osync_group_lock(updater->group, &error) == OSYNC_LOCKED) {
+		osync_error_set(&error, OSYNC_ERROR_GENERIC, "Group is locked. Can not process update on this group.");
 		goto error;
 	}
 
@@ -685,20 +630,20 @@ static void *osync_updater_run(void *userdata)
 	g_cond_signal(updater->updating);
 	g_mutex_unlock(updater->updating_mutex);
 
-	g_free(backup_dir);
+	osync_free(backup_dir);
 
 	osync_trace(TRACE_EXIT, "%s", __func__);
 	osync_thread_exit(updater->thread, 0);
 	return 0;
 
-error_and_restore:
+ error_and_restore:
 	osync_updater_restore_backup(updater, backup_dir);
-error_and_unlock:
+ error_and_unlock:
 	osync_group_unlock(updater->group);
-error:
+ error:
 
 	if (backup_dir)
-		g_free(backup_dir);
+		osync_free(backup_dir);
 
 	/* TODO: Emit error signal */
 	osync_updater_set_error(updater, error);
@@ -713,68 +658,36 @@ error:
 	return 0;
 }
 
-/*! @brief Change version of group configuration. Only for testing.
- *
- * @param updater Pointer to the OSyncUpdater 
- * @param major Major Version number to set
- */
 void osync_updater_set_group_version(OSyncUpdater *updater, int major)
 {
 	osync_assert(updater);
 	updater->group_version = major;
 }
 
-/*! @brief Change version of member configuration. Only for testing.
- *
- * @param updater Pointer to the OSyncUpdater 
- * @param major Major Version number to set
- */
 void osync_updater_set_member_version(OSyncUpdater *updater, int major)
 {
 	osync_assert(updater);
 	updater->member_version = major;
 }
 
-/*! @brief Change version of plugin configuration. Only for testing.
- *
- * @param updater Pointer to the OSyncUpdater 
- * @param major Major Version number to set
- */
 void osync_updater_set_plugin_version(OSyncUpdater *updater, int major)
 {
 	osync_assert(updater);
 	updater->plugin_version = major;
 }
 
-/*@}*/
-
-/**
- * @defgroup OSyncUpdaterAPI OpenSync Updater 
- * @ingroup OSyncPublic
- * @brief OpenSync update facilities
- * 
- */
-/*@{*/
-
-
-/*! @brief Registers a new updater
- *
- * @param group Pointer to the OSyncGroup to update
- * @param error Pointer to an error struct
- * @returns the newly created updater
- */
 OSyncUpdater *osync_updater_new(OSyncGroup *group, OSyncError **error)
 {
+	OSyncUpdater *updater = osync_try_malloc0(sizeof(OSyncUpdater), error);
 	osync_assert(group);
 
-	OSyncUpdater *updater = osync_try_malloc0(sizeof(OSyncUpdater), error);
 	if (!updater)
 		return NULL;
 
 	updater->ref_count = 1;
 	updater->group = group;
 
-	updater->updatesdir = g_strdup(OPENSYNC_UPDATESDIR);
+	updater->updatesdir = osync_strdup(OPENSYNC_UPDATESDIR);
 
 	updater->updating_mutex = g_mutex_new();
 	updater->updating = g_cond_new();
@@ -786,11 +699,6 @@ OSyncUpdater *osync_updater_new(OSyncGroup *group, OSyncError **error)
 	return updater;
 }
 
-/*! @brief Increase the reference count on a plugin
- * 
- * @param updater Pointer to the plugin
- * 
- */
 OSyncUpdater *osync_updater_ref(OSyncUpdater *updater)
 {
 	osync_assert(updater);
@@ -800,11 +708,6 @@ OSyncUpdater *osync_updater_ref(OSyncUpdater *updater)
 	return updater;
 }
 
-/*! @brief Decrease the reference count on a updater 
- * 
- * @param updater Pointer to the updater 
- * 
- */
 void osync_updater_unref(OSyncUpdater *updater)
 {
 	osync_assert(updater);
@@ -818,37 +721,24 @@ void osync_updater_unref(OSyncUpdater *updater)
 			g_mutex_free(updater->updating_mutex);
 
 		if (updater->updatesdir)
-			g_free(updater->updatesdir);
+			osync_free(updater->updatesdir);
 		
-		g_free(updater);
+		osync_free(updater);
 	}
 }
 
-/*! @brief Register OSyncUpdater callback
- * 
- * @param updater Pointer to the updater 
- * @param callback The callback function which get called on updater events 
- * 
- */
 void osync_updater_set_callback(OSyncUpdater *updater, osync_updater_cb callback)
 {
 	osync_assert(updater);
 	updater->status_callback = callback;
 }
 
-/*! @brief Check if an update is required for the group
- * 
- * @param updater Pointer to the updater 
- * @returns TRUE if update is required on that group, FALSE if no update is required.
- * 
- */
 osync_bool osync_updater_action_required(OSyncUpdater *updater)
 {
+	int i, num_members;
 	osync_assert(updater);
 	osync_trace(TRACE_ENTRY, "%s(%p)", __func__, updater);
 
-	int i, num_members;
-
 	if (!osync_group_is_uptodate(updater->group)) {
 		osync_trace(TRACE_EXIT, "%s: The group requires action.", __func__);
 		return TRUE;
@@ -873,13 +763,6 @@ osync_bool osync_updater_action_required(OSyncUpdater *updater)
 	return FALSE;
 }
 
-/*! @brief Process update on the group. This function is not blocking. 
- * 
- * @param updater Pointer to the updater 
- * @param error Pointer to OSyncError
- * @returns TRUE if updating process started successfully, FALSE on error.
- * 
- */
 osync_bool osync_updater_process(OSyncUpdater *updater, OSyncError **error)
 {
 	osync_assert(updater);
@@ -894,19 +777,12 @@ osync_bool osync_updater_process(OSyncUpdater *updater, OSyncError **error)
 	osync_trace(TRACE_EXIT, "%s", __func__);
 	return TRUE;
 
-error:
+ error:
 	osync_trace(TRACE_EXIT_ERROR, "%s: %s", __func__, osync_error_print(error));
 	return FALSE;
 
 }
 
-/*! @brief Process update on the group. This function is blocking. 
- * 
- * @param updater Pointer to the updater 
- * @param error Pointer to OSyncError
- * @returns TRUE if updating process was successfully, FALSE on error.
- * 
- */
 osync_bool osync_updater_process_and_block(OSyncUpdater *updater, OSyncError **error)
 {
 	osync_assert(updater);
@@ -928,38 +804,24 @@ osync_bool osync_updater_process_and_block(OSyncUpdater *updater, OSyncError **e
 	osync_trace(TRACE_EXIT, "%s", __func__);
 	return TRUE;
 
-error:
+ error:
 	osync_trace(TRACE_EXIT_ERROR, "%s: %s", __func__, osync_error_print(error));
 	return FALSE;
 }
 
-/*! @brief Set path of Updates directory, which inlcudes the update stylesheets. 
- * 
- * @param updater Pointer to the updater 
- * @param path Path to updates directory 
- * 
- */
 void osync_updater_set_updates_directory(OSyncUpdater *updater, const char *path)
 {
 	osync_assert(updater);
 	osync_assert(path);
 
 	if (updater->updatesdir)
-		g_free(updater->updatesdir);
+		osync_free(updater->updatesdir);
 
-	updater->updatesdir = g_strdup(path);
+	updater->updatesdir = osync_strdup(path);
 }
 
-/*! @brief Get path of Updates directory. 
- * 
- * @param updater Pointer to the updater 
- * @returns Path of Updates directory.
- * 
- */
 const char *osync_updater_get_updates_directory(OSyncUpdater *updater)
 {
 	return updater->updatesdir; 
 }
 
-/*@}*/
-
