@@ -31,19 +31,17 @@
 #include "config.h"
 
 #include <errno.h>
+#ifndef _WIN32
 extern int errno;
+#endif
 
+#ifdef __GNUC__
+#define OSYNC_UNUSED __attribute__ ((unused))
+#else
+#define OSYNC_UNUSED
+#endif
 
-#define osync_return_if_fail(condition) do {                                            \
-  if (!(condition)) {                                                                   \
-    return;                                                                             \
-  } } while (0)
+#include "opensync/common/opensync_common_internals.h"
 
-#define osync_return_val_if_fail(condition, val) do {                                   \
-  if (!(condition)) {                                                                   \
-    return (val);                                                                       \
-  } } while (0)
+#endif /* OPENSYNC_INTERNALS_H_ */
 
-#include "opensync_support_internals.h"
-
-#endif //OPENSYNC_INTERNALS_H_
