@@ -1,6 +1,6 @@
 /*
  * libopensync - A synchronization framework
- * Copyright (C) 2004-2005  Armin Bauer <armin.bauer@opensync.org>
+ * Copyright (C) 2009 Bjoern Ricks <bjoern.ricks@gmail.com>
  * 
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -18,29 +18,33 @@
  * 
  */
 
-#ifndef OPENSYNC_PLUGIN_INFO_INTERNALS_H_
-#define OPENSYNC_PLUGIN_INFO_INTERNALS_H_
+#ifndef _OPENSYNC_PLUGIN_INFO_INTERNALS_H_
+#define _OPENSYNC_PLUGIN_INFO_INTERNALS_H_
 
-struct OSyncPluginInfo {
-	void *loop;
-	OSyncPluginConfig *config;
-	GList *objtypes;
-	char *configdir;
+/**
+ * @defgroup OSyncPluginInfoInternalAPI OpenSync Plugin Info Internals
+ * @ingroup OSyncPluginPrivate
+ */
 
-	/* The main sink */
-	OSyncObjTypeSink *main_sink;
+/*@{*/
 
-	OSyncObjTypeSink *current_sink;
-	OSyncFormatEnv *formatenv;
-	//devinfo
-	int ref_count;
-	char *groupname;
-	OSyncVersion *version;
-	OSyncCapabilities *capabilities;
+/** @brief Returns the currently running sink
+ * 
+ * @param info Pointer to the plugin info object
+ * @returns the current sink
+ * 
+ */
+OSyncObjTypeSink *osync_plugin_info_get_sink(OSyncPluginInfo *info);
 
-#ifdef OPENSYNC_UNITTESTS
-	long long int memberid; // introduced only for testing purpose (mock-sync)
-#endif	
-};
+/** @brief Returns the plugin configuration directory
+ * 
+ * @param info Pointer to the plugin info object
+ * @returns the full path where configuration files for the plugin are stored
+ * 
+ */
+OSYNC_TEST_EXPORT const char *osync_plugin_info_get_configdir(OSyncPluginInfo *info);
 
-#endif /*OPENSYNC_PLUGIN_INFO_INTERNALS_H_*/
+/*@}*/
+
+
+#endif /* _OPENSYNC_PLUGIN_INFO_INTERNALS_H_ */
