@@ -1,16 +1,21 @@
 #include "support.h"
-#include <sys/wait.h>
 
 #include <opensync/opensync-client.h>
 #include <opensync/opensync-ipc.h>
 #include <opensync/opensync-plugin.h>
 
+#include "opensync/client/opensync_client_proxy_internals.h"
+#include "opensync/plugin/opensync_objtype_sink_internals.h"
+
 START_TEST (proxy_new)
 {
 	char *testbed = setup_testbed(NULL);
+	char *formatdir = g_strdup_printf("%s/formats",  testbed);
+
+	OSyncFormatEnv *formatenv = osync_testing_load_formatenv(formatdir);
 	
 	OSyncError *error = NULL;
-	OSyncClientProxy *proxy = osync_client_proxy_new(NULL, NULL, &error);
+	OSyncClientProxy *proxy = osync_client_proxy_new(formatenv, NULL, &error);
 	fail_unless(proxy != NULL, NULL);
 	fail_unless(error == NULL, NULL);
 	
@@ -25,9 +30,12 @@ END_TEST
 START_TEST (proxy_spawn)
 {
 	char *testbed = setup_testbed(NULL);
+	char *formatdir = g_strdup_printf("%s/formats",  testbed);
+
+	OSyncFormatEnv *formatenv = osync_testing_load_formatenv(formatdir);
 	
 	OSyncError *error = NULL;
-	OSyncClientProxy *proxy = osync_client_proxy_new(NULL, NULL, &error);
+	OSyncClientProxy *proxy = osync_client_proxy_new(formatenv, NULL, &error);
 	fail_unless(proxy != NULL, NULL);
 	fail_unless(error == NULL, NULL);
 	
@@ -89,6 +97,8 @@ START_TEST (proxy_init)
 	char *testbed = setup_testbed(NULL);
 	char *formatdir = g_strdup_printf("%s/formats",  testbed);
 	char *plugindir = g_strdup_printf("%s/plugins",  testbed);
+
+	OSyncFormatEnv *formatenv = osync_testing_load_formatenv(formatdir);
 	
 	OSyncError *error = NULL;
 	OSyncThread *thread = osync_thread_new(NULL, &error);
@@ -96,7 +106,7 @@ START_TEST (proxy_init)
 	fail_unless(error == NULL, NULL);
 	osync_thread_start(thread);
 	
-	OSyncClientProxy *proxy = osync_client_proxy_new(NULL, NULL, &error);
+	OSyncClientProxy *proxy = osync_client_proxy_new(formatenv, NULL, &error);
 	fail_unless(proxy != NULL, NULL);
 	fail_unless(error == NULL, NULL);
 	
@@ -125,7 +135,7 @@ START_TEST (proxy_init)
 	g_free(plugindir);
 	
 	osync_thread_stop(thread);
-	osync_thread_free(thread);
+	osync_thread_unref(thread);
 	
 	destroy_testbed(testbed);
 }
@@ -136,6 +146,8 @@ START_TEST (proxy_discover)
 	char *testbed = setup_testbed(NULL);
 	char *formatdir = g_strdup_printf("%s/formats",  testbed);
 	char *plugindir = g_strdup_printf("%s/plugins",  testbed);
+
+	OSyncFormatEnv *formatenv = osync_testing_load_formatenv(formatdir);
 	
 	OSyncError *error = NULL;
 	OSyncThread *thread = osync_thread_new(NULL, &error);
@@ -143,7 +155,7 @@ START_TEST (proxy_discover)
 	fail_unless(error == NULL, NULL);
 	osync_thread_start(thread);
 	
-	OSyncClientProxy *proxy = osync_client_proxy_new(NULL, NULL, &error);
+	OSyncClientProxy *proxy = osync_client_proxy_new(formatenv, NULL, &error);
 	fail_unless(proxy != NULL, NULL);
 	fail_unless(error == NULL, NULL);
 	
@@ -189,7 +201,7 @@ START_TEST (proxy_discover)
 	g_free(plugindir);
 	
 	osync_thread_stop(thread);
-	osync_thread_free(thread);
+	osync_thread_unref(thread);
 	
 	destroy_testbed(testbed);
 }
@@ -201,13 +213,15 @@ START_TEST (proxy_connect)
 	char *formatdir = g_strdup_printf("%s/formats",  testbed);
 	char *plugindir = g_strdup_printf("%s/plugins",  testbed);
 
+	OSyncFormatEnv *formatenv = osync_testing_load_formatenv(formatdir);
+
 	OSyncError *error = NULL;
 	OSyncThread *thread = osync_thread_new(NULL, &error);
 	fail_unless(thread != NULL, NULL);
 	fail_unless(error == NULL, NULL);
 	osync_thread_start(thread);
 	
-	OSyncClientProxy *proxy = osync_client_proxy_new(NULL, NULL, &error);
+	OSyncClientProxy *proxy = osync_client_proxy_new(formatenv, NULL, &error);
 	fail_unless(proxy != NULL, NULL);
 	fail_unless(error == NULL, NULL);
 
@@ -245,36 +259,17 @@ START_TEST (proxy_connect)
 	g_free(plugindir);
 	
 	osync_thread_stop(thread);
-	osync_thread_free(thread);
+	osync_thread_unref(thread);
 	
 	destroy_testbed(testbed);
 }
 END_TEST
 
-Suite *proxy_suite(void)
-{
-	Suite *s = suite_create("Proxy");
-//	Suite *s2 = suite_create("Proxy");
-	
-	create_case(s, "proxy_new", proxy_new);
-	create_case(s, "proxy_spawn", proxy_spawn);
-	create_case(s, "proxy_init", proxy_init);
-	create_case(s, "proxy_discover", proxy_discover);
-	create_case(s, "proxy_connect", proxy_connect);
-	
-	return s;
-}
+OSYNC_TESTCASE_START("proxy")
+OSYNC_TESTCASE_ADD(proxy_new)
+OSYNC_TESTCASE_ADD(proxy_spawn)
+OSYNC_TESTCASE_ADD(proxy_init)
+OSYNC_TESTCASE_ADD(proxy_discover)
+OSYNC_TESTCASE_ADD(proxy_connect)
+OSYNC_TESTCASE_END
 
-int main(void)
-{
-	int nf;
-
-	Suite *s = proxy_suite();
-	
-	SRunner *sr;
-	sr = srunner_create(s);
-	srunner_run_all(sr, CK_VERBOSE);
-	nf = srunner_ntests_failed(sr);
-	srunner_free(sr);
-	return (nf == 0) ? EXIT_SUCCESS : EXIT_FAILURE;
-}
