@@ -1,4 +1,4 @@
-#!/bin/bash
+#!/bin/sh
 
 echo -n "Checking Code Coverage of unit tests"
 DATE=$( date +%Y%m%d%H%M%S )
@@ -20,6 +20,10 @@ echo $TITLE
 lcov -t "$TITLE" -b ../ -d ../ -q -c -o coverage/$TITLE.info
 genhtml --legend -t "$TITLE" -o coverage/html/$TITLE coverage/$TITLE.info &> /dev/null
 
+cd coverage/html/
+ln -sf $TITLE LATEST
+cd ../../
+
 # XXX: reset counters ( lcov -z seems to be outdated... tries to dlete .da files instead of .gcda ) 
 find ../ -name "*.gcda" | xargs rm
 
