@@ -6,26 +6,31 @@
 #include <opensync/opensync-format.h>
 #include <opensync/opensync-client.h>
 #include <opensync/opensync-engine.h>
-#include <opensync/opensync-context.h>
+#include <opensync/opensync-plugin.h>
 
 #include "opensync/engine/opensync_engine_internals.h"
+#include "opensync/engine/opensync_engine_private.h"
+
 #include "opensync/group/opensync_member_internals.h"
+#include "opensync/client/opensync_client_internals.h"
 
 #include "../mock-plugin/mock_sync.h"
 #include "../mock-plugin/mock_format.h"
 
 static void _member_add_format(OSyncMember *member, const char *objtype, const char *objformat)
 {
-       OSyncObjTypeSink *sink = NULL;
-       osync_assert(member);
-       osync_assert(objtype);
+	OSyncObjTypeSink *sink = NULL;
+	OSyncError *error = NULL;
+	osync_assert(member);
+	osync_assert(objtype);
 
-       if (!osync_member_find_objtype_sink(member, objtype)) {
-               sink = osync_objtype_sink_new(objtype, NULL);
-	       osync_member_add_objtype_sink(member, sink);
-       }
+	if (!osync_member_find_objtype_sink(member, objtype)) {
+		sink = osync_objtype_sink_new(objtype, &error);
+		osync_member_add_objtype_sink(member, sink);
+		osync_objtype_sink_unref(sink);
+	}
 
-       osync_member_add_objformat(member, objtype, objformat);
+	osync_member_add_objformat(member, objtype, objformat, &error);
 }
 
 START_TEST (engine_new)
@@ -36,6 +41,8 @@ START_TEST (engine_new)
 	OSyncGroup *group = osync_group_new(&error);
 	fail_unless(group != NULL, NULL);
 	fail_unless(error == NULL, NULL);
+
+	osync_group_set_configdir(group, testbed);
 	
 	OSyncMember *member1 = osync_member_new(&error);
 	fail_unless(member1 != NULL, NULL);
@@ -74,6 +81,8 @@ START_TEST (engine_init)
 	OSyncGroup *group = osync_group_new(&error);
 	fail_unless(group != NULL, NULL);
 	fail_unless(error == NULL, NULL);
+
+	osync_group_set_configdir(group, testbed);
 	
 	OSyncMember *member1 = osync_member_new(&error);
 	fail_unless(member1 != NULL, NULL);
@@ -126,11 +135,12 @@ START_TEST (engine_init)
 }
 END_TEST
 
-static void connect(void *data, OSyncPluginInfo *info, OSyncContext *ctx)
+static void connect1(OSyncObjTypeSink *sink, OSyncPluginInfo *info, OSyncContext *ctx, void *data)
 {
 	mock_env *env = data;
 	osync_trace(TRACE_ENTRY, "%s(%p, %p, %p)", __func__, data, info, ctx);
 	
+	osync_assert(env);
 	osync_assert(env->num_connect == 0);
 	osync_assert(env->num_disconnect == 0);
 	osync_assert(env->num_get_changes == 0);
@@ -145,11 +155,12 @@ static void connect(void *data, OSyncPluginInfo *info, OSyncContext *ctx)
 	osync_trace(TRACE_EXIT, "%s", __func__);
 }
 
-static void disconnect(void *data, OSyncPluginInfo *info, OSyncContext *ctx)
+static void disconnect(OSyncObjTypeSink *sink, OSyncPluginInfo *info, OSyncContext *ctx, void *data)
 {
 	mock_env *env = data;
 	osync_trace(TRACE_ENTRY, "%s(%p, %p, %p)", __func__, data, info, ctx);
 	
+	osync_assert(env);
 	osync_assert(env->num_connect == 1);
 	osync_assert(env->num_disconnect == 0);
 	osync_assert(env->num_get_changes == 1);
@@ -164,11 +175,12 @@ static void disconnect(void *data, OSyncPluginInfo *info, OSyncContext *ctx)
 	osync_trace(TRACE_EXIT, "%s", __func__);
 }
 
-static void get_changes(void *data, OSyncPluginInfo *info, OSyncContext *ctx)
+static void get_changes(OSyncObjTypeSink *sink, OSyncPluginInfo *info, OSyncContext *ctx, osync_bool slow_sync, void *data)
 {
 	mock_env *env = data;
 	osync_trace(TRACE_ENTRY, "%s(%p, %p, %p)", __func__, data, info, ctx);
 	
+	osync_assert(env);
 	osync_assert(env->num_connect == 1);
 	osync_assert(env->num_disconnect == 0);
 	osync_assert(env->num_get_changes == 0);
@@ -197,14 +209,15 @@ static void *initialize(OSyncPlugin *plugin, OSyncPluginInfo *info, OSyncError *
 	
 	OSyncObjFormatSink *format_sink = osync_objformat_sink_new("mockobjtype1", error);
 	osync_objtype_sink_add_objformat_sink(sink, format_sink);
+	osync_objformat_sink_unref(format_sink);
 	
-	OSyncObjTypeSinkFunctions functions;
-	memset(&functions, 0, sizeof(functions));
-	functions.connect = connect;
-	functions.disconnect = disconnect;
-	functions.get_changes = get_changes;
-	
-	osync_objtype_sink_set_functions(sink, functions, NULL);
+	osync_objtype_sink_set_connect_func(sink, connect1);
+	osync_objtype_sink_set_disconnect_func(sink, disconnect);
+	osync_objtype_sink_set_get_changes_func(sink, get_changes);
+
+	/* ObjTypeSink port - functions and userdata never got set! */
+	osync_objtype_sink_set_userdata(sink, env);
+
 	osync_plugin_info_add_objtype(info, sink);
 	osync_objtype_sink_unref(sink);
 
@@ -220,6 +233,7 @@ static void finalize(void *data)
 {
 	mock_env *env = data;
 	
+	osync_assert(env);
 	osync_assert(env->num_connect == 1);
 	osync_assert(env->num_disconnect == 1);
 	osync_assert(env->num_get_changes == 1);
@@ -238,6 +252,8 @@ static OSyncDebugGroup *_create_group(char *testbed)
 	debug->group = osync_group_new(&error);
 	fail_unless(debug->group != NULL, NULL);
 	fail_unless(error == NULL, NULL);
+
+	osync_group_set_configdir(debug->group, testbed);
 	
 	debug->member1 = osync_member_new(&error);
 	fail_unless(debug->member1 != NULL, NULL);
@@ -311,10 +327,10 @@ static void _engine_instrument_pluginenv(OSyncEngine *engine, OSyncDebugGroup *d
 	engine->pluginenv = osync_plugin_env_new(NULL);
 
 	if (debug->plugin)
-		osync_plugin_env_register_plugin(engine->pluginenv, debug->plugin);
+		osync_plugin_env_register_plugin(engine->pluginenv, debug->plugin, NULL);
 
 	if (debug->plugin2)
-		osync_plugin_env_register_plugin(engine->pluginenv, debug->plugin2);
+		osync_plugin_env_register_plugin(engine->pluginenv, debug->plugin2, NULL);
 }
 
 START_TEST (engine_sync)
@@ -353,11 +369,12 @@ START_TEST (engine_sync)
 }
 END_TEST
 
-static void connect2(void *data, OSyncPluginInfo *info, OSyncContext *ctx)
+static void connect2(OSyncObjTypeSink *sink, OSyncPluginInfo *info, OSyncContext *ctx, void *data)
 {
 	mock_env *env = data;
 	osync_trace(TRACE_ENTRY, "%s(%p, %p, %p)", __func__, data, info, ctx);
 	
+	osync_assert(env);
 	osync_assert(env->num_connect < 3);
 	osync_assert(env->num_disconnect == 0);
 	osync_assert(env->num_get_changes == 0);
@@ -372,11 +389,12 @@ static void connect2(void *data, OSyncPluginInfo *info, OSyncContext *ctx)
 	osync_trace(TRACE_EXIT, "%s", __func__);
 }
 
-static void disconnect2(void *data, OSyncPluginInfo *info, OSyncContext *ctx)
+static void disconnect2(OSyncObjTypeSink *sink, OSyncPluginInfo *info, OSyncContext *ctx, void *data)
 {
 	mock_env *env = data;
 	osync_trace(TRACE_ENTRY, "%s(%p, %p, %p)", __func__, data, info, ctx);
 	
+	osync_assert(env);
 	osync_assert(env->num_connect == 3);
 	osync_assert(env->num_disconnect < 3);
 	osync_assert(env->num_get_changes == 3);
@@ -391,11 +409,12 @@ static void disconnect2(void *data, OSyncPluginInfo *info, OSyncContext *ctx)
 	osync_trace(TRACE_EXIT, "%s", __func__);
 }
 
-static void get_changes2(void *data, OSyncPluginInfo *info, OSyncContext *ctx)
+static void get_changes2(OSyncObjTypeSink *sink, OSyncPluginInfo *info, OSyncContext *ctx, osync_bool slow_sync, void *data)
 {
 	mock_env *env = data;
 	osync_trace(TRACE_ENTRY, "%s(%p, %p, %p)", __func__, data, info, ctx);
 	
+	osync_assert(env);
 	osync_assert(env->num_connect == 3);
 	osync_assert(env->num_disconnect == 0);
 	osync_assert(env->num_get_changes < 3);
@@ -410,11 +429,12 @@ static void get_changes2(void *data, OSyncPluginInfo *info, OSyncContext *ctx)
 	osync_trace(TRACE_EXIT, "%s", __func__);
 }
 
-static void main_connect2(void *data, OSyncPluginInfo *info, OSyncContext *ctx)
+static void main_connect2(OSyncObjTypeSink *sink, OSyncPluginInfo *info, OSyncContext *ctx, void *data)
 {
 	mock_env *env = data;
 	osync_trace(TRACE_ENTRY, "%s(%p, %p, %p)", __func__, data, info, ctx);
 	
+	osync_assert(env);
 	osync_assert(env->num_connect == 3);
 	osync_assert(env->num_disconnect == 0);
 	osync_assert(env->num_get_changes == 0);
@@ -429,11 +449,12 @@ static void main_connect2(void *data, OSyncPluginInfo *info, OSyncContext *ctx)
 	osync_trace(TRACE_EXIT, "%s", __func__);
 }
 
-static void main_disconnect2(void *data, OSyncPluginInfo *info, OSyncContext *ctx)
+static void main_disconnect2(OSyncObjTypeSink *sink, OSyncPluginInfo *info, OSyncContext *ctx, void *data)
 {
 	mock_env *env = data;
 	osync_trace(TRACE_ENTRY, "%s(%p, %p, %p)", __func__, data, info, ctx);
 	
+	osync_assert(env);
 	osync_assert(env->num_connect == 3);
 	osync_assert(env->num_disconnect == 3);
 	osync_assert(env->num_get_changes == 3);
@@ -448,11 +469,12 @@ static void main_disconnect2(void *data, OSyncPluginInfo *info, OSyncContext *ct
 	osync_trace(TRACE_EXIT, "%s", __func__);
 }
 
-static void main_get_changes2(void *data, OSyncPluginInfo *info, OSyncContext *ctx)
+static void main_get_changes2(OSyncObjTypeSink *sink, OSyncPluginInfo *info, OSyncContext *ctx, osync_bool slow_sync, void *data)
 {
 	mock_env *env = data;
 	osync_trace(TRACE_ENTRY, "%s(%p, %p, %p)", __func__, data, info, ctx);
 	
+	osync_assert(env);
 	osync_assert(env->num_connect == 3);
 	osync_assert(env->num_disconnect == 0);
 	osync_assert(env->num_get_changes == 3);
@@ -481,14 +503,14 @@ static void *initialize_multi(OSyncPlugin *plugin, OSyncPluginInfo *info, OSyncE
 	
 	OSyncObjFormatSink *format_sink = osync_objformat_sink_new("mockformat1", error);
 	osync_objtype_sink_add_objformat_sink(sink, format_sink);
+	osync_objformat_sink_unref(format_sink);
 	
-	OSyncObjTypeSinkFunctions functions;
-	memset(&functions, 0, sizeof(functions));
-	functions.connect = connect2;
-	functions.disconnect = disconnect2;
-	functions.get_changes = get_changes2;
+	osync_objtype_sink_set_connect_func(sink, connect2);
+	osync_objtype_sink_set_disconnect_func(sink, disconnect2);
+	osync_objtype_sink_set_get_changes_func(sink, get_changes2);
+
+	osync_objtype_sink_set_userdata(sink, env);
 	
-	osync_objtype_sink_set_functions(sink, functions, NULL);
 	osync_plugin_info_add_objtype(info, sink);
 	osync_objtype_sink_unref(sink);
 	
@@ -498,13 +520,15 @@ static void *initialize_multi(OSyncPlugin *plugin, OSyncPluginInfo *info, OSyncE
 	
 	format_sink = osync_objformat_sink_new("vcard", error);
 	osync_objtype_sink_add_objformat_sink(sink, format_sink);
+	osync_objformat_sink_unref(format_sink);
+
+	osync_objtype_sink_set_connect_func(sink, connect2);
+	osync_objtype_sink_set_disconnect_func(sink, disconnect2);
+	osync_objtype_sink_set_get_changes_func(sink, get_changes2);
+
+	osync_objtype_sink_set_userdata(sink, env);
 	
-	memset(&functions, 0, sizeof(functions));
-	functions.connect = connect2;
-	functions.disconnect = disconnect2;
-	functions.get_changes = get_changes2;
 	
-	osync_objtype_sink_set_functions(sink, functions, NULL);
 	osync_plugin_info_add_objtype(info, sink);
 	osync_objtype_sink_unref(sink);
 	
@@ -514,13 +538,14 @@ static void *initialize_multi(OSyncPlugin *plugin, OSyncPluginInfo *info, OSyncE
 	
 	format_sink = osync_objformat_sink_new("plain", error);
 	osync_objtype_sink_add_objformat_sink(sink, format_sink);
-	
-	memset(&functions, 0, sizeof(functions));
-	functions.connect = connect2;
-	functions.disconnect = disconnect2;
-	functions.get_changes = get_changes2;
-	
-	osync_objtype_sink_set_functions(sink, functions, NULL);
+	osync_objformat_sink_unref(format_sink);
+
+	osync_objtype_sink_set_connect_func(sink, connect2);
+	osync_objtype_sink_set_disconnect_func(sink, disconnect2);
+	osync_objtype_sink_set_get_changes_func(sink, get_changes2);
+
+	osync_objtype_sink_set_userdata(sink, env);
+
 	osync_plugin_info_add_objtype(info, sink);
 	osync_objtype_sink_unref(sink);
 	
@@ -529,12 +554,12 @@ static void *initialize_multi(OSyncPlugin *plugin, OSyncPluginInfo *info, OSyncE
 	if (!sink)
 		goto error;
 	
-	memset(&functions, 0, sizeof(functions));
-	functions.connect = main_connect2;
-	functions.disconnect = main_disconnect2;
-	functions.get_changes = main_get_changes2;
-	
-	osync_objtype_sink_set_functions(sink, functions, NULL);
+	osync_objtype_sink_set_connect_func(sink, main_connect2);
+	osync_objtype_sink_set_disconnect_func(sink, main_disconnect2);
+	osync_objtype_sink_set_get_changes_func(sink, main_get_changes2);
+
+	osync_objtype_sink_set_userdata(sink, env);
+
 	osync_plugin_info_set_main_sink(info, sink);
 	osync_objtype_sink_unref(sink);
 	
@@ -550,6 +575,7 @@ static void finalize_multi(void *data)
 {
 	mock_env *env = data;
 	
+	osync_assert(env);
 	osync_assert(env->num_connect == 3);
 	osync_assert(env->num_disconnect == 3);
 	osync_assert(env->num_get_changes == 3);
@@ -571,6 +597,8 @@ static OSyncDebugGroup *_create_group2(char *testbed)
 	fail_unless(debug->group != NULL, NULL);
 	fail_unless(error == NULL, NULL);
 	
+	osync_group_set_configdir(debug->group, testbed);
+
 	debug->member1 = osync_member_new(&error);
 	fail_unless(debug->member1 != NULL, NULL);
 	fail_unless(error == NULL, NULL);
@@ -663,11 +691,12 @@ START_TEST (engine_sync_multi_obj)
 }
 END_TEST
 
-static void connect3(void *data, OSyncPluginInfo *info, OSyncContext *ctx)
+static void connect3(OSyncObjTypeSink *sink, OSyncPluginInfo *info, OSyncContext *ctx, void *data)
 {
 	mock_env *env = data;
 	osync_trace(TRACE_ENTRY, "%s(%p, %p, %p)", __func__, data, info, ctx);
 	
+	osync_assert(env);
 	osync_assert(env->num_connect < 3);
 	osync_assert(env->num_disconnect == 0);
 	osync_assert(env->num_get_changes == 0);
@@ -683,11 +712,12 @@ static void connect3(void *data, OSyncPluginInfo *info, OSyncContext *ctx)
 	osync_trace(TRACE_EXIT, "%s", __func__);
 }
 
-static void disconnect3(void *data, OSyncPluginInfo *info, OSyncContext *ctx)
+static void disconnect3(OSyncObjTypeSink *sink, OSyncPluginInfo *info, OSyncContext *ctx, void *data)
 {
 	mock_env *env = data;
 	osync_trace(TRACE_ENTRY, "%s(%p, %p, %p)", __func__, data, info, ctx);
 	
+	osync_assert(env);
 	osync_assert(env->num_connect == 3);
 	osync_assert(env->num_disconnect < 3);
 	osync_assert(env->num_get_changes == 3);
@@ -705,11 +735,12 @@ static void disconnect3(void *data, OSyncPluginInfo *info, OSyncContext *ctx)
 	osync_trace(TRACE_EXIT, "%s", __func__);
 }
 
-static void get_changes3(void *data, OSyncPluginInfo *info, OSyncContext *ctx)
+static void get_changes3(OSyncObjTypeSink *sink, OSyncPluginInfo *info, OSyncContext *ctx, osync_bool slow_sync, void *data)
 {
 	mock_env *env = data;
 	osync_trace(TRACE_ENTRY, "%s(%p, %p, %p)", __func__, data, info, ctx);
 	
+	osync_assert(env);
 	osync_assert(env->num_connect == 3);
 	osync_assert(env->num_disconnect == 0);
 	osync_assert(env->num_get_changes < 3);
@@ -725,11 +756,12 @@ static void get_changes3(void *data, OSyncPluginInfo *info, OSyncContext *ctx)
 	osync_trace(TRACE_EXIT, "%s", __func__);
 }
 
-static void sync_done3(void *data, OSyncPluginInfo *info, OSyncContext *ctx)
+static void sync_done3(OSyncObjTypeSink *sink, OSyncPluginInfo *info, OSyncContext *ctx, void *data)
 {
 	mock_env *env = data;
 	osync_trace(TRACE_ENTRY, "%s(%p, %p, %p)", __func__, data, info, ctx);
 	
+	osync_assert(env);
 	osync_assert(env->num_connect == 3);
 	osync_assert(env->num_disconnect == 0);
 	osync_assert(env->num_get_changes == 3);
@@ -747,11 +779,12 @@ static void sync_done3(void *data, OSyncPluginInfo *info, OSyncContext *ctx)
 	osync_trace(TRACE_EXIT, "%s", __func__);
 }
 
-static void main_connect3(void *data, OSyncPluginInfo *info, OSyncContext *ctx)
+static void main_connect3(OSyncObjTypeSink *sink, OSyncPluginInfo *info, OSyncContext *ctx, void *data)
 {
 	mock_env *env = data;
 	osync_trace(TRACE_ENTRY, "%s(%p, %p, %p)", __func__, data, info, ctx);
 	
+	osync_assert(env);
 	osync_assert(env->num_connect == 3);
 	osync_assert(env->num_disconnect == 0);
 	osync_assert(env->num_get_changes == 0);
@@ -773,11 +806,12 @@ static void main_connect3(void *data, OSyncPluginInfo *info, OSyncContext *ctx)
 	osync_trace(TRACE_EXIT, "%s", __func__);
 }
 
-static void main_disconnect3(void *data, OSyncPluginInfo *info, OSyncContext *ctx)
+static void main_disconnect3(OSyncObjTypeSink *sink, OSyncPluginInfo *info, OSyncContext *ctx, void *data)
 {
 	mock_env *env = data;
 	osync_trace(TRACE_ENTRY, "%s(%p, %p, %p)", __func__, data, info, ctx);
 	
+	osync_assert(env);
 	osync_assert(env->num_connect == 3);
 	osync_assert(env->num_disconnect == 3);
 	osync_assert(env->num_get_changes == 3);
@@ -801,11 +835,12 @@ static void main_disconnect3(void *data, OSyncPluginInfo *info, OSyncContext *ct
 	osync_trace(TRACE_EXIT, "%s", __func__);
 }
 
-static void main_get_changes3(void *data, OSyncPluginInfo *info, OSyncContext *ctx)
+static void main_get_changes3(OSyncObjTypeSink *sink, OSyncPluginInfo *info, OSyncContext *ctx, osync_bool slow_sync, void *data)
 {
 	mock_env *env = data;
 	osync_trace(TRACE_ENTRY, "%s(%p, %p, %p)", __func__, data, info, ctx);
 	
+	osync_assert(env);
 	osync_assert(env->num_connect == 3);
 	osync_assert(env->num_disconnect == 0);
 	osync_assert(env->num_get_changes == 3);
@@ -828,11 +863,12 @@ static void main_get_changes3(void *data, OSyncPluginInfo *info, OSyncContext *c
 	osync_trace(TRACE_EXIT, "%s", __func__);
 }
 
-static void main_sync_done3(void *data, OSyncPluginInfo *info, OSyncContext *ctx)
+static void main_sync_done3(OSyncObjTypeSink *sink, OSyncPluginInfo *info, OSyncContext *ctx, void *data)
 {
 	mock_env *env = data;
 	osync_trace(TRACE_ENTRY, "%s(%p, %p, %p)", __func__, data, info, ctx);
 	
+	osync_assert(env);
 	osync_assert(env->num_connect == 3);
 	osync_assert(env->num_disconnect == 0);
 	osync_assert(env->num_get_changes == 3);
@@ -871,15 +907,15 @@ static void *initialize_order(OSyncPlugin *plugin, OSyncPluginInfo *info, OSyncE
 	
 	OSyncObjFormatSink *format_sink = osync_objformat_sink_new("mockobjtype1", error);
 	osync_objtype_sink_add_objformat_sink(sink, format_sink);
+	osync_objformat_sink_unref(format_sink);
 	
-	OSyncObjTypeSinkFunctions functions;
-	memset(&functions, 0, sizeof(functions));
-	functions.connect = connect3;
-	functions.disconnect = disconnect3;
-	functions.get_changes = get_changes3;
-	functions.sync_done = sync_done3;
-	
-	osync_objtype_sink_set_functions(sink, functions, NULL);
+	osync_objtype_sink_set_connect_func(sink, connect3);
+	osync_objtype_sink_set_disconnect_func(sink, disconnect3);
+	osync_objtype_sink_set_get_changes_func(sink, get_changes3);
+	osync_objtype_sink_set_sync_done_func(sink, sync_done3);
+
+	osync_objtype_sink_set_userdata(sink, env);
+
 	osync_plugin_info_add_objtype(info, sink);
 	osync_objtype_sink_unref(sink);
 	
@@ -889,14 +925,15 @@ static void *initialize_order(OSyncPlugin *plugin, OSyncPluginInfo *info, OSyncE
 	
 	format_sink = osync_objformat_sink_new("vcard", error);
 	osync_objtype_sink_add_objformat_sink(sink, format_sink);
-	
-	memset(&functions, 0, sizeof(functions));
-	functions.connect = connect3;
-	functions.disconnect = disconnect3;
-	functions.get_changes = get_changes3;
-	functions.sync_done = sync_done3;
-	
-	osync_objtype_sink_set_functions(sink, functions, NULL);
+	osync_objformat_sink_unref(format_sink);
+
+	osync_objtype_sink_set_connect_func(sink, connect3);
+	osync_objtype_sink_set_disconnect_func(sink, disconnect3);
+	osync_objtype_sink_set_get_changes_func(sink, get_changes3);
+	osync_objtype_sink_set_sync_done_func(sink, sync_done3);
+
+	osync_objtype_sink_set_userdata(sink, env);
+
 	osync_plugin_info_add_objtype(info, sink);
 	osync_objtype_sink_unref(sink);
 	
@@ -906,14 +943,15 @@ static void *initialize_order(OSyncPlugin *plugin, OSyncPluginInfo *info, OSyncE
 	
 	format_sink = osync_objformat_sink_new("plain", error);
 	osync_objtype_sink_add_objformat_sink(sink, format_sink);
+	osync_objformat_sink_unref(format_sink);
 	
-	memset(&functions, 0, sizeof(functions));
-	functions.connect = connect3;
-	functions.disconnect = disconnect3;
-	functions.get_changes = get_changes3;
-	functions.sync_done = sync_done3;
+	osync_objtype_sink_set_connect_func(sink, connect3);
+	osync_objtype_sink_set_disconnect_func(sink, disconnect3);
+	osync_objtype_sink_set_get_changes_func(sink, get_changes3);
+	osync_objtype_sink_set_sync_done_func(sink, sync_done3);
+
+	osync_objtype_sink_set_userdata(sink, env);
 	
-	osync_objtype_sink_set_functions(sink, functions, NULL);
 	osync_plugin_info_add_objtype(info, sink);
 	osync_objtype_sink_unref(sink);
 	
@@ -922,13 +960,13 @@ static void *initialize_order(OSyncPlugin *plugin, OSyncPluginInfo *info, OSyncE
 	if (!sink)
 		goto error;
 	
-	memset(&functions, 0, sizeof(functions));
-	functions.connect = main_connect3;
-	functions.disconnect = main_disconnect3;
-	functions.get_changes = main_get_changes3;
-	functions.sync_done = main_sync_done3;
-	
-	osync_objtype_sink_set_functions(sink, functions, NULL);
+	osync_objtype_sink_set_connect_func(sink, main_connect3);
+	osync_objtype_sink_set_disconnect_func(sink, main_disconnect3);
+	osync_objtype_sink_set_get_changes_func(sink, main_get_changes3);
+	osync_objtype_sink_set_sync_done_func(sink, main_sync_done3);
+
+	osync_objtype_sink_set_userdata(sink, env);
+
 	osync_plugin_info_set_main_sink(info, sink);
 	osync_objtype_sink_unref(sink);
 	
@@ -966,7 +1004,9 @@ static OSyncDebugGroup *_create_group3(char *testbed)
 	debug->group = osync_group_new(&error);
 	fail_unless(debug->group != NULL, NULL);
 	fail_unless(error == NULL, NULL);
-	
+
+	osync_group_set_configdir(debug->group, testbed);
+
 	debug->member1 = osync_member_new(&error);
 	fail_unless(debug->member1 != NULL, NULL);
 	fail_unless(error == NULL, NULL);
@@ -1059,11 +1099,12 @@ START_TEST (engine_sync_out_of_order)
 }
 END_TEST
 
-static void main_disconnect4(void *data, OSyncPluginInfo *info, OSyncContext *ctx)
+static void main_disconnect4(OSyncObjTypeSink *sink, OSyncPluginInfo *info, OSyncContext *ctx, void *data)
 {
 	mock_env *env = data;
 	osync_trace(TRACE_ENTRY, "%s(%p, %p, %p)", __func__, data, info, ctx);
 	
+	osync_assert(env);
 	osync_assert(env->num_connect == 3);
 	osync_assert(env->num_disconnect == 3);
 	osync_assert(env->num_get_changes == 3);
@@ -1108,15 +1149,15 @@ static void *initialize_reuse(OSyncPlugin *plugin, OSyncPluginInfo *info, OSyncE
 	
 	OSyncObjFormatSink *format_sink = osync_objformat_sink_new("mockobjtype1", error);
 	osync_objtype_sink_add_objformat_sink(sink, format_sink);
+	osync_objformat_sink_unref(format_sink);
 	
-	OSyncObjTypeSinkFunctions functions;
-	memset(&functions, 0, sizeof(functions));
-	functions.connect = connect3;
-	functions.disconnect = disconnect3;
-	functions.get_changes = get_changes3;
-	functions.sync_done = sync_done3;
+	osync_objtype_sink_set_connect_func(sink, connect3);
+	osync_objtype_sink_set_disconnect_func(sink, disconnect3);
+	osync_objtype_sink_set_get_changes_func(sink, get_changes3);
+	osync_objtype_sink_set_sync_done_func(sink, sync_done3);
+
+	osync_objtype_sink_set_userdata(sink, env);
 	
-	osync_objtype_sink_set_functions(sink, functions, NULL);
 	osync_plugin_info_add_objtype(info, sink);
 	osync_objtype_sink_unref(sink);
 	
@@ -1126,14 +1167,15 @@ static void *initialize_reuse(OSyncPlugin *plugin, OSyncPluginInfo *info, OSyncE
 	
 	format_sink = osync_objformat_sink_new("vcard", error);
 	osync_objtype_sink_add_objformat_sink(sink, format_sink);
+	osync_objformat_sink_unref(format_sink);
 	
-	memset(&functions, 0, sizeof(functions));
-	functions.connect = connect3;
-	functions.disconnect = disconnect3;
-	functions.get_changes = get_changes3;
-	functions.sync_done = sync_done3;
+	osync_objtype_sink_set_connect_func(sink, connect3);
+	osync_objtype_sink_set_disconnect_func(sink, disconnect3);
+	osync_objtype_sink_set_get_changes_func(sink, get_changes3);
+	osync_objtype_sink_set_sync_done_func(sink, sync_done3);
+
+	osync_objtype_sink_set_userdata(sink, env);
 	
-	osync_objtype_sink_set_functions(sink, functions, NULL);
 	osync_plugin_info_add_objtype(info, sink);
 	osync_objtype_sink_unref(sink);
 	
@@ -1143,14 +1185,15 @@ static void *initialize_reuse(OSyncPlugin *plugin, OSyncPluginInfo *info, OSyncE
 	
 	format_sink = osync_objformat_sink_new("plain", error);
 	osync_objtype_sink_add_objformat_sink(sink, format_sink);
+	osync_objformat_sink_unref(format_sink);
 	
-	memset(&functions, 0, sizeof(functions));
-	functions.connect = connect3;
-	functions.disconnect = disconnect3;
-	functions.get_changes = get_changes3;
-	functions.sync_done = sync_done3;
-	
-	osync_objtype_sink_set_functions(sink, functions, NULL);
+	osync_objtype_sink_set_connect_func(sink, connect3);
+	osync_objtype_sink_set_disconnect_func(sink, disconnect3);
+	osync_objtype_sink_set_get_changes_func(sink, get_changes3);
+	osync_objtype_sink_set_sync_done_func(sink, sync_done3);
+
+	osync_objtype_sink_set_userdata(sink, env);
+
 	osync_plugin_info_add_objtype(info, sink);
 	osync_objtype_sink_unref(sink);
 	
@@ -1159,13 +1202,13 @@ static void *initialize_reuse(OSyncPlugin *plugin, OSyncPluginInfo *info, OSyncE
 	if (!sink)
 		goto error;
 	
-	memset(&functions, 0, sizeof(functions));
-	functions.connect = main_connect3;
-	functions.disconnect = main_disconnect4;
-	functions.get_changes = main_get_changes3;
-	functions.sync_done = main_sync_done3;
+	osync_objtype_sink_set_connect_func(sink, main_connect3);
+	osync_objtype_sink_set_disconnect_func(sink, main_disconnect4);
+	osync_objtype_sink_set_get_changes_func(sink, main_get_changes3);
+	osync_objtype_sink_set_sync_done_func(sink, main_sync_done3);
+
+	osync_objtype_sink_set_userdata(sink, env);
 	
-	osync_objtype_sink_set_functions(sink, functions, NULL);
 	osync_plugin_info_set_main_sink(info, sink);
 	osync_objtype_sink_unref(sink);
 	
@@ -1194,6 +1237,8 @@ static OSyncDebugGroup *_create_group4(char *testbed)
 	debug->group = osync_group_new(&error);
 	fail_unless(debug->group != NULL, NULL);
 	fail_unless(error == NULL, NULL);
+
+	osync_group_set_configdir(debug->group, testbed);
 	
 	debug->member1 = osync_member_new(&error);
 	fail_unless(debug->member1 != NULL, NULL);
@@ -1295,7 +1340,7 @@ END_TEST
 
 START_TEST (engine_sync_stress)
 {
-	int n = 1000;
+	int n = 250; /* FIXME: Used to be 1000 - #1040 */
 	int i = 0;
 	
 	char *testbed = setup_testbed("sync_setup");
@@ -1333,7 +1378,7 @@ START_TEST (engine_sync_stress)
 }
 END_TEST
 
-static void connect5(void *data, OSyncPluginInfo *info, OSyncContext *ctx)
+static void connect5(OSyncObjTypeSink *sink, OSyncPluginInfo *info, OSyncContext *ctx, void *data)
 {
 	mock_env *env = data;
 	osync_trace(TRACE_ENTRY, "%s(%p, %p, %p)", __func__, data, info, ctx);
@@ -1345,7 +1390,7 @@ static void connect5(void *data, OSyncPluginInfo *info, OSyncContext *ctx)
 	osync_trace(TRACE_EXIT, "%s", __func__);
 }
 
-static void disconnect5(void *data, OSyncPluginInfo *info, OSyncContext *ctx)
+static void disconnect5(OSyncObjTypeSink *sink, OSyncPluginInfo *info, OSyncContext *ctx, void *data)
 {
 	mock_env *env = data;
 	osync_trace(TRACE_ENTRY, "%s(%p, %p, %p)", __func__, data, info, ctx);
@@ -1357,7 +1402,7 @@ static void disconnect5(void *data, OSyncPluginInfo *info, OSyncContext *ctx)
 	osync_trace(TRACE_EXIT, "%s", __func__);
 }
 
-static void get_changes5(void *data, OSyncPluginInfo *info, OSyncContext *ctx)
+static void get_changes5(OSyncObjTypeSink *sink, OSyncPluginInfo *info, OSyncContext *ctx, osync_bool slow_sync, void *data)
 {
 	mock_env *env = data;
 	osync_trace(TRACE_ENTRY, "%s(%p, %p, %p)", __func__, data, info, ctx);
@@ -1375,7 +1420,8 @@ static void get_changes5(void *data, OSyncPluginInfo *info, OSyncContext *ctx)
 	
 	osync_change_set_changetype(change, OSYNC_CHANGE_TYPE_ADDED);
 
-	char *uid = g_strdup_printf("uid_%p", change);
+	char *uid = osync_rand_str(16, &error); 
+	osync_assert(error == NULL);
 	osync_change_set_uid(change, uid);
 	g_free(uid);
 	
@@ -1395,6 +1441,7 @@ static void get_changes5(void *data, OSyncPluginInfo *info, OSyncContext *ctx)
 	osync_data_unref(changedata);
 	
 	osync_context_report_change(ctx, change);
+	osync_change_unref(change);
 	
 	g_atomic_int_inc(&(env->num_get_changes));
 	
@@ -1403,7 +1450,7 @@ static void get_changes5(void *data, OSyncPluginInfo *info, OSyncContext *ctx)
 	osync_trace(TRACE_EXIT, "%s", __func__);
 }
 
-static void commit_change5(void *data, OSyncPluginInfo *info, OSyncContext *ctx, OSyncChange *change)
+static void commit_change5(OSyncObjTypeSink *sink, OSyncPluginInfo *info, OSyncContext *ctx, OSyncChange *change, void *data)
 {
 	mock_env *env = data;
 	osync_trace(TRACE_ENTRY, "%s(%p, %p, %p, %p)", __func__, data, info, ctx, change);
@@ -1429,15 +1476,15 @@ static void *initialize5(OSyncPlugin *plugin, OSyncPluginInfo *info, OSyncError
 	
 	OSyncObjFormatSink *format_sink = osync_objformat_sink_new("mockformat1", error);
 	osync_objtype_sink_add_objformat_sink(sink, format_sink);
+	osync_objformat_sink_unref(format_sink);
 	
-	OSyncObjTypeSinkFunctions functions;
-	memset(&functions, 0, sizeof(functions));
-	functions.connect = connect5;
-	functions.disconnect = disconnect5;
-	functions.get_changes = get_changes5;
-	functions.commit = commit_change5;
-	
-	osync_objtype_sink_set_functions(sink, functions, NULL);
+	osync_objtype_sink_set_connect_func(sink, connect5);
+	osync_objtype_sink_set_disconnect_func(sink, disconnect5);
+	osync_objtype_sink_set_get_changes_func(sink, get_changes5);
+	osync_objtype_sink_set_commit_func(sink, commit_change5);
+
+	osync_objtype_sink_set_userdata(sink, env);
+
 	osync_plugin_info_add_objtype(info, sink);
 	osync_objtype_sink_unref(sink);
 	
@@ -1474,6 +1521,8 @@ static OSyncDebugGroup *_create_group5(char *testbed)
 	debug->group = osync_group_new(&error);
 	fail_unless(debug->group != NULL, NULL);
 	fail_unless(error == NULL, NULL);
+
+	osync_group_set_configdir(debug->group, testbed);
 	
 	debug->member1 = osync_member_new(&error);
 	fail_unless(debug->member1 != NULL, NULL);
@@ -1495,8 +1544,6 @@ static OSyncDebugGroup *_create_group5(char *testbed)
 	g_free(path);
 
 	_member_add_format(debug->member2, "mockobjtype1", "mockformat1");
-	_member_add_format(debug->member2, "mockobjtype2", "mockformat2");
-	_member_add_format(debug->member2, "mockobjtype3", "mockformat3");
 	
 	debug->plugin = osync_plugin_new(&error);
 	fail_unless(debug->plugin != NULL, NULL);
@@ -1564,7 +1611,9 @@ START_TEST (engine_sync_read_write)
 }
 END_TEST
 
-static void get_changes6(void *data, OSyncPluginInfo *info, OSyncContext *ctx)
+#define COMMIT_STRESS_GROUP6 250
+
+static void get_changes6(OSyncObjTypeSink *sink, OSyncPluginInfo *info, OSyncContext *ctx, osync_bool slow_sync, void *data)
 {
 	mock_env *env = data;
 	int i;
@@ -1578,15 +1627,18 @@ static void get_changes6(void *data, OSyncPluginInfo *info, OSyncContext *ctx)
 	
 	OSyncError *error = NULL;
 	
-	for (i = 0; i < 1000; i++) {
+	for (i = 0; i < COMMIT_STRESS_GROUP6; i++) {
 		OSyncChange *change = osync_change_new(&error);
 		osync_assert(change != NULL);
 		osync_assert(error == NULL);
 		
 		osync_change_set_changetype(change, OSYNC_CHANGE_TYPE_ADDED);
-		char *uid = g_strdup_printf("uid_%p_%i", change, i);
+		char *rand = osync_rand_str(16, &error);
+		osync_assert(error == NULL);
+		char *uid = osync_strdup_printf("uid_%s_%u", rand, i);
 		osync_change_set_uid(change, uid);
-		g_free(uid);
+		osync_free(uid);
+		osync_free(rand);
 
 		OSyncFileFormat *file = osync_try_malloc0(sizeof(OSyncFileFormat), &error);
 		osync_assert(file != NULL);
@@ -1606,6 +1658,7 @@ static void get_changes6(void *data, OSyncPluginInfo *info, OSyncContext *ctx)
 		osync_data_unref(changedata);
 		
 		osync_context_report_change(ctx, change);
+		osync_change_unref(change);
 	}
 	
 	g_atomic_int_inc(&(env->num_get_changes));
@@ -1629,15 +1682,15 @@ static void *initialize6(OSyncPlugin *plugin, OSyncPluginInfo *info, OSyncError
 	
 	OSyncObjFormatSink *format_sink = osync_objformat_sink_new("mockformat1", error);
 	osync_objtype_sink_add_objformat_sink(sink, format_sink);
+	osync_objformat_sink_unref(format_sink);
 	
-	OSyncObjTypeSinkFunctions functions;
-	memset(&functions, 0, sizeof(functions));
-	functions.connect = connect5;
-	functions.disconnect = disconnect5;
-	functions.get_changes = get_changes6;
-	functions.commit = commit_change5;
-	
-	osync_objtype_sink_set_functions(sink, functions, NULL);
+	osync_objtype_sink_set_connect_func(sink, connect5);
+	osync_objtype_sink_set_disconnect_func(sink, disconnect5);
+	osync_objtype_sink_set_get_changes_func(sink, get_changes6);
+	osync_objtype_sink_set_commit_func(sink, commit_change5);
+
+	osync_objtype_sink_set_userdata(sink, env);
+
 	osync_plugin_info_add_objtype(info, sink);
 	osync_objtype_sink_unref(sink);
 	
@@ -1656,7 +1709,7 @@ static void finalize6(void *data)
 	osync_assert(env->num_connect == 1);
 	osync_assert(env->num_disconnect == 1);
 	osync_assert(env->num_get_changes == 1);
-	osync_assert(env->num_commit_changes == 1000);
+	osync_assert(env->num_commit_changes == COMMIT_STRESS_GROUP6);
 	osync_assert(env->main_connect == 0);
 	osync_assert(env->main_disconnect == 0);
 	osync_assert(env->main_get_changes == 0);
@@ -1674,6 +1727,8 @@ static OSyncDebugGroup *_create_group6(char *testbed)
 	debug->group = osync_group_new(&error);
 	fail_unless(debug->group != NULL, NULL);
 	fail_unless(error == NULL, NULL);
+
+	osync_group_set_configdir(debug->group, testbed);
 	
 	debug->member1 = osync_member_new(&error);
 	fail_unless(debug->member1 != NULL, NULL);
@@ -1764,7 +1819,7 @@ START_TEST (engine_sync_read_write_stress)
 }
 END_TEST
 
-static void get_changes7(void *data, OSyncPluginInfo *info, OSyncContext *ctx)
+static void get_changes7(OSyncObjTypeSink *sink, OSyncPluginInfo *info, OSyncContext *ctx, osync_bool slow_sync, void *data)
 {
 	mock_env *env = data;
 	int i;
@@ -1784,7 +1839,8 @@ static void get_changes7(void *data, OSyncPluginInfo *info, OSyncContext *ctx)
 		osync_assert(error == NULL);
 		
 		osync_change_set_changetype(change, OSYNC_CHANGE_TYPE_ADDED);
-		char *uid = g_strdup_printf("uid_%p_%i", change, i);
+		char *uid = osync_rand_str(16, &error);
+		osync_assert(error == NULL);
 		osync_change_set_uid(change, uid);
 		g_free(uid);
 		
@@ -1806,6 +1862,7 @@ static void get_changes7(void *data, OSyncPluginInfo *info, OSyncContext *ctx)
 		osync_data_unref(changedata);
 		
 		osync_context_report_change(ctx, change);
+		osync_change_unref(change);
 	}
 	
 	g_atomic_int_inc(&(env->num_get_changes));
@@ -1829,15 +1886,15 @@ static void *initialize7(OSyncPlugin *plugin, OSyncPluginInfo *info, OSyncError
 	
 	OSyncObjFormatSink *format_sink = osync_objformat_sink_new("mockformat1", error);
 	osync_objtype_sink_add_objformat_sink(sink, format_sink);
+	osync_objformat_sink_unref(format_sink);
 	
-	OSyncObjTypeSinkFunctions functions;
-	memset(&functions, 0, sizeof(functions));
-	functions.connect = connect5;
-	functions.disconnect = disconnect5;
-	functions.get_changes = get_changes7;
-	functions.commit = commit_change5;
-	
-	osync_objtype_sink_set_functions(sink, functions, NULL);
+	osync_objtype_sink_set_connect_func(sink, connect5);
+	osync_objtype_sink_set_disconnect_func(sink, disconnect5);
+	osync_objtype_sink_set_get_changes_func(sink, get_changes7);
+	osync_objtype_sink_set_commit_func(sink, commit_change5);
+
+	osync_objtype_sink_set_userdata(sink, env);
+
 	osync_plugin_info_add_objtype(info, sink);
 	osync_objtype_sink_unref(sink);
 	
@@ -1874,6 +1931,8 @@ static OSyncDebugGroup *_create_group7(char *testbed)
 	debug->group = osync_group_new(&error);
 	fail_unless(debug->group != NULL, NULL);
 	fail_unless(error == NULL, NULL);
+
+	osync_group_set_configdir(debug->group, testbed);
 	
 	debug->member1 = osync_member_new(&error);
 	fail_unless(debug->member1 != NULL, NULL);
@@ -1896,8 +1955,6 @@ static OSyncDebugGroup *_create_group7(char *testbed)
 	g_free(path);
 
 	_member_add_format(debug->member2, "mockobjtype1", "mockformat1");
-	_member_add_format(debug->member2, "mockobjtype2", "mockformat2");
-	_member_add_format(debug->member2, "mockobjtype3", "mockformat3");
 	
 	debug->plugin = osync_plugin_new(&error);
 	fail_unless(debug->plugin != NULL, NULL);
@@ -1965,42 +2022,22 @@ START_TEST (engine_sync_read_write_stress2)
 }
 END_TEST
 
-Suite *engine_suite(void)
-{
-	Suite *s = suite_create("Engine");
-//	Suite *s2 = suite_create("Engine");
-	
-	create_case(s, "engine_new", engine_new);
-	create_case(s, "engine_init", engine_init);
-	create_case(s, "engine_sync", engine_sync);
-	create_case(s, "engine_sync_multi_obj", engine_sync_multi_obj);
-	create_case(s, "engine_sync_out_of_order", engine_sync_out_of_order);
-	create_case(s, "engine_sync_reuse", engine_sync_reuse);
-	create_case(s, "engine_sync_stress", engine_sync_stress);
+OSYNC_TESTCASE_START(engine)
 
-	create_case(s, "engine_sync_read_write_stress", engine_sync_read_write_stress);
-	create_case(s, "engine_sync_read_write", engine_sync_read_write);
-	create_case(s, "engine_sync_read_write_stress2", engine_sync_read_write_stress2);
-	
-	//batch commit
-	//connect problem
-	//get_changes problem
-	
-	return s;
-}
+OSYNC_TESTCASE_ADD(engine_new)
+OSYNC_TESTCASE_ADD(engine_init)
+OSYNC_TESTCASE_ADD(engine_sync)
+OSYNC_TESTCASE_ADD(engine_sync_multi_obj)
+OSYNC_TESTCASE_ADD(engine_sync_out_of_order)
+OSYNC_TESTCASE_ADD(engine_sync_reuse)
+OSYNC_TESTCASE_ADD(engine_sync_stress)
 
-int main(void)
-{
-	int nf;
+OSYNC_TESTCASE_ADD(engine_sync_read_write)
+OSYNC_TESTCASE_ADD(engine_sync_read_write_stress)
+OSYNC_TESTCASE_ADD(engine_sync_read_write_stress2)
 
-	check_env();
+//connect problem
+//get_changes problem
+
+OSYNC_TESTCASE_END
 
-	Suite *s = engine_suite();
-	
-	SRunner *sr;
-	sr = srunner_create(s);
-	srunner_run_all(sr, CK_VERBOSE);
-	nf = srunner_ntests_failed(sr);
-	srunner_free(sr);
-	return (nf == 0) ? EXIT_SUCCESS : EXIT_FAILURE;
-}
