@@ -10,7 +10,7 @@ static osync_bool detect_false(const char *data, int size, void *userdata)
 	return FALSE;
 }
 
-START_TEST (conv_env_detect_smart)
+START_TEST (detect_smart)
 {
 	OSyncError *error = NULL;
 
@@ -24,7 +24,7 @@ START_TEST (conv_env_detect_smart)
 	OSyncFormatConverter *conv = osync_converter_new_detector(format2, format1, detect, &error);
 	fail_unless(error == NULL);
 
-	osync_format_env_register_converter(env, conv);
+	osync_format_env_register_converter(env, conv, &error);
 
 	mark_point();
 
@@ -36,11 +36,11 @@ START_TEST (conv_env_detect_smart)
 	fail_unless(osync_data_get_objformat(data) == format2);
 	
 	osync_data_unref(data);
-	osync_format_env_free(env);
+	osync_format_env_unref(env);
 }
 END_TEST
 
-START_TEST (conv_env_detect_different_objtype)
+START_TEST (detect_different_objtype)
 {
 	OSyncError *error = NULL;
 
@@ -55,7 +55,7 @@ START_TEST (conv_env_detect_different_objtype)
 	OSyncFormatConverter *conv = osync_converter_new_detector(format2, format1, detect, &error);
 	fail_unless(error == NULL);
 
-	osync_format_env_register_converter(env, conv);
+	osync_format_env_register_converter(env, conv, &error);
 
 	mark_point();
 
@@ -67,12 +67,12 @@ START_TEST (conv_env_detect_different_objtype)
 	fail_unless(osync_data_get_objformat(data) == format2);
 	
 	osync_data_unref(data);
-	osync_format_env_free(env);
+	osync_format_env_unref(env);
 }
 END_TEST
 
 
-START_TEST (conv_env_detect_smart_no)
+START_TEST (detect_smart_no)
 {
 	OSyncError *error = NULL;
 
@@ -86,7 +86,7 @@ START_TEST (conv_env_detect_smart_no)
 	OSyncFormatConverter *conv = osync_converter_new_detector(format2, format1, detect_false, &error);
 	fail_unless(error == NULL);
 
-	osync_format_env_register_converter(env, conv);
+	osync_format_env_register_converter(env, conv, &error);
 
 	mark_point();
 
@@ -98,32 +98,13 @@ START_TEST (conv_env_detect_smart_no)
 	fail_unless(osync_data_get_objformat(data) == format2);
 	
 	osync_data_unref(data);
-	osync_format_env_free(env);
+	osync_format_env_unref(env);
 }
 END_TEST
 
-Suite *env_suite(void)
-{
-	Suite *s = suite_create("Detect");
+OSYNC_TESTCASE_START("detect")
+OSYNC_TESTCASE_ADD(detect_smart)
+OSYNC_TESTCASE_ADD(detect_different_objtype)
+OSYNC_TESTCASE_ADD(detect_smart_no)
+OSYNC_TESTCASE_END
 
-	create_case(s, "conv_env_detect_smart", conv_env_detect_smart);
-	create_case(s, "conv_env_detect_different_objtype", conv_env_detect_different_objtype);
-	create_case(s, "conv_env_detect_smart_no", conv_env_detect_smart_no);
-
-	return s;
-}
-
-int main(void)
-{
-	int nf;
-	
-	Suite *s = env_suite();
-	
-	SRunner *sr;
-	sr = srunner_create(s);
-
-	srunner_run_all(sr, CK_VERBOSE);
-	nf = srunner_ntests_failed(sr);
-	srunner_free(sr);
-	return (nf == 0) ? EXIT_SUCCESS : EXIT_FAILURE;
-}
