@@ -83,7 +83,8 @@ START_TEST (hashtable_reload)
 
 	osync_change_set_uid(fakechange, "test1");
 
-	char *rndhash = osync_rand_str(g_random_int_range(100, 200));
+	char *rndhash = osync_rand_str(g_random_int_range(100, 200), &error);
+	osync_assert(error == NULL);
 
 	osync_change_set_hash(fakechange, rndhash);
 	osync_change_set_changetype(fakechange, OSYNC_CHANGE_TYPE_ADDED);
@@ -190,30 +191,9 @@ START_TEST (hashtable_stress)
 }
 END_TEST
 
-Suite *env_suite(void)
-{
-	Suite *s = suite_create("Hashtable");
+OSYNC_TESTCASE_START("hashtable")
+OSYNC_TESTCASE_ADD(hashtable_new)
+OSYNC_TESTCASE_ADD(hashtable_reload)
+OSYNC_TESTCASE_ADD(hashtable_stress)
+OSYNC_TESTCASE_END
 
-	create_case(s, "hashtable_new", hashtable_new);
-	create_case(s, "hashtable_reload", hashtable_reload);
-	create_case(s, "hashtable_stress", hashtable_stress);
-
-	return s;
-}
-
-int main(void)
-{
-	int nf;
-
-	check_env();
-	
-	Suite *s = env_suite();
-	
-	SRunner *sr;
-	sr = srunner_create(s);
-
-	srunner_run_all(sr, CK_VERBOSE);
-	nf = srunner_ntests_failed(sr);
-	srunner_free(sr);
-	return (nf == 0) ? EXIT_SUCCESS : EXIT_FAILURE;
-}
