@@ -3,13 +3,16 @@
 #include <opensync/opensync-plugin.h>
 #include <opensync/opensync-format.h>
 
+#include "opensync/plugin/opensync_plugin_config_private.h"
 #include "opensync/plugin/opensync_plugin_config_internals.h"
 
 static const char *_format_sink_get_objformat(OSyncPluginResource *res) {
 
 	OSyncObjFormatSink *format_sink;
-	format_sink = (OSyncObjFormatSink *) osync_list_nth_data(osync_plugin_resource_get_objformat_sinks(res), 0);
+	OSyncList *objformats = osync_plugin_resource_get_objformat_sinks(res);
+	format_sink = (OSyncObjFormatSink *) osync_list_nth_data(objformats, 0);
 	fail_unless(!!format_sink, NULL);
+	osync_list_free(objformats);
 	return osync_objformat_sink_get_objformat(format_sink);
 }
 
@@ -42,7 +45,7 @@ START_TEST (plugin_config_new_nomemory)
 {
 	char *testbed = setup_testbed(NULL);
 
-	setenv("OSYNC_NOMEMORY", "1", TRUE);
+	g_setenv("OSYNC_NOMEMORY", "1", TRUE);
 	
 	OSyncError *error = NULL;
 	OSyncPluginConfig *config = osync_plugin_config_new(&error);
@@ -135,7 +138,7 @@ START_TEST (plugin_config_subcomponents_nomemory)
 	fail_unless(error == NULL, NULL);
 	fail_unless(config != NULL, NULL);
 
-	setenv("OSYNC_NOMEMORY", "1", TRUE);
+	g_setenv("OSYNC_NOMEMORY", "1", TRUE);
 
 	/* Adding Subcomponents (without memory -> booooooom!): */
 	/* Connection */
@@ -168,11 +171,37 @@ START_TEST (plugin_config_subcomponents_nomemory)
 }
 END_TEST
 
+START_TEST (plugin_config_supported)
+{
+	char *testbed = setup_testbed(NULL);
+
+	OSyncError *error = NULL;
+	OSyncPluginConfig *config = osync_plugin_config_new(&error);
+	fail_unless(error == NULL, NULL);
+	fail_unless(config != NULL, NULL);
+
+	osync_plugin_config_set_supported(config, OPENSYNC_PLUGIN_CONFIG_ADVANCEDOPTION);
+	fail_unless(osync_plugin_config_is_supported(config, OPENSYNC_PLUGIN_CONFIG_ADVANCEDOPTION), NULL);
+	fail_unless(!osync_plugin_config_is_supported(config, OPENSYNC_PLUGIN_CONFIG_AUTHENTICATION), NULL);
+	fail_unless(!osync_plugin_config_is_supported(config, OPENSYNC_PLUGIN_CONFIG_LOCALIZATION), NULL);
+	fail_unless(!osync_plugin_config_is_supported(config, OPENSYNC_PLUGIN_CONFIG_RESOURCES), NULL);
+	fail_unless(!osync_plugin_config_is_supported(config, OPENSYNC_PLUGIN_CONFIG_CONNECTION), NULL);
+
+	osync_plugin_config_set_supported(config, OPENSYNC_PLUGIN_CONFIG_CONNECTION);
+	fail_unless(osync_plugin_config_is_supported(config, OPENSYNC_PLUGIN_CONFIG_CONNECTION), NULL);
+	fail_unless(!osync_plugin_config_is_supported(config, OPENSYNC_PLUGIN_CONFIG_ADVANCEDOPTION), NULL);
+
+	osync_plugin_config_unref(config);
+	destroy_testbed(testbed);
+}
+END_TEST
+
 START_TEST (plugin_config_advancedoption)
 {
 	char *testbed = setup_testbed(NULL);
 
 	OSyncError *error = NULL;
+	OSyncList *valenums = NULL;
 	OSyncPluginConfig *config = osync_plugin_config_new(&error);
 	fail_unless(error == NULL, NULL);
 	fail_unless(config != NULL, NULL);
@@ -182,11 +211,11 @@ START_TEST (plugin_config_advancedoption)
 	fail_unless(error == NULL, NULL);
 	fail_unless(option != NULL, NULL);
 
-	osync_plugin_advancedoption_set_maxsize(option, 2323);
-	fail_unless(osync_plugin_advancedoption_get_maxsize(option) == 2323, NULL);
+	osync_plugin_advancedoption_set_max(option, 2323);
+	fail_unless(osync_plugin_advancedoption_get_max(option) == 2323, NULL);
 
-	osync_plugin_advancedoption_set_minsize(option, 0);
-	fail_unless(osync_plugin_advancedoption_get_minsize(option) == 0, NULL);
+	osync_plugin_advancedoption_set_min(option, 0);
+	fail_unless(osync_plugin_advancedoption_get_min(option) == 0, NULL);
 
 	osync_plugin_advancedoption_set_maxoccurs(option, 3232);
 	fail_unless(osync_plugin_advancedoption_get_maxoccurs(option) == 3232, NULL);
@@ -202,11 +231,13 @@ START_TEST (plugin_config_advancedoption)
 	osync_plugin_advancedoption_add_valenum(option, "foo");
 
 	/* Expected result: 2 - 1x foo, 1x bar */
-	fail_unless(osync_list_length(osync_plugin_advancedoption_get_valenums(option)) == 2, NULL);
+	valenums = osync_plugin_advancedoption_get_valenums(option);
+	fail_unless(osync_list_length(valenums) == 2, NULL);
 
 	/* Expected resut: 1 - 1x bar */
 	osync_plugin_advancedoption_remove_valenum(option, "bar");
-	fail_unless(osync_list_length(osync_plugin_advancedoption_get_valenums(option)) == 1, NULL);
+	valenums = osync_plugin_advancedoption_get_valenums(option);
+	fail_unless(osync_list_length(valenums) == 1, NULL);
 
 	/*****/
 
@@ -234,11 +265,98 @@ START_TEST (plugin_config_advancedoption)
 }
 END_TEST
 
+START_TEST (plugin_config_advancedoption_set_get)
+{
+	char *testbed = setup_testbed(NULL);
+
+	OSyncError *error = NULL;
+	OSyncPluginConfig *config = osync_plugin_config_new(&error);
+	fail_unless(error == NULL, NULL);
+	fail_unless(config != NULL, NULL);
+
+	/* Advanced Option */
+	OSyncPluginAdvancedOption *option = osync_plugin_advancedoption_new(&error);
+	fail_unless(error == NULL, NULL);
+	fail_unless(option != NULL, NULL);
+	osync_plugin_advancedoption_set_name(option, "foo");
+	osync_plugin_advancedoption_set_value(option, "value for foo");
+
+	osync_plugin_config_add_advancedoption(config, option);
+	OSyncList *advanced_options;
+	advanced_options = osync_plugin_config_get_advancedoptions(config);
+	fail_unless(osync_list_length(advanced_options) == 1, NULL);
+	fail_unless(advanced_options->data == option, NULL);
+	osync_list_free(advanced_options);
+
+	const char *test_option = NULL;
+	test_option = osync_plugin_config_get_advancedoption_value_by_name(config, "foo");
+	fail_unless(test_option == osync_plugin_advancedoption_get_value(option));
+
+	test_option = osync_plugin_config_get_advancedoption_value_by_name(config, "foo2");
+	fail_unless(test_option == NULL, NULL);
+
+	/* Add option again to make sure it is not added twice */
+	osync_plugin_config_add_advancedoption(config, option);	
+	advanced_options = osync_plugin_config_get_advancedoptions(config);
+	fail_unless(osync_list_length(advanced_options) == 1, NULL);
+	fail_unless(advanced_options->data == option, NULL);
+	osync_list_free(advanced_options);
+
+	OSyncPluginAdvancedOption *option2 = osync_plugin_advancedoption_new(&error);
+	fail_unless(error == NULL, NULL);
+	fail_unless(option2 != NULL, NULL);
+	osync_plugin_advancedoption_set_name(option2, "bar");
+	osync_plugin_advancedoption_set_value(option2, "value for bar");
+	osync_plugin_config_add_advancedoption(config, option2);
+
+	advanced_options = osync_plugin_config_get_advancedoptions(config);
+	fail_unless(osync_list_length(advanced_options) == 2, NULL);
+	osync_list_free(advanced_options);
+	
+	test_option = osync_plugin_config_get_advancedoption_value_by_name(config, "foo");
+	fail_unless(test_option == osync_plugin_advancedoption_get_value(option));
+
+	test_option = osync_plugin_config_get_advancedoption_value_by_name(config, "bar");
+	fail_unless(test_option == osync_plugin_advancedoption_get_value(option2), NULL);
+
+	/* Removal */
+	OSyncPluginAdvancedOption *option3 = osync_plugin_advancedoption_new(&error);
+	fail_unless(error == NULL, NULL);
+	fail_unless(option3 != NULL, NULL);
+	osync_plugin_advancedoption_set_name(option3, "nonexistant");
+
+	osync_plugin_config_remove_advancedoption(config, option3);
+	advanced_options = osync_plugin_config_get_advancedoptions(config);
+	fail_unless(osync_list_length(advanced_options) == 2, NULL);
+	osync_list_free(advanced_options);
+
+	osync_plugin_config_remove_advancedoption(config, option);
+	advanced_options = osync_plugin_config_get_advancedoptions(config);
+	fail_unless(osync_list_length(advanced_options) == 1, NULL);
+	fail_unless(advanced_options->data == option2, NULL);
+	test_option = osync_plugin_config_get_advancedoption_value_by_name(config, "foo");
+	fail_unless(test_option == NULL);
+	test_option = osync_plugin_config_get_advancedoption_value_by_name(config, "bar");
+	fail_unless(test_option == osync_plugin_advancedoption_get_value(option2));
+	osync_list_free(advanced_options);
+
+	osync_plugin_config_remove_advancedoption(config, option2);
+	fail_unless(osync_plugin_config_get_advancedoptions(config) == NULL, NULL);
+
+	osync_plugin_advancedoption_unref(option);
+	osync_plugin_advancedoption_unref(option2);
+	osync_plugin_advancedoption_unref(option3);
+	osync_plugin_config_unref(config);
+	destroy_testbed(testbed);
+}
+END_TEST
+
 START_TEST (plugin_config_advancedoption_param)
 {
 	char *testbed = setup_testbed(NULL);
 
 	OSyncError *error = NULL;
+	OSyncList *valenums = NULL;
 	OSyncPluginConfig *config = osync_plugin_config_new(&error);
 	fail_unless(error == NULL, NULL);
 	fail_unless(config != NULL, NULL);
@@ -259,11 +377,13 @@ START_TEST (plugin_config_advancedoption_param)
 	osync_plugin_advancedoption_param_add_valenum(param, "foo");
 
 	/* Expected result: 2 - 1x foo, 1x bar */
-	fail_unless(osync_list_length(osync_plugin_advancedoption_param_get_valenums(param)) == 2, NULL);
+	valenums = osync_plugin_advancedoption_param_get_valenums(param);
+	fail_unless(osync_list_length(valenums) == 2, NULL);
 
 	/* Expected resut: 1 - 1x bar */
 	osync_plugin_advancedoption_param_remove_valenum(param, "bar");
-	fail_unless(osync_list_length(osync_plugin_advancedoption_param_get_valenums(param)) == 1, NULL);
+	valenums = osync_plugin_advancedoption_param_get_valenums(param);
+	fail_unless(osync_list_length(valenums) == 1, NULL);
 
 	/*****/
 
@@ -569,15 +689,20 @@ START_TEST (plugin_config_resources)
 	fail_unless(!strcmp(_format_sink_get_objformat(resource), "barfoo"), NULL);
 
 	/* Check for correct number objformat sinks: 2 sinks! */
-	fail_unless(osync_list_length(osync_plugin_resource_get_objformat_sinks(resource)) == 2, NULL);
-
+	OSyncList *objformats = osync_plugin_resource_get_objformat_sinks(resource);
+	fail_unless(osync_list_length(objformats) == 2, NULL);
+	osync_list_free(objformats);
 	/* Check if removing of an objformat sink work at all.. */
 	/** reference format_sink2, otherwise refcount gets 0 */
 	osync_objformat_sink_ref(format_sink2);
 	osync_plugin_resource_remove_objformat_sink(resource, format_sink2);
-	fail_unless(osync_list_length(osync_plugin_resource_get_objformat_sinks(resource)) == 1, NULL);
+	objformats = osync_plugin_resource_get_objformat_sinks(resource);
+	fail_unless(osync_list_length(objformats) == 1, NULL);
+	osync_list_free(objformats);
 	osync_plugin_resource_add_objformat_sink(resource, format_sink2);
-	fail_unless(osync_list_length(osync_plugin_resource_get_objformat_sinks(resource)) == 2, NULL);
+	objformats = osync_plugin_resource_get_objformat_sinks(resource);
+	fail_unless(osync_list_length(objformats) == 2, NULL);
+	osync_list_free(objformats);
 	osync_objformat_sink_unref(format_sink2);
 
 	/* ObjType */
@@ -622,7 +747,6 @@ START_TEST (plugin_config_resources)
 
 	fail_unless(osync_plugin_resource_ref(resource) != NULL, NULL);
 	osync_plugin_resource_unref(resource);
-	osync_plugin_resource_unref(resource);
 
 	osync_plugin_config_unref(config);
 
@@ -635,6 +759,8 @@ START_TEST (plugin_config_save_and_load)
 	char *testbed = setup_testbed(NULL);
 
 	OSyncError *error = NULL;
+	OSyncList *valenums = NULL;
+	OSyncList *paramvalenums = NULL;
 	OSyncPluginConfig *config = osync_plugin_config_new(&error);
 	OSyncPluginConfig *reloaded_config = osync_plugin_config_new(&error); 
 	fail_unless(error == NULL, NULL);
@@ -645,8 +771,8 @@ START_TEST (plugin_config_save_and_load)
 	fail_unless(error == NULL, NULL);
 	fail_unless(option != NULL, NULL);
 
-	osync_plugin_advancedoption_set_maxsize(option, 1);
-	osync_plugin_advancedoption_set_minsize(option, 1);
+	osync_plugin_advancedoption_set_max(option, 1);
+	osync_plugin_advancedoption_set_min(option, 1);
 	osync_plugin_advancedoption_set_maxoccurs(option, 1);
 	osync_plugin_advancedoption_set_displayname(option, "foobar1");
 	osync_plugin_advancedoption_set_name(option, "foobar1");
@@ -655,8 +781,10 @@ START_TEST (plugin_config_save_and_load)
 
 	osync_plugin_advancedoption_add_valenum(option, "FOOBAR1");
 	osync_plugin_advancedoption_add_valenum(option, "FOOBAR2");
-	fail_unless(osync_list_length(osync_plugin_advancedoption_get_valenums(option)) == 2, NULL);
-
+	valenums = osync_plugin_advancedoption_get_valenums(option);
+	fail_unless(osync_list_length(valenums) == 2, NULL);
+	osync_list_free(valenums);
+	
 	/* Advanced Parameter */
 	OSyncPluginAdvancedOptionParameter *param = osync_plugin_advancedoption_param_new(&error);
 	fail_unless(error == NULL, NULL);
@@ -670,8 +798,11 @@ START_TEST (plugin_config_save_and_load)
 
 	osync_plugin_advancedoption_param_add_valenum(param, "BAR1");
 	osync_plugin_advancedoption_param_add_valenum(param, "BAR2");
-	fail_unless(osync_list_length(osync_plugin_advancedoption_param_get_valenums(param)) == 2, NULL);
+	paramvalenums = osync_plugin_advancedoption_param_get_valenums(param);
+	fail_unless(osync_list_length(paramvalenums) == 2, NULL);
+	osync_list_free(paramvalenums);
 	osync_plugin_advancedoption_add_parameter(option, param);
+	osync_plugin_advancedoption_param_unref(param);
 
 	/* Localization */
 	OSyncPluginLocalization *local = osync_plugin_localization_new(&error);
@@ -710,6 +841,7 @@ START_TEST (plugin_config_save_and_load)
 	osync_plugin_resource_set_name(resource1, "foobar1");
 	osync_plugin_resource_set_mime(resource1, "foobar1");
 	osync_plugin_resource_add_objformat_sink(resource1, format_sink1);
+	osync_objformat_sink_unref(format_sink1);
 	osync_plugin_resource_set_objtype(resource1, "foobar1");
 	osync_plugin_resource_set_path(resource1, "foobar1");
 	osync_plugin_resource_set_url(resource1, "foobar1");
@@ -727,6 +859,7 @@ START_TEST (plugin_config_save_and_load)
 	osync_plugin_resource_set_name(resource2, "foobar2");
 	osync_plugin_resource_set_mime(resource2, "foobar2");
 	osync_plugin_resource_add_objformat_sink(resource2, format_sink2);
+	osync_objformat_sink_unref(format_sink2);
 	osync_plugin_resource_set_objtype(resource2, "foobar2");
 	osync_plugin_resource_set_path(resource2, "foobar2");
 	osync_plugin_resource_set_url(resource2, "foobar2");
@@ -745,7 +878,8 @@ START_TEST (plugin_config_save_and_load)
 
 	char *config_file = g_strdup_printf("%s/dummy_config.xml", testbed);
 	fail_unless(osync_plugin_config_file_save(config, config_file, &error), "%s", osync_error_print(&error));
-	fail_unless(osync_plugin_config_file_load(reloaded_config, config_file, testbed, &error), NULL);
+	osync_plugin_config_set_schemadir(reloaded_config, testbed);
+	fail_unless(osync_plugin_config_file_load(reloaded_config, config_file, &error), NULL);
 	g_free(config_file);
 
 	/* Compare stored config with original config */
@@ -772,7 +906,7 @@ START_TEST (plugin_config_save_and_load)
 	fail_unless(!strcmp(osync_plugin_authentication_get_reference(reloaded_auth), "ref"), NULL);
 
 	OSyncList *r, *p, *v;
-	int i, j, k;
+	unsigned int i, j, k;
 	for (i = 1, r = reloaded_resources; r; r = r->next, i++) {
 		char *value = g_strdup_printf("foobar%i", i);
 		fail_unless(!strcmp(osync_plugin_resource_get_name(r->data), value), NULL);
@@ -789,8 +923,8 @@ START_TEST (plugin_config_save_and_load)
 		char *value = g_strdup_printf("foobar%i", i);
 		fail_unless(!strcmp(osync_plugin_advancedoption_get_name(r->data), value), NULL);
 		fail_unless(!strcmp(osync_plugin_advancedoption_get_displayname(r->data), value), NULL);
-		fail_unless(osync_plugin_advancedoption_get_maxsize(r->data) == i, NULL);
-		fail_unless(osync_plugin_advancedoption_get_minsize(r->data) == i, NULL);
+		fail_unless(osync_plugin_advancedoption_get_max(r->data) == i, NULL);
+		fail_unless(osync_plugin_advancedoption_get_min(r->data) == i, NULL);
 		fail_unless(osync_plugin_advancedoption_get_maxoccurs(r->data) == i, NULL);
 		fail_unless(osync_plugin_advancedoption_get_type(r->data) == OSYNC_PLUGIN_ADVANCEDOPTION_TYPE_CHAR, NULL);
 		g_free(value);
@@ -801,9 +935,9 @@ START_TEST (plugin_config_save_and_load)
 			fail_unless(!strcmp(v->data, value), NULL);
 			g_free(value);
 		}
-
 		fail_unless(!strcmp(osync_plugin_advancedoption_get_value(r->data), "FOOBAR1"));
-
+		osync_list_free(reloaded_advancedoption_valenums);
+		
 		reloaded_advancedoption_parameters = osync_plugin_advancedoption_get_parameters(r->data);
 		for (j = 1, p = reloaded_advancedoption_parameters; p; p = p->next, j++) {
 			char *value = g_strdup_printf("bar%i", j);
@@ -818,15 +952,18 @@ START_TEST (plugin_config_save_and_load)
 				g_free(value);
 
 			}
+			osync_list_free(reloaded_advancedoption_param_valenums);
 
 			fail_unless(!strcmp(osync_plugin_advancedoption_param_get_value(p->data), "BAR1"));
 
 		}
-
+		osync_list_free(reloaded_advancedoption_parameters);
 
 	}
 
-
+	osync_list_free(reloaded_resources);
+	osync_list_free(reloaded_advancedoptions);
+	
 	osync_plugin_config_unref(config);
 	osync_plugin_config_unref(reloaded_config);
 
@@ -849,6 +986,7 @@ START_TEST (plugin_config_save_and_load_connection_bluetooth)
 	fail_unless(error == NULL, NULL);
 	fail_unless(conn != NULL, NULL);
 
+	osync_plugin_connection_set_supported(conn, OSYNC_PLUGIN_CONNECTION_BLUETOOTH);
 	osync_plugin_connection_set_type(conn, OSYNC_PLUGIN_CONNECTION_BLUETOOTH);
 
 	/* Bluetooth Address */
@@ -867,7 +1005,8 @@ START_TEST (plugin_config_save_and_load_connection_bluetooth)
 
 	char *config_file = g_strdup_printf("%s/dummy_config.xml", testbed);
 	fail_unless(osync_plugin_config_file_save(config, config_file, &error), "%s", osync_error_print(&error));
-	fail_unless(osync_plugin_config_file_load(reloaded_config, config_file, testbed, &error), NULL);
+	osync_plugin_config_set_schemadir(reloaded_config, testbed);
+	fail_unless(osync_plugin_config_file_load(reloaded_config, config_file, &error), NULL);
 	g_free(config_file);
 
 	/* Compare stored config with original config */
@@ -902,6 +1041,7 @@ START_TEST (plugin_config_save_and_load_connection_irda)
 	fail_unless(error == NULL, NULL);
 	fail_unless(conn != NULL, NULL);
 
+	osync_plugin_connection_set_supported(conn, OSYNC_PLUGIN_CONNECTION_IRDA);
 	osync_plugin_connection_set_type(conn, OSYNC_PLUGIN_CONNECTION_IRDA);
 
 	osync_plugin_connection_irda_set_service(conn, "FancyIR Mobile");
@@ -912,7 +1052,8 @@ START_TEST (plugin_config_save_and_load_connection_irda)
 
 	char *config_file = g_strdup_printf("%s/dummy_config.xml", testbed);
 	fail_unless(osync_plugin_config_file_save(config, config_file, &error), "%s", osync_error_print(&error));
-	fail_unless(osync_plugin_config_file_load(reloaded_config, config_file, testbed, &error), NULL);
+	osync_plugin_config_set_schemadir(reloaded_config, testbed);
+	fail_unless(osync_plugin_config_file_load(reloaded_config, config_file, &error), NULL);
 	g_free(config_file);
 
 	/* Compare stored config with original config */
@@ -945,6 +1086,7 @@ START_TEST (plugin_config_save_and_load_connection_usb)
 	fail_unless(error == NULL, NULL);
 	fail_unless(conn != NULL, NULL);
 
+	osync_plugin_connection_set_supported(conn, OSYNC_PLUGIN_CONNECTION_USB);
 	osync_plugin_connection_set_type(conn, OSYNC_PLUGIN_CONNECTION_USB);
 
 	/* USB Vendor ID */
@@ -962,7 +1104,8 @@ START_TEST (plugin_config_save_and_load_connection_usb)
 
 	char *config_file = g_strdup_printf("%s/dummy_config.xml", testbed);
 	fail_unless(osync_plugin_config_file_save(config, config_file, &error), "%s", osync_error_print(&error));
-	fail_unless(osync_plugin_config_file_load(reloaded_config, config_file, testbed, &error), NULL);
+	osync_plugin_config_set_schemadir(reloaded_config, testbed);
+	fail_unless(osync_plugin_config_file_load(reloaded_config, config_file, &error), NULL);
 	g_free(config_file);
 
 	/* Compare stored config with original config */
@@ -996,6 +1139,7 @@ START_TEST (plugin_config_save_and_load_connection_network)
 	fail_unless(error == NULL, NULL);
 	fail_unless(conn != NULL, NULL);
 
+	osync_plugin_connection_set_supported(conn, OSYNC_PLUGIN_CONNECTION_NETWORK);
 	osync_plugin_connection_set_type(conn, OSYNC_PLUGIN_CONNECTION_NETWORK);
 
 
@@ -1017,7 +1161,8 @@ START_TEST (plugin_config_save_and_load_connection_network)
 
 	char *config_file = g_strdup_printf("%s/dummy_config.xml", testbed);
 	fail_unless(osync_plugin_config_file_save(config, config_file, &error), "%s", osync_error_print(&error));
-	fail_unless(osync_plugin_config_file_load(reloaded_config, config_file, testbed, &error), NULL);
+	osync_plugin_config_set_schemadir(reloaded_config, testbed);
+	fail_unless(osync_plugin_config_file_load(reloaded_config, config_file, &error), NULL);
 	g_free(config_file);
 
 	/* Compare stored config with original config */
@@ -1052,6 +1197,7 @@ START_TEST (plugin_config_save_and_load_connection_serial)
 	fail_unless(error == NULL, NULL);
 	fail_unless(conn != NULL, NULL);
 
+	osync_plugin_connection_set_supported(conn, OSYNC_PLUGIN_CONNECTION_SERIAL);
 	osync_plugin_connection_set_type(conn, OSYNC_PLUGIN_CONNECTION_SERIAL);
 
 	/* Serial Speed */
@@ -1066,7 +1212,8 @@ START_TEST (plugin_config_save_and_load_connection_serial)
 
 	char *config_file = g_strdup_printf("%s/dummy_config.xml", testbed);
 	fail_unless(osync_plugin_config_file_save(config, config_file, &error), "%s", osync_error_print(&error));
-	fail_unless(osync_plugin_config_file_load(reloaded_config, config_file, testbed, &error), NULL);
+	osync_plugin_config_set_schemadir(reloaded_config, testbed);
+	fail_unless(osync_plugin_config_file_load(reloaded_config, config_file, &error), NULL);
 	g_free(config_file);
 
 	/* Compare stored config with original config */
@@ -1084,41 +1231,24 @@ START_TEST (plugin_config_save_and_load_connection_serial)
 }
 END_TEST
 
-Suite *client_suite(void)
-{
-	Suite *s = suite_create("PluginConfig");
-//	Suite *s2 = suite_create("PluginConfig");
-	
-	create_case(s, "plugin_config_new", plugin_config_new);
-	create_case(s, "plugin_config_new_nomemory", plugin_config_new_nomemory);
-	create_case(s, "plugin_config_subcomponents", plugin_config_subcomponents);
-	create_case(s, "plugin_config_subcomponents_nomemory", plugin_config_subcomponents_nomemory);
-	create_case(s, "plugin_config_advancedoption", plugin_config_advancedoption);
-	create_case(s, "plugin_config_advancedoption_param", plugin_config_advancedoption_param);
-	create_case(s, "plugin_config_authentication", plugin_config_authentication);
-	create_case(s, "plugin_config_connection", plugin_config_connection);
-	create_case(s, "plugin_config_localization", plugin_config_localization);
-	create_case(s, "plugin_config_resources", plugin_config_resources);
-	create_case(s, "plugin_config_save_and_load", plugin_config_save_and_load);
-	create_case(s, "plugin_config_save_and_load_connection_bluetooth", plugin_config_save_and_load_connection_bluetooth);
-	create_case(s, "plugin_config_save_and_load_connection_usb", plugin_config_save_and_load_connection_usb);
-	create_case(s, "plugin_config_save_and_load_connection_irda", plugin_config_save_and_load_connection_irda);
-	create_case(s, "plugin_config_save_and_load_connection_network", plugin_config_save_and_load_connection_network);
-	create_case(s, "plugin_config_save_and_load_connection_serial", plugin_config_save_and_load_connection_serial);
-	
-	return s;
-}
+OSYNC_TESTCASE_START("plugin_config")
+OSYNC_TESTCASE_ADD(plugin_config_new)
+OSYNC_TESTCASE_ADD(plugin_config_new_nomemory)
+OSYNC_TESTCASE_ADD(plugin_config_supported)
+OSYNC_TESTCASE_ADD(plugin_config_subcomponents)
+OSYNC_TESTCASE_ADD(plugin_config_subcomponents_nomemory)
+OSYNC_TESTCASE_ADD(plugin_config_advancedoption)
+OSYNC_TESTCASE_ADD(plugin_config_advancedoption_set_get)
+OSYNC_TESTCASE_ADD(plugin_config_advancedoption_param)
+OSYNC_TESTCASE_ADD(plugin_config_authentication)
+OSYNC_TESTCASE_ADD(plugin_config_connection)
+OSYNC_TESTCASE_ADD(plugin_config_localization)
+OSYNC_TESTCASE_ADD(plugin_config_resources)
+OSYNC_TESTCASE_ADD(plugin_config_save_and_load)
+OSYNC_TESTCASE_ADD(plugin_config_save_and_load_connection_bluetooth)
+OSYNC_TESTCASE_ADD(plugin_config_save_and_load_connection_usb)
+OSYNC_TESTCASE_ADD(plugin_config_save_and_load_connection_irda)
+OSYNC_TESTCASE_ADD(plugin_config_save_and_load_connection_network)
+OSYNC_TESTCASE_ADD(plugin_config_save_and_load_connection_serial)
+OSYNC_TESTCASE_END
 
-int main(void)
-{
-	int nf;
-
-	Suite *s = client_suite();
-	
-	SRunner *sr;
-	sr = srunner_create(s);
-	srunner_run_all(sr, CK_VERBOSE);
-	nf = srunner_ntests_failed(sr);
-	srunner_free(sr);
-	return (nf == 0) ? EXIT_SUCCESS : EXIT_FAILURE;
-}
