@@ -1,28 +1,88 @@
 #include "support.h"
+#ifndef _WIN32
+#include <sys/wait.h> /* For WEXITSTATUS */
+#endif
+
+#include "opensync/engine/opensync_engine_internals.h"
+#include "opensync/archive/opensync_archive_internals.h"
+#include "opensync/group/opensync_group_internals.h"
+#include "opensync/mapping/opensync_mapping_table_internals.h"
+#include "opensync/mapping/opensync_mapping_internals.h"
 
 char *olddir = NULL;
 
+
+void osync_testsuite_all(Suite *s, struct osync_testcase_s *tc)
+{
+	unsigned int i;
+	for (i=0; tc[i].name; i++)
+		create_case(s, tc[i].name, tc[i].func);
+}
+
+unsigned int osync_testsuite_selected(Suite *s, int argc, char **argv,
+		struct osync_testcase_s *tc)
+{
+	int i, j, n=0;
+	/* Also argv[0]! for symlink-ed calls */
+	for (i=0; argc > i; i++) {
+		for (j=0; tc[j].name; j++) {
+			if (strcmp(g_basename(argv[i]), tc[j].name))
+					continue;
+
+			create_case(s, tc[j].name, tc[j].func);
+			n++;
+		}
+	}
+
+	return n;
+}
+
+int osync_testsuite(int argc, char **argv, const char *unittest,
+		struct osync_testcase_s *tc)
+{
+	int nf;
+	Suite *s = suite_create(unittest);
+	SRunner *sr;
+
+	check_env();
+
+	if (!osync_testsuite_selected(s, argc, argv, tc))
+		osync_testsuite_all(s, tc);
+
+	sr = srunner_create(s);
+	srunner_run_all(sr, CK_VERBOSE);
+	nf = srunner_ntests_failed(sr);
+	srunner_free(sr);
+	return (nf == 0) ? EXIT_SUCCESS : EXIT_FAILURE;
+}
+
 static void reset_env(void)
 {
-	unsetenv("CONNECT_ERROR");
-	unsetenv("CONNECT_TIMEOUT");
-	unsetenv("CONNECT_SLOWSYNC");
-	unsetenv("INIT_NULL");
-	unsetenv("GET_CHANGES_ERROR");
-	unsetenv("GET_CHANGES_TIMEOUT");
-	unsetenv("GET_CHANGES_TIMEOUT2");
-	unsetenv("COMMIT_ERROR");
-	unsetenv("COMMIT_TIMEOUT");
-	unsetenv("SYNC_DONE_ERROR");
-	unsetenv("SYNC_DONE_TIMEOUT");
-	unsetenv("DISCONNECT_ERROR");
-	unsetenv("DISCONNECT_TIMEOUT");
-	unsetenv("BATCH_COMMIT");
-	unsetenv("COMMITTED_ALL_ERROR");
-	unsetenv("NO_COMMITTED_ALL_CHECK");
-	unsetenv("MAINSINK_CONNECT");
+	g_unsetenv("CONNECT_ERROR");
+	g_unsetenv("CONNECT_TIMEOUT");
+	g_unsetenv("CONNECT_SLOWSYNC");
+	g_unsetenv("INIT_NULL");
+	g_unsetenv("INIT_NULL_NOERROR");
+	g_unsetenv("GET_CHANGES_ERROR");
+	g_unsetenv("GET_CHANGES_TIMEOUT");
+	g_unsetenv("GET_CHANGES_TIMEOUT2");
+	g_unsetenv("COMMIT_ERROR");
+	g_unsetenv("COMMIT_TIMEOUT");
+	g_unsetenv("SYNC_DONE_ERROR");
+	g_unsetenv("SYNC_DONE_TIMEOUT");
+	g_unsetenv("DISCONNECT_ERROR");
+	g_unsetenv("DISCONNECT_TIMEOUT");
+	g_unsetenv("COMMITTED_ALL_ERROR");
+	g_unsetenv("NO_COMMITTED_ALL_CHECK");
+	g_unsetenv("MAINSINK_CONNECT");
 
-	unsetenv("OSYNC_NOMEMORY");
+	g_unsetenv("OSYNC_NOMEMORY");
+	g_unsetenv("MOCK_SYNC_EXPECT_COMMIT_ALWAYS_ADDED");
+	g_unsetenv("MOCK_SYNC_ALWAYS_CHANGETYPE_MODIFIED");
+
+	g_unsetenv("MOCK_FORMAT_PATH_COMPARE_NO");
+
+	g_unsetenv("MOCK_DISCOVER_ERROR");
 }
 
 
@@ -39,40 +99,54 @@ void check_env(void) {
 
 char *setup_testbed(const char *fkt_name)
 {
-	
+#ifndef _WIN32	
 	setuid(65534);
+#endif
 	char *testbed = g_strdup_printf("%s/testbed.XXXXXX", g_get_tmp_dir());
-	mkdtemp(testbed);
+	char *command = NULL, *dirname = NULL;
+#ifdef _WIN32
+	if(g_file_test(testbed, G_FILE_TEST_IS_DIR))
+		destroy_testbed(g_strdup(testbed));
+
+	if(g_mkdir(testbed,0777) < 0){
+			osync_trace(TRACE_INTERNAL, "%s: Cannot create testbed directory %s", __func__, testbed);
+			abort();
+	}
+#else /* WIN32 */
+	if (!mkdtemp(testbed))
+		abort();
+#endif /* WIN32 */
 	
-	char *command = NULL;
 	if (fkt_name) {
-		char *dirname = g_strdup_printf(OPENSYNC_TESTDATA"/%s", fkt_name);
+		dirname = g_strdup_printf(OPENSYNC_TESTDATA"/%s", fkt_name);
 		if (!g_file_test(dirname, G_FILE_TEST_IS_DIR)) {
 			osync_trace(TRACE_INTERNAL, "%s: Path %s not exist.", __func__, dirname);
 			abort();
 		}
 		command = g_strdup_printf("cp -R %s/* %s", dirname, testbed);
-		osync_trace(TRACE_INTERNAL, "tb_cmd: %s", command);
 		if (system(command))
 			abort();
 		g_free(command);
 		g_free(dirname);
 	}
 	
-	/*command = g_strdup_printf("cp -R ../osplugin/osplugin %s", testbed);
-	if (system(command))
-		abort();
-	g_free(command);*/
-	
-	command = g_strdup_printf("mkdir %s/formats",  testbed);
-	if (system(command))
-		abort();
-	g_free(command);
-	
-	command = g_strdup_printf("mkdir %s/plugins",  testbed);
-	if (system(command))
-		abort();
-	g_free(command);
+	dirname = g_strdup_printf("%s/formats",  testbed);
+	if (!g_file_test(dirname, G_FILE_TEST_IS_DIR)) {
+		if (g_mkdir(dirname,0777) < 0) {
+			osync_trace(TRACE_ERROR, "Could not create format direcotry %s", dirname);
+			abort();
+		}
+	}
+	g_free(dirname);
+
+	dirname = g_strdup_printf("%s/plugins",  testbed);
+	if (!g_file_test(dirname, G_FILE_TEST_IS_DIR)) {
+		if (g_mkdir(dirname,0777) < 0) {
+			osync_trace(TRACE_ERROR, "Could not create plugin direcotry %s", dirname);
+			abort();
+		}
+	}
+	g_free(dirname);
 	
 	command = g_strdup_printf("cp ./mock-plugin/mock-sync.%s %s/plugins", G_MODULE_SUFFIX, testbed);
 	if (system(command))
@@ -83,25 +157,24 @@ char *setup_testbed(const char *fkt_name)
 	if (system(command))
 		abort();
 	g_free(command);
-	
-	command = g_strdup_printf("cp -R ../formats/*.%s %s/formats", G_MODULE_SUFFIX, testbed);
-	if (system(command))
-		abort();
-	g_free(command);
 
 	command = g_strdup_printf("cp -R %s/../../misc/schemas/*.xsd %s", OPENSYNC_TESTDATA, testbed);
 	if (system(command))
 		abort();
 	g_free(command);
-	
+
+#ifndef _WIN32	/* chmod is useless on windows system */
 	command = g_strdup_printf("chmod -R 700 %s", testbed);
 	if (system(command))
 		abort();
 	g_free(command);
+#endif
 		
 	olddir = g_get_current_dir();
-	if (chdir(testbed))
+	if (g_chdir(testbed) < 0){
+		osync_trace(TRACE_ERROR, "Could not chdir to testbed");
 		abort();
+	}
 	
 	reset_counters();
 
@@ -116,10 +189,13 @@ void destroy_testbed(char *path)
 {
 	char *command = g_strdup_printf("rm -rf %s", path);
 	if (olddir) {
-		chdir(olddir);
+		if (g_chdir(olddir) < 0)
+			abort();
 		g_free(olddir);
 	}
-	system(command);
+	if (system(command))
+		abort();
+
 	g_free(command);
 	osync_trace(TRACE_INTERNAL, "Tearing down %s", path);
 	g_free(path);
@@ -143,16 +219,19 @@ void create_case_timeout(Suite *s, const char *name, TFun function, int timeout)
 	tcase_add_test(tc_new, function);
 }
 
-OSyncMappingTable *mappingtable_load(const char *path, const char *objtype, int num_mappings)
+OSyncMappingTable *mappingtable_load(const char *path, const char *objtype, unsigned int num_mappings)
 {
+	OSyncError *error = NULL;
+	OSyncMappingTable *table = NULL;
+	OSyncArchive *archive = NULL;
+
 	osync_trace(TRACE_ENTRY, "%s(%s, %s, %i)", __func__, path, objtype, num_mappings);
 	
-	OSyncError *error = NULL;
-	OSyncMappingTable *table = osync_mapping_table_new(&error);
+	table = osync_mapping_table_new(&error);
 	fail_unless(table != NULL, NULL);
 	fail_unless(error == NULL, NULL);
 	
-	OSyncArchive *archive = osync_archive_new(path, &error);
+	archive = osync_archive_new(path, &error);
 	fail_unless(archive != NULL, NULL);
 	fail_unless(error == NULL, NULL);
 	
@@ -167,67 +246,57 @@ OSyncMappingTable *mappingtable_load(const char *path, const char *objtype, int
 	return table;
 }
 
-void check_mapping(OSyncMappingTable *maptable, int memberid, int mappingid, int numentries, const char *uid)
+void check_mapping(OSyncMappingTable *maptable, int memberid, int mappingid, unsigned int numentries, const char *uid)
 {
+	unsigned int i = 0;
 	osync_trace(TRACE_ENTRY, "%s(%p, %i, %i, %i, %s)", __func__, maptable, memberid, mappingid, numentries, uid);
 	
-	int i = 0;
 	for (i = 0; i < osync_mapping_table_num_mappings(maptable); i++) {
 		OSyncMapping *mapping = osync_mapping_table_nth_mapping(maptable, i);
 		OSyncMappingEntry *testentry = osync_mapping_find_entry_by_member_id(mapping, memberid);
-		if ((mappingid != -1 && osync_mapping_get_id(mapping) == mappingid) || (mappingid == -1 && !strcmp(osync_mapping_entry_get_uid(testentry), uid))) {
-			fail_unless(osync_mapping_num_entries(mapping) == numentries);
-			int n = 0;
-			for (n = 0; n < osync_mapping_num_entries(mapping); n++) {
-				OSyncMappingEntry *entry = osync_mapping_nth_entry(mapping, n);
-				if (osync_mapping_entry_get_member_id(entry) == memberid) {
-					fail_unless(!strcmp(osync_mapping_entry_get_uid(entry), uid), NULL);
-					goto out;
+		if (testentry) {
+			if ((mappingid != -1 && osync_mapping_get_id(mapping) == mappingid) || (mappingid == -1 && !strcmp(osync_mapping_entry_get_uid(testentry), uid))) {
+				unsigned int n = 0;
+				fail_unless(osync_mapping_num_entries(mapping) == numentries);
+				for (n = 0; n < osync_mapping_num_entries(mapping); n++) {
+					OSyncMappingEntry *entry = osync_mapping_nth_entry(mapping, n);
+					if (osync_mapping_entry_get_member_id(entry) == memberid) {
+						fail_unless(!strcmp(osync_mapping_entry_get_uid(entry), uid), NULL);
+						goto out;
+					}
 				}
+				fail(NULL);
 			}
-			fail(NULL);
 		}
 	}
 	fail(NULL);
 
 out:
-	/*OSyncMember *member = osync_member_from_id(maptable->group, memberid);
-	OSyncMappingView *view = osengine_mappingtable_find_view(maptable, member);
-	mark_point();
-	if (mappingid != -1) {
-		mapping = g_list_nth_data(maptable->mappings, mappingid);
-	} else {
-		GList *m;
-		for (m = maptable->mappings; m; m = m->next) {
-			mapping = m->data;
-			OSyncMappingEntry *entry = osengine_mapping_find_entry(mapping, NULL, view);
-			if (!entry)
-				continue;
-			OSyncChange *change = entry->change;
-			fail_unless(change != NULL, NULL);
-			if (!strcmp(osync_change_get_uid(change), uid))
-				break;
-		}
-	}
-	fail_unless(mapping != NULL, NULL);
-	fail_unless(osengine_mapping_num_changes(mapping) == numentries, "osengine_mapping_num_changes(mapping) == numentries for %s, %i: %i != %i", uid, memberid, osengine_mapping_num_changes(mapping), numentries);*/
-	
-	
-	
-	/*OSyncChange *change = osengine_mapping_find_entry(mapping, NULL, view)->change;
-	fail_unless(change != NULL, NULL);
-	if (format)
-		fail_unless(!strcmp(osync_objformat_get_name(osync_change_get_objformat(change)), format), NULL);
-	if (objecttype)
-		fail_unless(!strcmp(osync_objtype_get_name(osync_change_get_objtype(change)), objecttype), NULL);
-	if (uid && strcmp(osync_change_get_uid(change), uid)) {
-		printf("uid mismatch: %s != %s for member %i and mapping %i\n", osync_change_get_uid(change), uid, memberid, mappingid);
-		fail("uid mismatch");
-	}*/
 	osync_trace(TRACE_EXIT, "%s", __func__);
 }
 
-OSyncHashTable *hashtable_load(const char *path, const char *objtype, int entries)
+void validate_mapping_table(const char *testbed, unsigned int num_members, const char *uids[], unsigned int num_uids)
+{
+	unsigned int member;
+	unsigned int uid;
+	osync_trace(TRACE_ENTRY, "%s(%s, %i, %p, %i)", __func__, testbed, num_members, uids, num_uids);
+
+	char *path = g_strdup_printf("%s/configs/group/archive.db", testbed);
+	OSyncMappingTable *maptable = mappingtable_load(path, "mockobjtype1", num_uids);
+	g_free(path);
+
+	/* check we have num_members mapping entries for each uid */
+	for (uid = 0; uid < num_uids; uid++) {
+		for (member = 1; member <= num_members; member++) {
+			check_mapping(maptable, member, -1, num_members, uids[uid]);
+		}
+	}
+	osync_mapping_table_close(maptable);
+	osync_mapping_table_unref(maptable);
+	osync_trace(TRACE_EXIT, "%s", __func__);
+}
+
+OSyncHashTable *hashtable_load(const char *path, const char *objtype, unsigned int entries)
 {
 	OSyncError *error = NULL;
 	OSyncHashTable *table = osync_hashtable_new(path, objtype, &error);
@@ -237,8 +306,8 @@ OSyncHashTable *hashtable_load(const char *path, const char *objtype, int entrie
 	fail_unless(osync_hashtable_load(table, &error), NULL);
 	
 	fail_unless(osync_hashtable_num_entries(table) == entries, NULL);
-    
-    return table;
+
+	return table;
 }
 
 void check_hash(OSyncHashTable *table, const char *cmpuid)
@@ -249,78 +318,93 @@ void check_hash(OSyncHashTable *table, const char *cmpuid)
 
 void create_random_file(const char *path)
 {
-	char *content = osync_rand_str(g_random_int_range(100, 200));
+	OSyncError *error = NULL;
+	char *content = osync_rand_str(g_random_int_range(100, 200), &error);
+	fail_unless(error == NULL, NULL);
 	osync_assert(osync_file_write(path, content, strlen(content), 0700, NULL) == TRUE);
 	g_free(content);
 }
 
-void member_status(OSyncMemberUpdate *status, void *user_data)
+void member_status(OSyncEngineMemberUpdate *status, void *user_data)
 {
-	osync_trace(TRACE_ENTRY, "%s(%p (%i), %p)", __func__, status, status->type, user_data);
+	osync_trace(TRACE_ENTRY, "%s(%p), %p)", __func__, status, user_data);
 	fail_unless(GINT_TO_POINTER(1) == user_data, NULL);
+
+        OSyncError *error = osync_engine_member_update_get_error(status);
+        const char *objtype = osync_engine_member_update_get_objtype(status);
 	
-	switch (status->type) {
-		case OSYNC_CLIENT_EVENT_CONNECTED:
-			fail_unless(!osync_error_is_set(&(status->error)), NULL);
+	switch (osync_engine_member_update_get_event(status)) {
+		case OSYNC_ENGINE_MEMBER_EVENT_CONNECTED:
+			fail_unless(!osync_error_is_set(&(error)), NULL);
 			
-			if (status->objtype == NULL) {
+			if (objtype == NULL) {
 				num_client_main_connected++;
 			} else {
-				fail_unless(!strncmp(status->objtype, "mockobjtype", 11), NULL);
+				fail_unless(!strncmp(objtype, "mockobjtype", 11), NULL);
 				num_client_connected++;
 			}
 			
 			break;
-		case OSYNC_CLIENT_EVENT_DISCONNECTED:
-			fail_unless(!osync_error_is_set(&(status->error)), NULL);
+		case OSYNC_ENGINE_MEMBER_EVENT_CONNECT_DONE:
+			fail_unless(!osync_error_is_set(&(error)), NULL);
 			
-			if (status->objtype == NULL) {
+			if (objtype == NULL) {
+				num_client_main_connect_done++;
+			} else {
+				fail_unless(!strncmp(objtype, "mockobjtype", 11), NULL);
+				num_client_connect_done++;
+			}
+			break;
+		case OSYNC_ENGINE_MEMBER_EVENT_DISCONNECTED:
+			fail_unless(!osync_error_is_set(&(error)), NULL);
+			
+			if (objtype == NULL) {
 				num_client_main_disconnected++;
 			} else {
-				fail_unless(!strncmp(status->objtype, "mockobjtype", 11), NULL);
+				fail_unless(!strncmp(objtype, "mockobjtype", 11), NULL);
 				num_client_disconnected++;
 			}
 			
 			break;
-		case OSYNC_CLIENT_EVENT_READ:
-			fail_unless(!osync_error_is_set(&(status->error)), NULL);
+		case OSYNC_ENGINE_MEMBER_EVENT_READ:
+			fail_unless(!osync_error_is_set(&(error)), NULL);
 			
-			if (status->objtype == NULL) {
+			if (objtype == NULL) {
 				num_client_main_read++;
 			} else {
-				fail_unless(!strncmp(status->objtype, "mockobjtype", 11), NULL);
+				fail_unless(!strncmp(objtype, "mockobjtype", 11), NULL);
 				num_client_read++;
 			}
 			
 			break;
-		case OSYNC_CLIENT_EVENT_WRITTEN:
-			fail_unless(!osync_error_is_set(&(status->error)), NULL);
+		case OSYNC_ENGINE_MEMBER_EVENT_WRITTEN:
+			fail_unless(!osync_error_is_set(&(error)), NULL);
 			
-			if (status->objtype == NULL) {
+			if (objtype == NULL) {
 				num_client_main_written++;
 			} else {
-				fail_unless(!strncmp(status->objtype, "mockobjtype", 11), NULL);
+				fail_unless(!strncmp(objtype, "mockobjtype", 11), NULL);
 				num_client_written++;
 			}
 			
 			break;
-		case OSYNC_CLIENT_EVENT_ERROR:
-			fail_unless(osync_error_is_set(&(status->error)), NULL);
+		case OSYNC_ENGINE_MEMBER_EVENT_ERROR:
+			fail_unless(osync_error_is_set(&(error)), NULL);
 			num_client_errors++;
 			break;
-		case OSYNC_CLIENT_EVENT_SYNC_DONE:
-			fail_unless(!osync_error_is_set(&(status->error)), NULL);
+		case OSYNC_ENGINE_MEMBER_EVENT_SYNC_DONE:
+			fail_unless(!osync_error_is_set(&(error)), NULL);
 			
-			if (status->objtype == NULL) {
+			if (objtype == NULL) {
 				num_client_main_sync_done++;
 			} else {
-				fail_unless(!strncmp(status->objtype, "mockobjtype", 11), NULL);
+				fail_unless(!strncmp(objtype, "mockobjtype", 11), NULL);
 				num_client_sync_done++;
 			}
 			
 			break;
-		case OSYNC_CLIENT_EVENT_DISCOVERED:
-			fail_unless(!osync_error_is_set(&(status->error)), NULL);
+		case OSYNC_ENGINE_MEMBER_EVENT_DISCOVERED:
+			fail_unless(!osync_error_is_set(&(error)), NULL);
 			num_client_discovered++;
 			break;
 	}
@@ -328,21 +412,23 @@ void member_status(OSyncMemberUpdate *status, void *user_data)
 	osync_trace(TRACE_EXIT, "%s", __func__);
 }
 
-void entry_status(OSyncChangeUpdate *status, void *user_data)
+void entry_status(OSyncEngineChangeUpdate *status, void *user_data)
 {
-	osync_trace(TRACE_ENTRY, "%s(%p (%i), %p)", __func__, status, status->type, user_data);
+	osync_trace(TRACE_ENTRY, "%s(%p, %p)", __func__, status, user_data);
+
+        OSyncError *error = osync_engine_change_update_get_error(status);
 	
-	switch (status->type) {
-		case OSYNC_CHANGE_EVENT_READ:
-			fail_unless(!osync_error_is_set(&(status->error)), NULL);
+	switch (osync_engine_change_update_get_event(status)) {
+		case OSYNC_ENGINE_CHANGE_EVENT_READ:
+			fail_unless(!osync_error_is_set(&(error)), NULL);
 			num_change_read++;
 			break;
-		case OSYNC_CHANGE_EVENT_WRITTEN:
-			fail_unless(!osync_error_is_set(&(status->error)), NULL);
+		case OSYNC_ENGINE_CHANGE_EVENT_WRITTEN:
+			fail_unless(!osync_error_is_set(&(error)), NULL);
 			num_change_written++;
 			break;
-		case OSYNC_CHANGE_EVENT_ERROR:
-			fail_unless(osync_error_is_set(&(status->error)), NULL);
+		case OSYNC_ENGINE_CHANGE_EVENT_ERROR:
+			fail_unless(osync_error_is_set(&(error)), NULL);
 			num_change_error++;
 			break;
 	}
@@ -352,42 +438,65 @@ void entry_status(OSyncChangeUpdate *status, void *user_data)
 
 void engine_status(OSyncEngineUpdate *status, void *user_data)
 {
-	osync_trace(TRACE_ENTRY, "%s(%p(%i), %p)", __func__, status, status->type, user_data);
-	switch (status->type) {
+	osync_trace(TRACE_ENTRY, "%s(%p, %p)", __func__, status, user_data);
+
+        OSyncError *error = osync_engine_update_get_error(status);
+
+	switch (osync_engine_update_get_event(status)) {
 		case OSYNC_ENGINE_EVENT_CONNECTED:
-			fail_unless(!osync_error_is_set(&(status->error)), NULL);
+			fail_unless(!osync_error_is_set(&(error)), NULL);
 			num_engine_connected++;
 			break;
+		case OSYNC_ENGINE_EVENT_CONNECT_DONE:
+			fail_unless(!osync_error_is_set(&(error)), NULL);
+			num_engine_connect_done++;
+			break;
 		case OSYNC_ENGINE_EVENT_READ:
-			fail_unless(!osync_error_is_set(&(status->error)), NULL);
+			fail_unless(!osync_error_is_set(&(error)), NULL);
 			num_engine_read++;
 			break;
+		case OSYNC_ENGINE_EVENT_PREPARED_MAP:
+			fail_unless(!osync_error_is_set(&(error)), NULL);
+			num_engine_prepared_map++;
+			break;
+		case OSYNC_ENGINE_EVENT_MAPPED:
+			fail_unless(!osync_error_is_set(&(error)), NULL);
+			num_engine_mapped++;
+			break;
+		case OSYNC_ENGINE_EVENT_MULTIPLIED: 
+			fail_unless(!osync_error_is_set(&(error)), NULL);
+			num_engine_multiplied++;
+			break;
+		case OSYNC_ENGINE_EVENT_PREPARED_WRITE:
+			fail_unless(!osync_error_is_set(&(error)), NULL);
+			num_engine_prepared_write++;
+			break;
 		case OSYNC_ENGINE_EVENT_WRITTEN:
-			fail_unless(!osync_error_is_set(&(status->error)), NULL);
+			fail_unless(!osync_error_is_set(&(error)), NULL);
 			num_engine_written++;
 			break;
 		case OSYNC_ENGINE_EVENT_DISCONNECTED:
-			fail_unless(!osync_error_is_set(&(status->error)), NULL);
+			fail_unless(!osync_error_is_set(&(error)), NULL);
 			num_engine_disconnected++;
 			break;
 		case OSYNC_ENGINE_EVENT_ERROR:
-			fail_unless(osync_error_is_set(&(status->error)), NULL);
+			fail_unless(osync_error_is_set(&(error)), NULL);
 			num_engine_errors++;
 			break;
 		case OSYNC_ENGINE_EVENT_SUCCESSFUL:
-			fail_unless(!osync_error_is_set(&(status->error)), NULL);
+			fail_unless(!osync_error_is_set(&(error)), NULL);
 			num_engine_successful++;
 			break;
 		case OSYNC_ENGINE_EVENT_PREV_UNCLEAN:
-			fail_unless(!osync_error_is_set(&(status->error)), NULL);
+			fail_unless(!osync_error_is_set(&(error)), NULL);
 			num_engine_prev_unclean++;
 			break;
 		case OSYNC_ENGINE_EVENT_END_CONFLICTS:
-			fail_unless(!osync_error_is_set(&(status->error)), NULL);
+			fail_unless(!osync_error_is_set(&(error)), NULL);
 			num_engine_end_conflicts++;
 			break;
 		case OSYNC_ENGINE_EVENT_SYNC_DONE:
-			fail_unless(!osync_error_is_set(&(status->error)), NULL);
+			fail_unless(!osync_error_is_set(&(error)), NULL);
 			num_engine_sync_done++;
 			break;
 	}
@@ -395,19 +504,21 @@ void engine_status(OSyncEngineUpdate *status, void *user_data)
 	osync_trace(TRACE_EXIT, "%s", __func__);
 }
 
-void mapping_status(OSyncMappingUpdate *status, void *user_data)
+void mapping_status(OSyncEngineMappingUpdate *status, void *user_data)
 {
-	switch (status->type) {
-		case OSYNC_MAPPING_EVENT_SOLVED:
-			fail_unless(!osync_error_is_set(&(status->error)), NULL);
+        OSyncError *error = osync_engine_mapping_update_get_error(status);
+
+	switch (osync_engine_mapping_update_get_event(status)) {
+		case OSYNC_ENGINE_MAPPING_EVENT_SOLVED:
+			fail_unless(!osync_error_is_set(&(error)), NULL);
 			num_mapping_solved++;
 			break;
-		/*case OSYNC_MAPPING_EVENT_WRITTEN:
-			fail_unless(!osync_error_is_set(&(status->error)), NULL);
+		/*case OSYNC_ENGINE_MAPPING_EVENT_WRITTEN:
+			fail_unless(!osync_error_is_set(&(error)), NULL);
 			num_mapping_written++;
 			break;*/
-		case OSYNC_MAPPING_EVENT_ERROR:
-			fail_unless(osync_error_is_set(&(status->error)), NULL);
+		case OSYNC_ENGINE_MAPPING_EVENT_ERROR:
+			fail_unless(osync_error_is_set(&(error)), NULL);
 			num_mapping_errors++;
 			break;
 	}
@@ -415,14 +526,17 @@ void mapping_status(OSyncMappingUpdate *status, void *user_data)
 
 void conflict_handler_choose_first(OSyncEngine *engine, OSyncMappingEngine *mapping, void *user_data)
 {
+	OSyncChange *change = NULL;
+	OSyncError *error = NULL;
+
 	osync_trace(TRACE_ENTRY, "%s(%p, %p, %p)", __func__, engine, mapping, user_data);
 	
 	num_mapping_conflicts++;
-	fail_unless(osync_mapping_engine_num_changes(mapping) == GPOINTER_TO_INT(user_data), NULL);
+	if (user_data)
+		fail_unless(osync_mapping_engine_num_changes(mapping) == GPOINTER_TO_UINT(user_data), NULL);
 	fail_unless(num_engine_end_conflicts == 0, NULL);
 
-	OSyncChange *change = osync_mapping_engine_member_change(mapping, 1);
-	OSyncError *error = NULL;
+	change = osync_mapping_engine_member_change(mapping, 1);
 	osync_assert(osync_engine_mapping_solve(engine, mapping, change, &error));
 	osync_assert(error == NULL);
 	
@@ -432,13 +546,14 @@ void conflict_handler_choose_first(OSyncEngine *engine, OSyncMappingEngine *mapp
 void conflict_handler_choose_deleted(OSyncEngine *engine, OSyncMappingEngine *mapping, void *user_data)
 {
 	OSyncError *error = NULL;
+	unsigned int i;
 	osync_trace(TRACE_ENTRY, "%s(%p, %p, %p)", __func__, engine, mapping, user_data);
 	
 	num_mapping_conflicts++;
-	fail_unless(osync_mapping_engine_num_changes(mapping) == GPOINTER_TO_INT(user_data), NULL);
+	if (user_data)
+		fail_unless(osync_mapping_engine_num_changes(mapping) == GPOINTER_TO_UINT(user_data), NULL);
 	fail_unless(num_engine_end_conflicts == 0, NULL);
 
-	int i;
 	for (i = 0; i < osync_mapping_engine_num_changes(mapping); i++) {
 		OSyncChange *change = osync_mapping_engine_nth_change(mapping, i);
 		if (osync_change_get_changetype(change) == OSYNC_CHANGE_TYPE_DELETED) {
@@ -454,15 +569,15 @@ void conflict_handler_choose_deleted(OSyncEngine *engine, OSyncMappingEngine *ma
 
 void conflict_handler_choose_modified(OSyncEngine *engine, OSyncMappingEngine *mapping, void *user_data)
 {
+	OSyncError *error = NULL;
+	unsigned int i;
 	osync_trace(TRACE_ENTRY, "%s(%p, %p, %p)", __func__, engine, mapping, user_data);
 
-	OSyncError *error = NULL;
-	
 	num_mapping_conflicts++;
-	fail_unless(osync_mapping_engine_num_changes(mapping) == GPOINTER_TO_INT(user_data), NULL);
+	if (user_data)
+		fail_unless(osync_mapping_engine_num_changes(mapping) == GPOINTER_TO_UINT(user_data), NULL);
 	fail_unless(num_engine_end_conflicts == 0, NULL);
 
-	int i;
 	for (i = 0; i < osync_mapping_engine_num_changes(mapping); i++) {
 		OSyncChange *change = osync_mapping_engine_nth_change(mapping, i);
 		if (osync_change_get_changetype(change) == OSYNC_CHANGE_TYPE_MODIFIED) {
@@ -477,25 +592,26 @@ void conflict_handler_choose_modified(OSyncEngine *engine, OSyncMappingEngine *m
 
 void conflict_handler_ignore(OSyncEngine *engine, OSyncMappingEngine *mapping, void *user_data)
 {
+	OSyncError *error = NULL;
 	num_mapping_conflicts++;
 	if (user_data)
-		fail_unless(osync_mapping_engine_num_changes(mapping) == GPOINTER_TO_INT(user_data), NULL);
+		fail_unless(osync_mapping_engine_num_changes(mapping) == GPOINTER_TO_UINT(user_data), NULL);
 	fail_unless(num_engine_end_conflicts == 0, NULL);
 	
-	OSyncError *error = NULL;
 	osync_engine_mapping_ignore_conflict(engine, mapping, &error);
 	fail_unless(error == NULL, NULL);
 }
 
 void conflict_handler_duplicate(OSyncEngine *engine, OSyncMappingEngine *mapping, void *user_data)
 {
+	OSyncError *error = NULL;
 	osync_trace(TRACE_ENTRY, "%s(%p, %p, %p)", __func__, engine, mapping, user_data);
 	
 	num_mapping_conflicts++;
-	fail_unless(osync_mapping_engine_num_changes(mapping) == GPOINTER_TO_INT(user_data), NULL);
+	if (user_data)
+		fail_unless(osync_mapping_engine_num_changes(mapping) == GPOINTER_TO_UINT(user_data), NULL);
 	fail_unless(num_engine_end_conflicts == 0, NULL);
 	
-	OSyncError *error = NULL;
 	fail_unless(osync_engine_mapping_duplicate(engine, mapping, &error), NULL);
 	
 	osync_trace(TRACE_EXIT, "%s", __func__);
@@ -503,12 +619,13 @@ void conflict_handler_duplicate(OSyncEngine *engine, OSyncMappingEngine *mapping
 
 void conflict_handler_abort(OSyncEngine *engine, OSyncMappingEngine *mapping, void *user_data)
 {
+	OSyncError *error = NULL;
 	osync_trace(TRACE_ENTRY, "%s(%p, %p, %p)", __func__, engine, mapping, user_data);
 	
-	fail_unless(osync_mapping_engine_num_changes(mapping) == GPOINTER_TO_INT(user_data), NULL);
+	if (user_data)
+		fail_unless(osync_mapping_engine_num_changes(mapping) == GPOINTER_TO_UINT(user_data), NULL);
 	fail_unless(num_engine_end_conflicts == 0, NULL);
 	
-	OSyncError *error = NULL;
 	fail_unless(osync_engine_abort(engine, &error), NULL);
 	
 	osync_trace(TRACE_EXIT, "%s", __func__);
@@ -518,10 +635,12 @@ OSyncEngine *gengine = NULL;
 
 void solve_conflict(OSyncMappingEngine *mapping)
 {
-	sleep(5);
-	
-	OSyncChange *change = osync_mapping_engine_nth_change(mapping, 0);
+	OSyncChange *change = NULL;
 	OSyncError *error = NULL;
+
+	g_usleep(5*G_USEC_PER_SEC);
+	
+	change = osync_mapping_engine_nth_change(mapping, 0);
 	osync_assert(osync_engine_mapping_solve(gengine, mapping, change, &error));
 	osync_assert(error == NULL);
 }
@@ -531,7 +650,8 @@ void conflict_handler_delay(OSyncEngine *engine, OSyncMappingEngine *mapping, vo
 	osync_trace(TRACE_ENTRY, "%s(%p, %p, %p)", __func__, engine, mapping, user_data);
 	
 	num_mapping_conflicts++;
-	fail_unless(osync_mapping_engine_num_changes(mapping) == GPOINTER_TO_INT(user_data), NULL);
+	if (user_data)
+		fail_unless(osync_mapping_engine_num_changes(mapping) == GPOINTER_TO_UINT(user_data), NULL);
 	fail_unless(num_engine_end_conflicts == 0, NULL);
 	gengine = engine;
 	g_thread_create ((GThreadFunc)solve_conflict, mapping, TRUE, NULL);
@@ -560,6 +680,9 @@ void reset_counters()
 	
 	num_engine_connected = 0;
 	num_engine_read = 0;
+	num_engine_prepared_map = 0;
+	num_engine_mapped = 0;
+	num_engine_multiplied = 0;
 	num_engine_written = 0;
 	num_engine_disconnected = 0;
 	num_engine_errors = 0;
@@ -578,6 +701,7 @@ void reset_counters()
 osync_bool synchronize_once(OSyncEngine *engine, OSyncError **error)
 {
 	reset_counters();
+	fail_unless(osync_engine_repair(engine, error), "Repair of engine failed: %s", osync_error_print(error));
 	return osync_engine_synchronize_and_block(engine, error);
 }
 
@@ -592,7 +716,17 @@ void discover_all_once(OSyncEngine *engine, OSyncError **error)
 	}
 }
 
-/*! @brief Check if file or directory exists. No check for regular file! 
+OSyncFormatEnv *osync_testing_load_formatenv(const char *formatdir)
+{
+	OSyncError *error = NULL;
+	OSyncFormatEnv *formatenv = osync_format_env_new(&error);
+	fail_unless(formatenv != NULL, NULL);
+	fail_unless(error == NULL, NULL);
+	fail_unless(osync_format_env_load_plugins(formatenv, formatdir, &error));
+	return formatenv;
+}
+
+/** @brief Check if file or directory exists. No check for regular file! 
  * 
  * @param file filename or fullpath of file/directory 
  * @returns TRUE if exists, FALSE otherwise
@@ -603,7 +737,7 @@ osync_bool osync_testing_file_exists(const char *file)
 	return g_file_test(file, G_FILE_TEST_EXISTS);
 }
 
-/*! @brief Removes files and directories 
+/** @brief Removes files and directories 
  * 
  * @param file filename or fullpath of file/directory 
  * @returns TRUE on success, FALSE otherwise
@@ -614,7 +748,7 @@ osync_bool osync_testing_file_remove(const char *file)
 	return g_remove(file);
 }
 
-/*! @brief Modifies permission of file - like chmod() 
+/** @brief Modifies permission of file - like chmod() 
  * 
  * @param file filename or fullpath of file 
  * @param mode the permission mode like chmod()
@@ -630,7 +764,7 @@ osync_bool osync_testing_file_chmod(const char *file, int mode)
 	return g_chmod(file, mode);
 }
 
-/*! @brief Copy files
+/** @brief Copy files
  *
  * @param source source filename
  * @param dest destination filename
@@ -639,48 +773,63 @@ osync_bool osync_testing_file_chmod(const char *file, int mode)
  */
 osync_bool osync_testing_file_copy(const char *source, const char *dest)
 {
-        gboolean ret;
-        const char *argv[] = { "cp", source, dest, NULL };
-        int exitstatus = -1;
+	gboolean ret;
+	const char *argv[] = { "cp", source, dest, NULL };
+	int exitstatus = -1;
 
-        ret = g_spawn_sync(NULL,	        /* working directory */
-                           (char **)argv,	/* arguments */
-                           NULL,	        /* environment */
-                           G_SPAWN_STDOUT_TO_DEV_NULL | G_SPAWN_STDERR_TO_DEV_NULL,
-                           NULL,	        /* child setup function */
-                           NULL,	        /* user data for child setup func */
-                           NULL,	        /* stdin */
-                           NULL,	        /* stdout */
-                           &exitstatus,	        /* exit status */
-                           NULL		        /* error function */
-                        );
-
-        return ret && WEXITSTATUS(exitstatus) == 0;
+	ret = g_spawn_sync(NULL,			/* working directory */
+						(char **)argv,	/* arguments */
+						NULL,			/* environment */
+						G_SPAWN_STDOUT_TO_DEV_NULL | G_SPAWN_STDERR_TO_DEV_NULL,
+						NULL,			/* child setup function */
+						NULL,			/* user data for child setup func */
+						NULL,			/* stdin */
+						NULL,			/* stdout */
+						&exitstatus,	/* exit status */
+						NULL			/* error function */
+						);
+#ifndef _WIN32
+	return ret && WEXITSTATUS(exitstatus) == 0;
+#else
+	return ret;
+#endif
 }
 
-/*! @brief Find differences between two files
+/** @brief Find differences between two files
  *
  * @param source source filename
  * @param dest destination filename
- * @returns TRUE on success, FALSE otherwise
+ * @returns TRUE on when equal/success, FALSE otherwise
  *
  */
 osync_bool osync_testing_diff(const char *file1, const char *file2)
 {
-        gchar *cmd;
-        int ret;
+	gchar *cmd;
+	int ret;
 
 	osync_assert(file1);
 	osync_assert(file2);
 
-        cmd = g_strdup_printf(DIFF " -x \".*\" %s %s", file1, file2);
-        ret = system(cmd);
-        g_free(cmd);
+	cmd = g_strdup_printf(DIFF " -x \".*\" %s %s", file1, file2);
+	ret = system(cmd);
+	g_free(cmd);
 
-        return ret;
+	return !ret;
 }
 
-/*! @brief Creates a simple OSyncPluginConfig with a single resource.
+/**
+ * @brief Test if the directory is empty 
+ */
+osync_bool osync_testing_directory_is_empty(const char *dirname)
+{
+	gchar *cmd;
+	int ret;
+	cmd = g_strdup_printf("ls %s | grep \".*\"", dirname);
+	ret = system(cmd);
+	return (ret != 0);
+}
+
+/** @brief Creates a simple OSyncPluginConfig with a single resource.
  *         If config is not null the resource information gets added.
  * 
  * @param config OSyncPluginConfig pointer to add resource info
@@ -691,23 +840,23 @@ osync_bool osync_testing_diff(const char *file1, const char *file2)
  * 
  */
 OSyncPluginConfig *simple_plugin_config(OSyncPluginConfig *config, const char *path, const char *objtype, const char *objformat, const char *format_config) {
-
+	OSyncError *error = NULL;
+	OSyncObjFormatSink *format_sink = NULL;
+	OSyncPluginResource *res = NULL;
 	osync_assert(objtype);
 	osync_assert(objformat);
 
-	OSyncError *error = NULL;
-
 	if (!config)
 		config = osync_plugin_config_new(&error);
 
 	fail_unless(config != NULL, NULL);
 	fail_unless(error == NULL, NULL);
 
-	OSyncObjFormatSink *format_sink = osync_objformat_sink_new(objformat, &error);
+	format_sink = osync_objformat_sink_new(objformat, &error);
 	if (format_config)
 		osync_objformat_sink_set_config(format_sink, format_config);
 
-	OSyncPluginResource *res = osync_plugin_resource_new(&error);
+	res = osync_plugin_resource_new(&error);
 	osync_plugin_resource_set_objtype(res, objtype);
 	osync_plugin_resource_set_path(res, path); 
 	osync_plugin_resource_enable(res, TRUE);
@@ -719,7 +868,7 @@ OSyncPluginConfig *simple_plugin_config(OSyncPluginConfig *config, const char *p
 	return config;
 }
 
-/*! @brief Creates a simple OSyncEngine with nth Members with mock-sync
+/** @brief Creates a simple OSyncEngine with nth Members with mock-sync
  *         plugins.
  * 
  * @param member_size The number of member the Group for this Engine should have
@@ -730,8 +879,9 @@ OSyncEngine *osync_testing_create_engine_dummy(unsigned int member_size)
 {
 	unsigned int u;
 	OSyncError *error = NULL;
-
+	OSyncEngine *engine = NULL;
 	OSyncGroup *group = osync_group_new(&error);
+
 	fail_unless(group != NULL, NULL);
 	fail_unless(error == NULL, NULL);
 
@@ -743,10 +893,16 @@ OSyncEngine *osync_testing_create_engine_dummy(unsigned int member_size)
 		osync_group_add_member(group, member);
 	}
 
-	OSyncEngine *engine = osync_engine_new(group, &error);
+	engine = osync_engine_new(group, &error);
 	fail_unless(engine != NULL, NULL);
 	fail_unless(error == NULL, NULL);
 
 	return engine;
 }
 
+void osync_testing_system_abort(const char *command)
+{
+	if (system(command))
+		abort();
+}
+
