@@ -5,7 +5,7 @@
 
 START_TEST (version_new)
 {
-	char *testbed = setup_testbed("merger");
+	char *testbed = setup_testbed(NULL);
 
 	OSyncError *error = NULL;
 	OSyncVersion *version = osync_version_new(&error);
@@ -23,7 +23,7 @@ END_TEST
 
 START_TEST (version_matches)
 {
-	char *testbed = setup_testbed("merger");
+	char *testbed = setup_testbed(NULL);
 
 	OSyncError *error = NULL;
 	OSyncVersion *version = osync_version_new(&error);
@@ -61,10 +61,10 @@ END_TEST
 
 START_TEST (version_load_from_descriptions)
 {
-	char *testbed = setup_testbed("merger");
+	char *testbed = setup_testbed(NULL);
 
 	OSyncError *error = NULL;
-	OSyncList *versions = _osync_version_load_from_descriptions(&error, testbed, testbed);
+	OSyncList *versions = osync_version_load_from_descriptions(&error, testbed, testbed);
 	//fail_unless(versions != NULL, NULL);
 	fail_unless(error == NULL, NULL);
 
@@ -79,25 +79,9 @@ START_TEST (version_load_from_descriptions)
 }
 END_TEST
 
-Suite *version_suite(void)
-{
-	Suite *s = suite_create("Version");
-	create_case(s, "version_new", version_new);
-	create_case(s, "version_matches", version_matches);
-	create_case(s, "version_load_from_descriptions", version_load_from_descriptions);
-	return s;
-}
+OSYNC_TESTCASE_START("version")
+OSYNC_TESTCASE_ADD(version_new)
+OSYNC_TESTCASE_ADD(version_matches)
+OSYNC_TESTCASE_ADD(version_load_from_descriptions)
+OSYNC_TESTCASE_END
 
-int main(void)
-{
-	int nf;
-
-	Suite *s = version_suite();
-	
-	SRunner *sr;
-	sr = srunner_create(s);
-	srunner_run_all(sr, CK_VERBOSE);
-	nf = srunner_ntests_failed(sr);
-	srunner_free(sr);
-	return (nf == 0) ? EXIT_SUCCESS : EXIT_FAILURE;
-}
