@@ -1,7 +1,5 @@
 #include <opensync/opensync.h>
-#include <opensync/opensync_internals.h>
 #include <opensync/opensync-group.h>
-#include <opensync/opensync-archive.h>
 #include <opensync/opensync-mapping.h>
 #include <opensync/opensync-helper.h>
 
@@ -10,7 +8,8 @@
 #include <string.h>
 #include <sys/types.h>
 //#include <sys/wait.h>
-#include <sqlite3.h>
+
+#include <glib.h>
 
 static void usage (char *name, int ecode)
 {
@@ -22,7 +21,7 @@ static void usage (char *name, int ecode)
 	exit(ecode);
 }
 
-typedef enum  {
+typedef enum {
 	DUMPMAPS = 1,
 	DUMPHASH = 2,
 	RESET = 4
@@ -30,27 +29,33 @@ typedef enum  {
 
 static void dump_map_objtype(OSyncGroupEnv *env, const char *objtype, const char *groupname)
 {
+	printf("Dumping of mappings got temporarliy disabled\n");
+
+/* FIXME: Temporarily disabled - OSyncArchive got removed from 0.40 API draft */
+#if 0	
+
 	OSyncError *error = NULL;
 	OSyncGroup *group = osync_group_env_find_group(env, groupname);
-	
+
 	char *path = g_strdup_printf("%s/archive.db", osync_group_get_configdir(group));
 	OSyncArchive *archive = osync_archive_new(path, &error);
-	if (!archive)
-		goto error;
-	g_free(path);
-	
 	OSyncList *ids = NULL;
 	OSyncList *uids = NULL;
 	OSyncList *mappingids = NULL;
 	OSyncList *memberids = NULL;
+	OSyncList *d , *u, *m, *i;
+
+	if (!archive)
+		goto error;
+	g_free(path);
 	
 	if (!osync_archive_load_changes(archive, objtype, &ids, &uids, &mappingids, &memberids, &error))
 		goto error;
 	
-	OSyncList *d = ids;
-	OSyncList *u = uids;
-	OSyncList *m = mappingids;
-	OSyncList *i = memberids;
+	d = ids;
+	u = uids;
+	m = mappingids;
+	i = memberids;
 	
 	for (; u; u = u->next) {
 		long long int id = (long long int)GPOINTER_TO_INT(d->data);
@@ -73,72 +78,95 @@ static void dump_map_objtype(OSyncGroupEnv *env, const char *objtype, const char
 	osync_archive_unref(archive);
 	return;
 
-error:
+ error:
 	printf("ERROR: %s", osync_error_print(&error));
 	osync_error_unref(&error);
+#endif	
 }
 
 static void dump_map(OSyncGroupEnv *env, const char *groupname)
 {
 
 	OSyncGroup *group = osync_group_env_find_group(env, groupname);
+	int num_objtypes;
 	
 	if (!group) {
 		printf("Unable to find group with name \"%s\"\n", groupname);
 		return;
 	}
-
-        int i, num_objtypes = osync_group_num_objtypes(group); 
-        if (num_objtypes == 0) { 
+	
+	OSyncList *objtypes = osync_group_get_objtypes(group);
+	num_objtypes = osync_list_length(objtypes); 
+	if (num_objtypes == 0) { 
 		printf("Group has no objtypes. Have the objtypes already been discovered?\n"); 
 		return;
-        }
+	}
 
-        for (i = 0; i < num_objtypes; i++) {
-		const char *objtype = osync_group_nth_objtype(group, i);
+	OSyncList *o;
+	for (o = objtypes; o; o = o->next) {
+		const char *objtype = (char *) o->data;
 		printf("Mappings for objtype \"%s\":\n", objtype);
 		dump_map_objtype(env, objtype, groupname);
 	}
-
+	osync_list_free(objtypes);
 }
 
+#if 0
 static void print_hashtable(const char *uid, const char *hash, void *user_data)
 {
 	printf("UID: %s\tHASH:%s\n", uid, hash);
 }
+#endif
 
 static void dump_hash(OSyncGroupEnv *env, const char *objtype, const char *groupname, char *memberid)
 {
-	OSyncError *error = NULL;
 	OSyncGroup *group = osync_group_env_find_group(env, groupname);
+#if 0 /* Unused */	
+	OSyncError *error = NULL;
+	long long int id = 0;
+	OSyncMember *member = NULL;
+	char *path = NULL;
+	OSyncHashTable *table = NULL;
+#endif	
 	
 	if (!group) {
 		printf("Unable to find group with name \"%s\"\n", groupname);
 		return;
 	}
+
+	printf("Dumping hashtable is currently not implemented!\n");
 	
-	long long int id = atoi(memberid);
-	OSyncMember *member = osync_group_find_member(group, id);
+	return;
+
+/* FIXME: public interface osync_hashltable_new() is gone!
+ * no interface to access the disk directly!
+ * */
+#if 0	
+	id = atoi(memberid);
+	member = osync_group_find_member(group, id);
 	if (!member) {
 		printf("Unable to find member with id %s\n", memberid);
 		return;
 	}
 	
-	char *path = g_strdup_printf("%s/hashtable.db", osync_member_get_configdir(member));
-	OSyncHashTable *table = osync_hashtable_new(path, objtype, &error);
+	path = g_strdup_printf("%s/hashtable.db", osync_member_get_configdir(member));
+	table = osync_hashtable_new(path, objtype, &error);
 	if (!table)
 		goto error;
 	g_free(path);
+
+	if (!osync_hashtable_load(table, &error))
+		goto error;
 	
 	osync_hashtable_foreach(table, print_hashtable, NULL);
 
 	osync_hashtable_unref(table);
-	
-	return;
 
-error:
+ error:
 	printf("ERROR: %s", osync_error_print(&error));
 	osync_error_unref(&error);
+
+#endif	
 }
 
 static void reset(OSyncGroupEnv *osync, char *groupname)
@@ -161,6 +189,8 @@ int main (int argc, char *argv[])
 	ToolAction action = DUMPMAPS;
 	char *configdir = NULL;
 	char *objtype = NULL;
+	OSyncError *error = NULL;
+	OSyncGroupEnv *env = NULL;
 	
 	if (argc == 1)
 		usage (argv[0], 1);
@@ -199,9 +229,7 @@ int main (int argc, char *argv[])
 		}
 	}
 	
-	OSyncError *error = NULL;
-	
-	OSyncGroupEnv *env = osync_group_env_new(&error);
+	env = osync_group_env_new(&error);
 	if (!env)
 		goto error;
 	
@@ -209,23 +237,25 @@ int main (int argc, char *argv[])
 		goto error;
 	
 	switch (action) {
-		case DUMPMAPS:
-			if (objtype)
-				dump_map_objtype(env, objtype, groupname);
-			else
-				dump_map(env, groupname);
-			break;
-		case DUMPHASH:
-			dump_hash(env, objtype, groupname, membername);
-			break;
-		case RESET:
-			reset(env, groupname);
-			break;
+	case DUMPMAPS:
+		if (objtype)
+			dump_map_objtype(env, objtype, groupname);
+		else
+			dump_map(env, groupname);
+		break;
+	case DUMPHASH:
+		dump_hash(env, objtype, groupname, membername);
+		break;
+	case RESET:
+		reset(env, groupname);
+		break;
 	}
-	
+	osync_group_env_unref(env);
 	return 0;
 
-error:
+ error:
+	if (env)
+		osync_group_env_unref(env);
 	printf("ERROR: %s", osync_error_print(&error));
 	osync_error_unref(&error);
 	return 1;
