@@ -1,6 +1,6 @@
 /*
  * osyncplugin - swiss-knife for testing OpenSync synchronization plugins 
- * Copyright (C) 2008  Daniel Gollub <dgollub@suse.de>
+ * Copyright (C) 2008  Daniel Gollub <gollub@b1-systems.de>
  * 
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -23,12 +23,13 @@
 #include <stdlib.h>
 #include <string.h>
 #include <errno.h>
+#ifndef _WIN32
 #include <unistd.h>
+#endif
 
 #include <glib.h>
 
 #include <opensync/opensync.h>
-#include <opensync/opensync-context.h>
 #include <opensync/opensync-ipc.h>
 #include <opensync/opensync-data.h>
 #include <opensync/opensync-plugin.h>
@@ -39,7 +40,9 @@ char *formatpath = NULL;
 char *pluginname = NULL;
 char *configfile = NULL;
 char *configdir = NULL;
-osync_bool pluginlist= FALSE;
+char *syncgroup = NULL;
+osync_bool pluginlist = FALSE;
+osync_bool connect_requested_slowsync = FALSE;
 GList *sinks = NULL;
 GList *cmdlist = NULL;
 GList *changesList = NULL;
@@ -59,7 +62,6 @@ typedef enum {
 	CMD_SLOWSYNC,
 	CMD_FASTSYNC,
 	CMD_COMMIT,
-	CMD_BATCHCOMMIT,
 	CMD_COMMITTEDALL,
 	CMD_READ,
 	CMD_WRITE,
@@ -128,6 +130,7 @@ static void usage(const char *name)
 	fprintf (stderr, "Configuration options:\n");
 	fprintf (stderr, "[--config] \tSet config file\n");
 	fprintf (stderr, "[--configdir] \tSet different config directory. Default: ~./opensync\n");
+	fprintf (stderr, "[--syncgroup] \tSet the name of the sync group. Default: osyncplugin\n");
 	
 	fprintf (stderr, "Plugin options:\n");
 	fprintf (stderr, "[--plugin] \tSet plugin\n");
@@ -148,7 +151,6 @@ static void usage(const char *name)
 	fprintf (stderr, "[--syncdone] \t\n");
 	fprintf (stderr, "[--committedall] \t\n");
 	fprintf (stderr, "[--commit] \t\n");
-	fprintf (stderr, "[--batchcommit] \t\n");
 	fprintf (stderr, "[--write] \t\n");
 	fprintf (stderr, "[--read] \t\n");
 	fprintf (stderr, "[--empty] \t\n");
@@ -177,6 +179,11 @@ static void parse_args(int argc, char **argv) {
 
 			i++;
 			continue;
+		} else if (!strcmp(arg, "--syncgroup")) {
+			if (!syncgroup)
+				syncgroup = strdup(argv[i+1]);
+			i++;
+			continue;
 		} else if (!strcmp(arg, "--plugin") || !strcmp(arg, "-p")) {
 			if (!pluginname)
 				pluginname = strdup(argv[i+1]);
@@ -261,13 +268,6 @@ static void parse_args(int argc, char **argv) {
 			else
 				new_command(CMD_COMMIT, argv[++i]);
 
-			continue;
-		} else if (!strcmp(arg, "--batchcommit")) {
-			if (!argv[i+1] || *argv[i+1] == '-')
-				new_command(CMD_BATCHCOMMIT, NULL);
-			else
-				new_command(CMD_BATCHCOMMIT, argv[++i]);
-
 			continue;
 		} else if (!strcmp(arg, "--write")) {
 			if (!argv[i+1] || *argv[i+1] == '-')
@@ -324,11 +324,10 @@ static void parse_args(int argc, char **argv) {
  */
 
 static osync_bool init(OSyncError **error) {
+	OSyncPluginConfig *config;
 	assert(!plugin);
 	assert(!plugin_env);
 
-	OSyncPluginConfig *config;
-
 	if (!(plugin_env = osync_plugin_env_new(error)))
 		goto error;
 
@@ -338,9 +337,21 @@ static osync_bool init(OSyncError **error) {
 	if (!osync_format_env_load_plugins(format_env, formatpath, error))
 		goto error_free_formatenv;
 
+	if (osync_error_is_set(error)) {
+		fprintf(stderr, "WARNING! Some plugins couldn't get loaded in "
+				"format plugin environment: %s\n", osync_error_print(error));
+		osync_error_unref(error);
+	}
+
 	if (!osync_plugin_env_load(plugin_env, pluginpath, error))
 		goto error_free_pluginenv;
 
+	if (osync_error_is_set(error)) {
+		fprintf(stderr, "WARNING! Some plugins couldn't get loaded in "
+				"plugin environment: %s\n", osync_error_print(error));
+		osync_error_unref(error);
+	}
+
 	if (!(plugin = osync_plugin_env_find_plugin(plugin_env, pluginname))) {
 		osync_error_set(error, OSYNC_ERROR_PLUGIN_NOT_FOUND, "Plugin not found: \"%s\"", pluginname);
 		goto error_free_pluginenv;
@@ -353,15 +364,15 @@ static osync_bool init(OSyncError **error) {
 	if (!config)
 		goto error_free_plugininfo;
 
-        if (osync_plugin_get_config_type(plugin) != OSYNC_PLUGIN_NO_CONFIGURATION && configfile) {
-		if (!osync_plugin_config_file_load(config, configfile, NULL, error))
+	if (osync_plugin_get_config_type(plugin) != OSYNC_PLUGIN_NO_CONFIGURATION && configfile) {
+		OSyncList *r = NULL;
+		if (!osync_plugin_config_file_load(config, configfile, error))
 			goto error_free_pluginconfig;
 
 		osync_plugin_info_set_config(plugin_info, config);
 
 		/** Redudant(aka. stolen) code from opensync/client/opensync_client.c */
 		/* Enable active sinks */
-		OSyncList *r = NULL;
 
 		if (config)
 			r = osync_plugin_config_get_resources(config);
@@ -371,6 +382,7 @@ static osync_bool init(OSyncError **error) {
 			OSyncObjTypeSink *sink;
 
 			const char *objtype = osync_plugin_resource_get_objtype(res); 
+			OSyncList *o = NULL;
 			/* Check for ObjType sink */
 			if (!(sink = osync_plugin_info_find_objtype(plugin_info, objtype))) {
 				sink = osync_objtype_sink_new(objtype, error);
@@ -379,12 +391,12 @@ static osync_bool init(OSyncError **error) {
 
 				osync_plugin_info_add_objtype(plugin_info, sink);
 			}
-
-			OSyncList *o = osync_plugin_resource_get_objformat_sinks(res);
-			for (; o; o = o->next) {
+			OSyncList *objformats = osync_plugin_resource_get_objformat_sinks(res);
+			for ( o = objformats; o; o = o->next) {
 				OSyncObjFormatSink *format_sink = (OSyncObjFormatSink *) o->data; 
 				osync_objtype_sink_add_objformat_sink(sink, format_sink);
 			}
+			osync_list_free(objformats);
 		}
 
 		osync_plugin_config_unref(config);
@@ -402,35 +414,54 @@ static osync_bool init(OSyncError **error) {
 	osync_plugin_info_set_configdir(plugin_info, configdir);
 	osync_plugin_info_set_loop(plugin_info, ctx);
 	osync_plugin_info_set_format_env(plugin_info, format_env);
+	osync_plugin_info_set_groupname(plugin_info, syncgroup);
 
 	return TRUE;
 
-/*
-error_free_loop:
-	g_main_context_unref(ctx);
-*/
-error_free_pluginconfig:
+	/*
+		error_free_loop:
+		g_main_context_unref(ctx);
+	*/
+ error_free_pluginconfig:
 	osync_plugin_config_unref(config);
-error_free_plugininfo:
+ error_free_plugininfo:
 	osync_plugin_info_unref(plugin_info);
-error_free_formatenv:
-	osync_format_env_free(format_env);
+ error_free_formatenv:
+	osync_format_env_unref(format_env);
 	format_env = NULL;
-error_free_pluginenv:
-	osync_plugin_env_free(plugin_env);
+ error_free_pluginenv:
+	osync_plugin_env_unref(plugin_env);
 	plugin_env = NULL;
-error:	
+ error:	
 	return FALSE;
 }
 
 static void *plugin_initialize(OSyncError **error)
 {
-	void *plugin_data = osync_plugin_initialize(plugin, plugin_info, error);
-	
-	if (!plugin_data)
-		return NULL;
+	OSyncList *list;
+	OSyncList *objtypesinks = NULL;
+	void *plugin_data;
+	osync_bool couldinit;
+	couldinit = osync_plugin_initialize(plugin, &(plugin_data), plugin_info, error);
+
+	objtypesinks = osync_plugin_info_get_objtype_sinks(plugin_info);
+	list = objtypesinks;
+	while(list) {
+		OSyncObjTypeSink *sink = (OSyncObjTypeSink*)list->data;
+		if (!osync_objtype_sink_open_state_db(sink, plugin_info, error)) 
+			goto error;
+
+		if (!osync_objtype_sink_load_hashtable(sink, plugin_info, error))
+			goto error;
+		
+		list = list->next;
+	}
+	osync_list_free(objtypesinks);
 
 	return plugin_data;
+error:
+	osync_list_free(objtypesinks);
+	return NULL;
 }
 
 static void finalize_plugin(void **plugin_data)
@@ -446,9 +477,10 @@ static void finalize_plugin(void **plugin_data)
 
 static OSyncObjTypeSink *find_sink(const char *objtype, OSyncError **error)
 {
+	OSyncObjTypeSink *sink = NULL;
 	assert(objtype);
 
-	OSyncObjTypeSink *sink = osync_plugin_info_find_objtype(plugin_info, objtype);
+	sink = osync_plugin_info_find_objtype(plugin_info, objtype);
 	if (!sink) {
 		osync_error_set(error, OSYNC_ERROR_GENERIC, "Unable to find sink for %s", objtype);
 		return NULL;
@@ -464,27 +496,26 @@ static OSyncObjTypeSink *get_main_sink()
 
 static const char *_osyncplugin_changetype_str(OSyncChange *change)
 {
+	const char *type;
 	assert(change);
 
-	const char *type;
-
 	switch (osync_change_get_changetype(change)) {
-		case OSYNC_CHANGE_TYPE_ADDED:
-			type = "ADDED";
-			break;
-		case OSYNC_CHANGE_TYPE_UNMODIFIED:
-			type = "UNMODIFIED";
-			break;
-		case OSYNC_CHANGE_TYPE_DELETED:
-			type = "DELETED";
-			break;
-		case OSYNC_CHANGE_TYPE_MODIFIED:
-			type = "MODIFIED";
-			break;
-		case OSYNC_CHANGE_TYPE_UNKNOWN:
-		default:
-			type = "UNKNOWN";
-			break;
+	case OSYNC_CHANGE_TYPE_ADDED:
+		type = "ADDED";
+		break;
+	case OSYNC_CHANGE_TYPE_UNMODIFIED:
+		type = "UNMODIFIED";
+		break;
+	case OSYNC_CHANGE_TYPE_DELETED:
+		type = "DELETED";
+		break;
+	case OSYNC_CHANGE_TYPE_MODIFIED:
+		type = "MODIFIED";
+		break;
+	case OSYNC_CHANGE_TYPE_UNKNOWN:
+	default:
+		type = "UNKNOWN";
+		break;
 	}
 
 	return type;
@@ -497,9 +528,9 @@ static void _osyncplugin_ctx_change_callback(OSyncChange *change, void *user_dat
 	OSyncObjTypeSink *sink = cmd->sink;
 
 	printf("GETCHANGES:\t%s\t%s\t%s", 
-			_osyncplugin_changetype_str(change), 
-			osync_objtype_sink_get_name(sink),  
-			osync_change_get_uid(change));
+	       _osyncplugin_changetype_str(change), 
+	       osync_objtype_sink_get_name(sink),	
+	       osync_change_get_uid(change));
 
 	if (osync_change_get_objformat(change))
 		printf("\t%s", osync_objformat_get_name(osync_change_get_objformat(change)));
@@ -526,22 +557,26 @@ static void _osyncplugin_ctx_callback_getchanges(void *user_data, OSyncError *er
 
 }
 
-static osync_bool get_changes_sink(Command *cmd, OSyncObjTypeSink *sink, SyncType type, void *plugin_data, OSyncError **error)
+static osync_bool get_changes_sink(Command *cmd, OSyncObjTypeSink *sink, SyncType type, OSyncError **error)
 {
+	OSyncContext *context = NULL;
+	osync_bool slowsync = FALSE;
+
 	assert(sink);
 
 	switch (type) {
-		case SYNCTYPE_NORMAL:
-			break;
-		case SYNCTYPE_FORCE_FASTSYNC:
-			osync_objtype_sink_set_slowsync(sink, FALSE);
-			break;
-		case SYNCTYPE_FORCE_SLOWSYNC:
-			osync_objtype_sink_set_slowsync(sink, TRUE);
-			break;
+	case SYNCTYPE_NORMAL:
+		slowsync = connect_requested_slowsync;
+		break;
+	case SYNCTYPE_FORCE_FASTSYNC:
+		slowsync = FALSE;
+		break;
+	case SYNCTYPE_FORCE_SLOWSYNC:
+		slowsync = TRUE;
+		break;
 	}
 
-	OSyncContext *context = osync_context_new(error);
+	context = osync_context_new(error);
 	if (!context)
 		goto error;
 
@@ -550,21 +585,22 @@ static osync_bool get_changes_sink(Command *cmd, OSyncObjTypeSink *sink, SyncTyp
 
 	osync_plugin_info_set_sink(plugin_info, sink);
 
-	osync_objtype_sink_get_changes(sink, plugin_data, plugin_info, context);
+	osync_objtype_sink_get_changes(sink, plugin_info, slowsync, context);
 
 	osync_context_unref(context);
 
 
 	return TRUE;
 
-error:
+ error:
 	return FALSE;
 }
 
-static osync_bool get_changes(Command *cmd, SyncType type, void *plugin_data, OSyncError **error)
+static osync_bool get_changes(Command *cmd, SyncType type, OSyncError **error)
 {
-	int num, i;
 	OSyncObjTypeSink *sink = NULL;
+	OSyncList *list;
+	OSyncList *objtypesinks = NULL;
 	const char *objtype = cmd->arg;
 
 	if (objtype) {
@@ -573,93 +609,111 @@ static osync_bool get_changes(Command *cmd, SyncType type, void *plugin_data, OS
 			goto error;
 
 		cmd->sink = sink;
-		if (!get_changes_sink(cmd, sink, type, plugin_data, error))
+		if (!get_changes_sink(cmd, sink, type, error))
 			goto error;
 
 	} else {
 		/* all available objtypes */
-		num = osync_plugin_info_num_objtypes(plugin_info);
-		for (i=0; i < num; i++) {
-			sink = osync_plugin_info_nth_objtype(plugin_info, i);
+		objtypesinks = osync_plugin_info_get_objtype_sinks(plugin_info);
+		list = objtypesinks;
+		while(list) {
+			sink = (OSyncObjTypeSink*)list->data;
 
 			cmd->sink = sink;
-			if (!get_changes_sink(cmd, sink, type, plugin_data, error))
+			if (!get_changes_sink(cmd, sink, type, error))
 				goto error;
+			list = list->next;
 		}
-
 		/* last but not least - the main sink */
 		if (get_main_sink())
-		
-			if (!get_changes_sink(cmd, get_main_sink(), type, plugin_data, error))
+			if (!get_changes_sink(cmd, get_main_sink(), type, error))
 				goto error;
+		
+		osync_list_free(objtypesinks);
 	}
 
 	return TRUE;
 
-error:
+ error:
+	osync_list_free(objtypesinks);
 	return FALSE;
 }
 
+static void _osyncplugin_ctx_callback_slowsync(void *user_data)
+{
+
+	OSyncObjTypeSink *sink = (OSyncObjTypeSink *) user_data;
+
+	printf("SlowSync got requested by CONNECT function");
+	if (osync_objtype_sink_get_name(sink))
+		printf(" for ObjType: \"%s\"", osync_objtype_sink_get_name(sink));
+
+	connect_requested_slowsync = TRUE;
+
+	printf(".\n");
+}
+
 static void _osyncplugin_ctx_callback_connect(void *user_data, OSyncError *error)
 {
+	OSyncError *locerror = NULL;
+	Command *cmd = NULL;
+	OSyncObjTypeSink *sink = NULL;
+
 	assert(user_data);
 
-	OSyncError *locerror = NULL;
-
-	Command *cmd = (Command *) user_data;		
-	OSyncObjTypeSink *sink = cmd->sink;
+	cmd = (Command *) user_data;		
+	sink = cmd->sink;
 
 	if (error) {
 		osync_error_set_from_error(&locerror, &error);
 		goto error;
 	}
 
-	if (osync_objtype_sink_get_slowsync(sink)) {
-		printf("SlowSync got requested by CONNECT function");
-		if (osync_objtype_sink_get_name(sink))
-			printf(" for ObjType: \"%s\"", osync_objtype_sink_get_name(sink));
-
-		printf(".\n");
-	}
-
 	g_mutex_lock(cmd->mutex);
 	g_cond_signal(cmd->cond);
+	cmd->done = TRUE;
 	g_mutex_unlock(cmd->mutex);
 
 	return;
-error:
+ error:
 	fprintf(stderr, "ERROR: %s\n", osync_error_print(&locerror));
 	return;
 }
 
-static osync_bool connect_sink(Command *cmd, OSyncObjTypeSink *sink, void *plugin_data, OSyncError **error) {
-
+static osync_bool connect_sink(Command *cmd, OSyncObjTypeSink *sink, OSyncError **error) {
+	OSyncContext *context = NULL;
 	assert(sink);
-	cmd->sink = sink;
+	assert(cmd);
 
-	OSyncContext *context = osync_context_new(error);
+	context = osync_context_new(error);
 	if (!context)
 		goto error;
 
+	cmd->sink = sink;
+
 	osync_context_set_callback(context, _osyncplugin_ctx_callback_connect, cmd);
+	osync_context_set_slowsync_callback(context, _osyncplugin_ctx_callback_slowsync, sink);
 
 	osync_plugin_info_set_sink(plugin_info, sink);
 
-	osync_objtype_sink_connect(sink, plugin_data, plugin_info, context);
+	/* Reset slow-sync state */
+	connect_requested_slowsync = FALSE;
+
+	osync_objtype_sink_connect(sink, plugin_info, context);
 
 	osync_context_unref(context);
 
 	return TRUE;
 
-error:	
+ error:	
 	return FALSE;
 }
 
-static osync_bool connect(Command *cmd, void *plugin_data, OSyncError **error)
-{
-	
-	unsigned int i, num;
+static osync_bool connect_plugin(Command *cmd, OSyncError **error)
+{	
 	OSyncObjTypeSink *sink = NULL;
+	OSyncList *objtypesinks = NULL;
+	OSyncList *list;
 	const char *objtype = cmd->arg;
 
 	if (objtype) {
@@ -667,36 +721,43 @@ static osync_bool connect(Command *cmd, void *plugin_data, OSyncError **error)
 		if (!sink)
 			goto error;
 
-		if (!connect_sink(cmd, sink, plugin_data, error))
+		if (!connect_sink(cmd, sink, error))
 			goto error;
 	} else {
-		num = osync_plugin_info_num_objtypes(plugin_info);
-		for (i=0; i < num; i++) {
-			sink = osync_plugin_info_nth_objtype(plugin_info, i);
+		objtypesinks = osync_plugin_info_get_objtype_sinks(plugin_info);
+		list = objtypesinks;
+		while(list) {
+			sink = (OSyncObjTypeSink*)list->data;
 
-			if (!connect_sink(cmd, sink, plugin_data, error))
+			if (!connect_sink(cmd, sink, error))
 				goto error;
+			list = list->next;
 		}
 
 		/* last but not least - the main sink */
 		if (get_main_sink())
-			if (!connect_sink(cmd, get_main_sink(), plugin_data, error))
+			if (!connect_sink(cmd, get_main_sink(), error))
 				goto error;
+		
+		osync_list_free(objtypesinks);
 	}
 
 	return TRUE;
-error:
+ error:
+	osync_list_free(objtypesinks);
 	return FALSE;
 }
 
 static void _osyncplugin_ctx_callback_disconnect(void *user_data, OSyncError *error)
 {
+	OSyncError *locerror = NULL;
+	Command *cmd = NULL;
+	OSyncObjTypeSink *sink = NULL;
+
 	assert(user_data);
 
-	OSyncError *locerror = NULL;
-
-	Command *cmd = (Command *) user_data;		
-	OSyncObjTypeSink *sink = cmd->sink;
+	cmd = (Command *) user_data;		
+	sink = cmd->sink;
 
 	if (error) {
 		osync_error_set_from_error(&locerror, &error);
@@ -716,38 +777,39 @@ static void _osyncplugin_ctx_callback_disconnect(void *user_data, OSyncError *er
 	g_mutex_unlock(cmd->mutex);
 
 	return;
-error:
+ error:
 	fprintf(stderr, "ERROR: %s\n", osync_error_print(&locerror));
 	return;
 }
 
-static osync_bool disconnect_sink(Command *cmd, OSyncObjTypeSink *sink, void *plugin_data, OSyncError **error) {
-
-	assert(sink);
-
+static osync_bool disconnect_sink(Command *cmd, OSyncObjTypeSink *sink, OSyncError **error) {
 	OSyncContext *context = osync_context_new(error);
+	assert(sink);
+	assert(cmd);
+
 	if (!context)
 		goto error;
 
 	cmd->sink = sink;
+
 	osync_context_set_callback(context, _osyncplugin_ctx_callback_disconnect, cmd);
 
 	osync_plugin_info_set_sink(plugin_info, sink);
 
-	osync_objtype_sink_disconnect(sink, plugin_data, plugin_info, context);
+	osync_objtype_sink_disconnect(sink, plugin_info, context);
 
 	osync_context_unref(context);
 
 	return TRUE;
 
-error:	
+ error:	
 	return FALSE;
 }
 
-static osync_bool disconnect(Command *cmd, void *plugin_data, OSyncError **error)
+static osync_bool disconnect(Command *cmd, OSyncError **error)
 {
-	
-	int i, num;
+	OSyncList *list;
+	OSyncList *objtypesinks = NULL;
 	OSyncObjTypeSink *sink = NULL;
 	const char *objtype = cmd->arg;
 
@@ -756,36 +818,43 @@ static osync_bool disconnect(Command *cmd, void *plugin_data, OSyncError **error
 		if (!sink)
 			goto error;
 
-		if (!disconnect_sink(cmd, sink, plugin_data, error))
+		if (!disconnect_sink(cmd, sink, error))
 			goto error;
 	} else {
-		num = osync_plugin_info_num_objtypes(plugin_info);
-		for (i=0; i < num; i++) {
-			sink = osync_plugin_info_nth_objtype(plugin_info, i);
+		objtypesinks = osync_plugin_info_get_objtype_sinks(plugin_info);
+		list = objtypesinks;
+		while(list) {
+			sink = (OSyncObjTypeSink*)list->data;
 
-			if (!disconnect_sink(cmd, sink, plugin_data, error))
+			if (!disconnect_sink(cmd, sink, error))
 				goto error;
+			list = list->next;
 		}
 
 		/* last but not least - the main sink */
 		if (get_main_sink())
-			if (!disconnect_sink(cmd, get_main_sink(), plugin_data, error))
+			if (!disconnect_sink(cmd, get_main_sink(), error))
 				goto error;
+		
+		osync_list_free(objtypesinks);
 	}
 
 	return TRUE;
-error:
+ error:
+	osync_list_free(objtypesinks);
 	return FALSE;
 }
 
 static void _osyncplugin_ctx_callback_commit_change(void *user_data, OSyncError *error)
 {
+	OSyncError *locerror = NULL;
+	Command *cmd = NULL;
+	OSyncObjTypeSink *sink = NULL;
+
 	assert(user_data);
 
-	OSyncError *locerror = NULL;
-
-	Command *cmd = (Command *) user_data;		
-	OSyncObjTypeSink *sink = cmd->sink;
+	cmd = (Command *) user_data;		
+	sink = cmd->sink;
 
 	if (error) {
 		osync_error_set_from_error(&locerror, &error);
@@ -794,86 +863,95 @@ static void _osyncplugin_ctx_callback_commit_change(void *user_data, OSyncError
 
 	g_mutex_lock(cmd->mutex);
 	g_cond_signal(cmd->cond);
+	cmd->done = TRUE;
 	g_mutex_unlock(cmd->mutex);
 
 	return;
-error:
+ error:
 	fprintf(stderr, "ERROR for sink \"%s\": %s\n", osync_objtype_sink_get_name(sink), osync_error_print(&locerror));
 	return;
 
 }
 
-static osync_bool commit_sink(OSyncObjTypeSink *sink, OSyncChange *change, void *plugin_data, OSyncError **error) {
-
+static osync_bool commit_sink(Command *cmd, OSyncObjTypeSink *sink, OSyncChange *change, OSyncError **error) {
+	OSyncContext *context = NULL;
 	assert(sink);
 	assert(change);
 
-	OSyncContext *context = osync_context_new(error);
+	context = osync_context_new(error);
 	if (!context)
 		goto error;
 
-	osync_context_set_callback(context, _osyncplugin_ctx_callback_commit_change, sink);
+	cmd->sink = sink;
+
+	osync_context_set_callback(context, _osyncplugin_ctx_callback_commit_change, cmd);
 
 	osync_plugin_info_set_sink(plugin_info, sink);
 
 	printf("COMMIT:\t%s\t%s\t%s\n", 
-				_osyncplugin_changetype_str(change), 
-				osync_data_get_objtype(osync_change_get_data(change)), 
-				osync_change_get_uid(change));
+	       _osyncplugin_changetype_str(change), 
+	       osync_data_get_objtype(osync_change_get_data(change)), 
+	       osync_change_get_uid(change));
 
-	osync_objtype_sink_commit_change(sink, plugin_data, plugin_info, change, context);
+	osync_objtype_sink_commit_change(sink, plugin_info, change, context);
 
 	osync_context_unref(context);
 
 	return TRUE;
 
-error:	
+ error:	
 	return FALSE;
 }
 
-static osync_bool commit(Command *cmd, OSyncChange *change, void *plugin_data, OSyncError **error)
+static osync_bool commit(Command *cmd, OSyncChange *change, OSyncError **error)
 {
-	assert(change);
-
-	int i, num;
+	OSyncList *list;
+	OSyncList *objtypesinks = NULL;
 	OSyncObjTypeSink *sink = NULL;
 	const char *objtype = cmd->arg;
 
+	assert(change);
+
 	if (objtype) {
 		sink = find_sink(objtype, error);
 		if (!sink)
 			goto error;
 
-		if (!commit_sink(sink, change, plugin_data, error))
+		if (!commit_sink(cmd, sink, change, error))
 			goto error;
 	} else {
-		num = osync_plugin_info_num_objtypes(plugin_info);
-		for (i=0; i < num; i++) {
-			sink = osync_plugin_info_nth_objtype(plugin_info, i);
+		objtypesinks = osync_plugin_info_get_objtype_sinks(plugin_info);
+		list = objtypesinks;
+		while(list){
+			sink = (OSyncObjTypeSink*)list->data;
 
-			if (!commit_sink(sink, change, plugin_data, error))
+			if (!commit_sink(cmd, sink, change, error))
 				goto error;
+			list = list->next;
 		}
 
 		/* last but not least - the main sink */
 		if (get_main_sink())
-			if (!commit_sink(get_main_sink(), change, plugin_data, error))
+			if (!commit_sink(cmd, get_main_sink(), change, error))
 				goto error;
+		
+		osync_list_free(objtypesinks);
 	}
 	
 	return TRUE;
-error:
+ error:
+	osync_list_free(objtypesinks);
 	return FALSE;
 }
 
-static osync_bool empty(Command *cmd, void *plugin_data, OSyncError **error)
+static osync_bool empty(Command *cmd, OSyncError **error)
 {
 	int i;
 	GList *c;
 	//const char *objtype = cmd->arg;
 	
 	/* Perform slowync - if objtype is set for this objtype, otherwise slowsync for ALL */
-	if (!get_changes(cmd, SYNCTYPE_FORCE_SLOWSYNC, plugin_data, error))
+	if (!get_changes(cmd, SYNCTYPE_FORCE_SLOWSYNC, error))
 		goto error;
 
 
@@ -881,25 +959,26 @@ static osync_bool empty(Command *cmd, void *plugin_data, OSyncError **error)
 		OSyncChange *change = c->data;
 		osync_change_set_changetype(change, OSYNC_CHANGE_TYPE_DELETED);
 
-		if (!commit(cmd, change, plugin_data, error))
+		if (!commit(cmd, change, error))
 			goto error;
 
 	}
 
 	return TRUE;
 
-error:
+ error:
 	return FALSE;
 }
 
 static void _osyncplugin_ctx_callback_syncdone(void *user_data, OSyncError *error)
 {
+	OSyncError *locerror = NULL;
+	Command *cmd = (Command *) user_data;		
+	OSyncObjTypeSink *sink = NULL;
+
 	assert(user_data);
 
-	OSyncError *locerror = NULL;
-
-	Command *cmd = (Command *) user_data;		
-	OSyncObjTypeSink *sink = cmd->sink;
+	sink = cmd->sink;
 
 	if (error) {
 		osync_error_set_from_error(&locerror, &error);
@@ -908,40 +987,48 @@ static void _osyncplugin_ctx_callback_syncdone(void *user_data, OSyncError *erro
 
 	g_mutex_lock(cmd->mutex);
 	g_cond_signal(cmd->cond);
+	cmd->done = TRUE;
 	g_mutex_unlock(cmd->mutex);
 
 	return;
-error:
+ error:
 	fprintf(stderr, "ERROR for sink \"%s\": %s\n", osync_objtype_sink_get_name(sink), osync_error_print(&locerror));
 	return;
 }
 
-static osync_bool syncdone_sink(OSyncObjTypeSink *sink, void *plugin_data, OSyncError **error) {
-
+static osync_bool syncdone_sink(Command *cmd, OSyncObjTypeSink *sink, OSyncError **error) {
+	OSyncContext *context = NULL;
 	assert(sink);
+	assert(cmd);
 
-	OSyncContext *context = osync_context_new(error);
+	context = osync_context_new(error);
 	if (!context)
 		goto error;
 
-	osync_context_set_callback(context, _osyncplugin_ctx_callback_syncdone, sink);
+	cmd->sink = sink;
+
+	osync_context_set_callback(context, _osyncplugin_ctx_callback_syncdone, cmd);
 
 	osync_plugin_info_set_sink(plugin_info, sink);
 
-	osync_objtype_sink_sync_done(sink, plugin_data, plugin_info, context);
+	osync_objtype_sink_sync_done(sink, plugin_info, context);
 
 	osync_context_unref(context);
 
+	if (!osync_objtype_sink_save_hashtable(sink, error))
+		goto error;
+
 	return TRUE;
 
-error:	
+ error:	
 	return FALSE;
 }
 
-static osync_bool syncdone(Command *cmd, void *plugin_data, OSyncError **error)
+static osync_bool syncdone(Command *cmd, OSyncError **error)
 {
 	
-	int i, num;
+	OSyncList *objtypesinks = NULL;
+	OSyncList *list;
 	OSyncObjTypeSink *sink = NULL;
 	const char *objtype = cmd->arg;
 
@@ -950,34 +1037,41 @@ static osync_bool syncdone(Command *cmd, void *plugin_data, OSyncError **error)
 		if (!sink)
 			goto error;
 
-		if (!syncdone_sink(sink, plugin_data, error))
+		if (!syncdone_sink(cmd, sink, error))
 			goto error;
 	} else {
-		num = osync_plugin_info_num_objtypes(plugin_info);
-		for (i=0; i < num; i++) {
-			sink = osync_plugin_info_nth_objtype(plugin_info, i);
+		objtypesinks = osync_plugin_info_get_objtype_sinks(plugin_info);
+		list = objtypesinks;
+		while(list) {
+			sink = (OSyncObjTypeSink*)list->data;
 
-			if (!syncdone_sink(sink, plugin_data, error))
+			if (!syncdone_sink(cmd, sink, error))
 				goto error;
+			
+			list = list->next;
 		}
 
 		/* last but not least - the main sink */
 		if (get_main_sink())
-			if (!syncdone_sink(get_main_sink(), plugin_data, error))
+			if (!syncdone_sink(cmd, get_main_sink(), error))
 				goto error;
+		
+		osync_list_free(objtypesinks);
 	}
 
 	return TRUE;
-error:
+ error:
+	osync_list_free(objtypesinks);
 	return FALSE;
 }
 
 static void _osyncplugin_ctx_callback_committedall(void *user_data, OSyncError *error)
 {
-	assert(user_data);
-
 	OSyncError *locerror = NULL;
 	Command *cmd = (Command *) user_data;		
+
+	assert(user_data);
+
 	//OSyncObjTypeSink *sink = cmd->sink;
 
 	if (error) {
@@ -987,39 +1081,44 @@ static void _osyncplugin_ctx_callback_committedall(void *user_data, OSyncError *
 
 	g_mutex_lock(cmd->mutex);
 	g_cond_signal(cmd->cond);
+	cmd->done = TRUE;
 	g_mutex_unlock(cmd->mutex);
 
 	return;
-error:
+ error:
 	fprintf(stderr, "ERROR: %s\n", osync_error_print(&locerror));
 	return;
 }
 
-static osync_bool committedall_sink(OSyncObjTypeSink *sink, void *plugin_data, OSyncError **error) {
-
+static osync_bool committedall_sink(Command *cmd, OSyncObjTypeSink *sink, OSyncError **error) {
+	OSyncContext *context = NULL;
 	assert(sink);
+	assert(cmd);
 
-	OSyncContext *context = osync_context_new(error);
+	context = osync_context_new(error);
 	if (!context)
 		goto error;
+	
+	cmd->sink = sink; 
 
-	osync_context_set_callback(context, _osyncplugin_ctx_callback_committedall, sink);
+	osync_context_set_callback(context, _osyncplugin_ctx_callback_committedall, cmd);
 
 	osync_plugin_info_set_sink(plugin_info, sink);
 
-	osync_objtype_sink_committed_all(sink, plugin_data, plugin_info, context);
+	osync_objtype_sink_committed_all(sink, plugin_info, context);
 
 	osync_context_unref(context);
 
 	return TRUE;
 
-error:	
+ error:	
 	return FALSE;
 }
 
-static osync_bool committedall(Command *cmd, void *plugin_data, OSyncError **error)
+static osync_bool committedall(Command *cmd, OSyncError **error)
 {
-	int i, num;
+	OSyncList *list;
+	OSyncList *objtypesinks = NULL;
 	OSyncObjTypeSink *sink = NULL;
 	const char *objtype = cmd->arg;
 
@@ -1028,25 +1127,31 @@ static osync_bool committedall(Command *cmd, void *plugin_data, OSyncError **err
 		if (!sink)
 			goto error;
 
-		if (!committedall_sink(sink, plugin_data, error))
+		if (!committedall_sink(cmd, sink, error))
 			goto error;
 	} else {
-		num = osync_plugin_info_num_objtypes(plugin_info);
-		for (i=0; i < num; i++) {
-			sink = osync_plugin_info_nth_objtype(plugin_info, i);
+		objtypesinks = osync_plugin_info_get_objtype_sinks(plugin_info);
+		list = objtypesinks;
+		while(list) {
+			sink = (OSyncObjTypeSink*)list->data;
 
-			if (!committedall_sink(sink, plugin_data, error))
+			if (!committedall_sink(cmd, sink, error))
 				goto error;
+			
+			list = list->next;
 		}
 
 		/* last but not least - the main sink */
 		if (get_main_sink())
-			if (!committedall_sink(get_main_sink(), plugin_data, error))
+			if (!committedall_sink(cmd, get_main_sink(), error))
 				goto error;
+		
+		osync_list_free(objtypesinks);
 	}
 
 	return TRUE;
-error:
+ error:
+	osync_list_free(objtypesinks);
 	return FALSE;
 }
 
@@ -1064,98 +1169,94 @@ static osync_bool run_command(Command *cmd, void **plugin_data, OSyncError **err
 
 
 	switch (cmd->cmd) {
-		case CMD_EMPTY:
-			if (!empty(cmd, *plugin_data, error))
-				goto error;
-			break;
-		case CMD_INITIALIZE:
-			if (!(*plugin_data = plugin_initialize(error)))
-				goto error;
-			break;
-		case CMD_FINALIZE:
-			finalize_plugin(plugin_data);
-			break;
-		case CMD_CONNECT:
-			if (!connect(cmd, *plugin_data, error))
-				goto error;
-			break;
-		case CMD_DISCONNECT:
-			if (!disconnect(cmd, *plugin_data, error))
-				goto error;
-			break;
-		case CMD_SLOWSYNC:
-			if (!get_changes(cmd, SYNCTYPE_FORCE_SLOWSYNC, *plugin_data, error))
-				goto error;
-			break;
-		case CMD_FASTSYNC:
-			if (!get_changes(cmd, SYNCTYPE_FORCE_FASTSYNC, *plugin_data, error))
-				goto error;
-			break;
-		case CMD_SYNC:
-			if (!get_changes(cmd, SYNCTYPE_NORMAL, *plugin_data, error))
-				goto error;
-			break;
-		case CMD_COMMIT:
-			fprintf(stderr, "COMMIT not yet implemented\n");
-			break;
-		case CMD_BATCHCOMMIT:
-			fprintf(stderr, "BATCHCOMMIT not yet implemented\n");
-			break;
-		case CMD_COMMITTEDALL:
-			if (!committedall(cmd, *plugin_data, error))
-				goto error;
-			break;
-		case CMD_READ:
-			fprintf(stderr, "READ not yet implemented\n");
-			break;
-		case CMD_WRITE:
-			fprintf(stderr, "WRITE not yet implemented\n");
-			break;
-		case CMD_SYNCDONE:
-			if (!syncdone(cmd, *plugin_data, error))
-				goto error;
-			break;
-		case CMD_DISCOVER:
-			fprintf(stderr, "DISCOVER not yet implemented\n");
-			break;
+	case CMD_EMPTY:
+		if (!empty(cmd, error))
+			goto error;
+		break;
+	case CMD_INITIALIZE:
+		if (!(*plugin_data = plugin_initialize(error)))
+			goto error;
+		break;
+	case CMD_FINALIZE:
+		finalize_plugin(plugin_data);
+		break;
+	case CMD_CONNECT:
+		if (!connect_plugin(cmd, error))
+			goto error;
+		break;
+	case CMD_DISCONNECT:
+		if (!disconnect(cmd, error))
+			goto error;
+		break;
+	case CMD_SLOWSYNC:
+		if (!get_changes(cmd, SYNCTYPE_FORCE_SLOWSYNC, error))
+			goto error;
+		break;
+	case CMD_FASTSYNC:
+		if (!get_changes(cmd, SYNCTYPE_FORCE_FASTSYNC, error))
+			goto error;
+		break;
+	case CMD_SYNC:
+		if (!get_changes(cmd, SYNCTYPE_NORMAL, error))
+			goto error;
+		break;
+	case CMD_COMMIT:
+		fprintf(stderr, "COMMIT not yet implemented\n");
+		break;
+	case CMD_COMMITTEDALL:
+		if (!committedall(cmd, error))
+			goto error;
+		break;
+	case CMD_READ:
+		fprintf(stderr, "READ not yet implemented\n");
+		break;
+	case CMD_WRITE:
+		fprintf(stderr, "WRITE not yet implemented\n");
+		break;
+	case CMD_SYNCDONE:
+		if (!syncdone(cmd, error))
+			goto error;
+		break;
+	case CMD_DISCOVER:
+		fprintf(stderr, "DISCOVER not yet implemented\n");
+		break;
 	}
 
 
 	printf("%s: %u - sink: %p\n", __func__, cmd->cmd, cmd->sink);
 	switch (cmd->cmd) {
-		case CMD_INITIALIZE:
-		case CMD_FINALIZE:
-		case CMD_DISCOVER:
-			break;
+	case CMD_INITIALIZE:
+	case CMD_FINALIZE:
+	case CMD_DISCOVER:
+		break;
 
-		case CMD_EMPTY:
-		case CMD_CONNECT:
-		case CMD_DISCONNECT:
-		case CMD_SLOWSYNC:
-		case CMD_FASTSYNC:
-		case CMD_SYNC:
-		case CMD_COMMIT:
-		case CMD_BATCHCOMMIT:
-		case CMD_COMMITTEDALL:
-		case CMD_READ:
-		case CMD_WRITE:
-		case CMD_SYNCDONE:
-			printf("waiting .....\n");
-			if (!cmd->done)
-				g_cond_wait(cmd->cond, cmd->mutex);
-			printf("DONE\n");
-			break;
+	case CMD_EMPTY:
+	case CMD_CONNECT:
+	case CMD_DISCONNECT:
+	case CMD_SLOWSYNC:
+	case CMD_FASTSYNC:
+	case CMD_SYNC:
+	case CMD_COMMIT:
+	case CMD_COMMITTEDALL:
+	case CMD_READ:
+	case CMD_WRITE:
+	case CMD_SYNCDONE:
+		printf("waiting .....\n");
+		if (!cmd->done)
+			g_cond_wait(cmd->cond, cmd->mutex);
+		printf("DONE\n");
+		break;
 	}
 
 	return TRUE;
 
-error:
+ error:
 	return FALSE;
 }
 
 static osync_bool plugin_list(OSyncError **error) {
-	int num;
-	int i;
+	OSyncList *plugins;
+	OSyncList *list;
 	
 	assert(!plugin_env);
 
@@ -1171,21 +1272,24 @@ static osync_bool plugin_list(OSyncError **error) {
 	if (!osync_plugin_env_load(plugin_env, pluginpath, error))
 		goto error_free_formatenv;
 
-	num= osync_plugin_env_num_plugins(plugin_env);
-	for(i= 0; i < num; i++) {
-		OSyncPlugin* plugin= osync_plugin_env_nth_plugin(plugin_env, i);
+	plugins = osync_plugin_env_get_plugins(plugin_env);
+	list = plugins;
+	while(list) {
+		OSyncPlugin* plugin = (OSyncPlugin*)list->data;
 		fprintf (stdout, "Name:        %s\n", osync_plugin_get_name(plugin));
-		fprintf (stdout, "Description: %s\n", osync_plugin_get_description(plugin));		
-	} 
+		fprintf (stdout, "Description: %s\n", osync_plugin_get_description(plugin));
+		list = list->next;
+	}
+	osync_list_free(plugins);
 	return TRUE;
 	
-error_free_formatenv:
-	osync_format_env_free(format_env);
+ error_free_formatenv:
+	osync_format_env_unref(format_env);
 	format_env = NULL;
-error_free_pluginenv:
-	osync_plugin_env_free(plugin_env);
+ error_free_pluginenv:
+	osync_plugin_env_unref(plugin_env);
 	plugin_env = NULL;
-error:	
+ error:	
 	return FALSE;
 }
 
@@ -1199,6 +1303,9 @@ int main(int argc, char **argv) {
 		g_thread_init(NULL);
 
 	parse_args(argc, argv);
+	/* Set defaults if not set on the command line */
+	if (!syncgroup) 
+		syncgroup = strdup("osyncplugin");
 	
 	if (pluginlist) {
 		if (!plugin_list(&error))
@@ -1215,9 +1322,9 @@ int main(int argc, char **argv) {
 			goto error_disconnect_and_finalize;
 
 
-success:
+ success:
 	if (plugin_env)
-		osync_plugin_env_free(plugin_env);
+		osync_plugin_env_unref(plugin_env);
 
 	for (o=cmdlist; o; o = o->next) {
 		Command *cmd = o->data;
@@ -1227,21 +1334,21 @@ success:
 
 	return EXIT_SUCCESS;
 
-error_disconnect_and_finalize:
+ error_disconnect_and_finalize:
 	if (plugin_data)
-		disconnect(NULL, plugin_data, NULL);
-//error_finalize:
+		disconnect(NULL, NULL);
+	//error_finalize:
 	finalize_plugin(&plugin_data);
-//error_free_plugin_env:
+	//error_free_plugin_env:
 	if (plugin_env)
-		osync_plugin_env_free(plugin_env);
+		osync_plugin_env_unref(plugin_env);
 
 	for (o=cmdlist; o; o = o->next) {
 		Command *cmd = o->data;
 		free_command(&cmd);
 	}
 
-error:	
+ error:	
 	fprintf(stderr, "Error: %s\n", osync_error_print(&error));
 	osync_error_unref(&error);
 	return EXIT_FAILURE;
